#========================================================================================
#   Require working connection to internet/google
#   See https://developers.google.com/maps/documentation/geocoding/?hl=en
#   Quick test:
#   http://maps.googleapis.com/maps/api/geocode/json?address=Heinrich-Heine-Strae+1+07749+Jena+Germany&sensor=false
#========================================================================================
use strict;
use Test::More;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp );
use DE_EPAGES::Address::API::GeoData qw ( GetGeoLocation GetGeoData FilterLocation);
use Net::Ping;

use constant GOOGLE_MAPS_API => 'maps.googleapis.com';

sub IsGoogleaAPIReachable {
    my $p = Net::Ping->new('icmp');
    my $IsReachable = $p->ping( GOOGLE_MAPS_API );
    $p->close();

    return $IsReachable ? 1 : 0
}

sub TestGetGeoLocationJena {

    my $hAddr = {
        Street  => 'Heinrich-Heine-Strae 1',
        Zipcode => '07749',
        City => 'Jena',
        Country => 'Germany',
    };

    my $hLocation = GetGeoLocation($hAddr);
    my $gotZeroCoord = _Compare($hLocation, {
            'lat' => '0.000000',
            'lng' => '0.000000'
    } );

    SKIP: {
        skip( 'The Google Maps Geocoding API has blocked our request.', 1 ) if $gotZeroCoord;
        ok(_Compare($hLocation, {
                'lat' => '50.934451',
                'lng' => '11.598713'
        } ), 'buro in Jena');
    }
}

sub TestGetGeoLocationZero {

    my $hAddr = {
        Street  => 'aaa 66',
        Zipcode => '666',
        City => 'aaa',
        Country => 'Argentinia',
    };

    my $hLocation = GetGeoLocation($hAddr);
    is($hLocation, undef, 'invalid address -> empty result -> return undef');
}


sub TestGetGeoDataLondon {

    my $hAddr = {
        Street  => 'Linen Hall 162- 168 Regent Street',
        City => 'London, W1B 5TF',
        Country => 'Great Britain',
    };
    my $hLocation = GetGeoData($hAddr, \&FilterLocation);
    my $gotZeroCoord = _Compare($hLocation, {
            'lat' => '0.000000',
            'lng' => '0.000000'
    } );

    SKIP: {
        skip( 'The Google Maps Geocoding API has blocked our request.', 1 ) if $gotZeroCoord;
        ok(_Compare($hLocation, {
            'lat' => '51.512043',
            'lng' => '-0.139324'
        }, 2), 'buro in London (with digit precision 2)');
    }
}

sub TestGetGeoDataBarcelona {
     my $hAddr = {
        Street  => 'Diagonal, 435 5-1',
        City => '08036 Barcelona',
        Country => 'Spain',
    };
    my $hLocation = GetGeoData($hAddr, \&FilterLocation);
    is(ref ($hLocation), 'HASH' ,'use GeoData directly with UTF-8 chars' );
    my $gotZeroCoord = _Compare($hLocation, {
            'lat' => '0.000000',
            'lng' => '0.000000'
    } );

    SKIP: {
        skip( 'The Google Maps Geocoding API has blocked our request.', 1 ) if $gotZeroCoord;
        ok(_Compare($hLocation, {
            'lat' => '41.393969',
            'lng' => '2.150830'
        } ), 'buro in Barcelona');
    }
}

sub _Compare($$;$) {
    my ($hGot, $hExpected, $precision) = @_;
    $precision = 4 unless defined $precision;
    if (fcmp($hGot->{lat}, $hExpected->{lat}, $precision) == 0 and
    fcmp($hGot->{lng}, $hExpected->{lng}, $precision) == 0 ) {
        return 1
    }
    elsif ( $hExpected->{lat} == 0.000000 and $hExpected->{lng} == 0.000000 ) {
        return 0;
    }
    else {
        diag( sprintf("comparing cordinates\nlatitude got: %f expected: %f\nlangitude got: %f expected: %f",
        $hGot->{lat}, $hExpected->{lat}, $hGot->{lng}, $hExpected->{lng}  )  );
        return 0;
    }
}

#========================================================================================
#   MAIN
#========================================================================================

RunScript ( Sub => sub {
    SKIP : {
        if( $ENV{SKIP_IN_PR_TESTS} ) {
                plan skip_all => "Skipping it because SKIP_IN_PR_TESTS == 1";
            }
            else {
                skip( GOOGLE_MAPS_API . ' is not reachable. Check your internet connection.', 6) unless IsGoogleaAPIReachable();
                TestGetGeoLocationJena();
                TestGetGeoLocationZero();
                TestGetGeoDataLondon();
                TestGetGeoDataBarcelona();
                done_testing();
            }
    }
} );
