#========================================================================================
# package      DE_EPAGES::ConfianzaOnline::UI::Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::ConfianzaOnline::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::ConfianzaOnline::UI::Shop;
use DE_EPAGES::ConfianzaOnline::API::ConfianzaRequest;
use Test::More;
use Test::Exception;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object HashRef String );
use Test::Mockify::ReturnValue;
use Test::Mockify::Verify qw ( WasCalled );

use feature 'switch';

use strict;

sub test {
    my $self = shift;
    $self->_test_validFormValuesAndSuccessRequest();
    $self->_test_validFormValuesAndFailedRequest();
    $self->_test_emptyFormValues();
    $self->_test_undefinedFormValues();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_validFormValuesAndSuccessRequest {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Servlet = $self->_createServlet();

    my $hCheckedValues = {
        'Values' => {
            'ConfianzaOnlineCode' => 'validCode'
        }
    };

    my $ConfianzaCode = $hCheckedValues->{'Values'}->{'ConfianzaOnlineCode'};

    my $ConfianzaRequest = $self->_createConfianzaRequest($ConfianzaCode);

    my $SUT = $self->_createSUT($hCheckedValues, $ConfianzaRequest);

    my $FakeObject = "FakeObject";
    my $FakeClass = "FakeClass";

    lives_ok(
        sub { $SUT->attributeValues($Servlet, $FakeObject, $FakeClass);},
        "$SubTestName - test if correct Form and Request lives"
    );

    ok(WasCalled($ConfianzaRequest, 'isValidConfianzaCode'), "$SubTestName - ConfianzaRequest isValidConfianzaCode was called");

    return;
}

sub _test_validFormValuesAndFailedRequest {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Servlet = $self->_createServlet();

    my $hCheckedValues = {
        'Values' => {
            'ConfianzaOnlineCode' => 'invalidCode'
        }
    };

    my $ConfianzaCode = $hCheckedValues->{'Values'}->{'ConfianzaOnlineCode'};

    my $ConfianzaRequest = $self->_createConfianzaRequest($ConfianzaCode);

    my $SUT = $self->_createSUT($hCheckedValues, $ConfianzaRequest);

    my $FakeObject = "FakeObject";
    my $FakeClass = "FakeClass";

    throws_ok(
        sub { $SUT->attributeValues($Servlet, $FakeObject, $FakeClass);},
        qr/ConfianzaOnlineCode/,
        "$SubTestName - test if correct Form and failed Request throws Error ConfianzaOnlineCode"
    );

    ok(WasCalled($ConfianzaRequest, 'isValidConfianzaCode'), "$SubTestName - ConfianzaRequest isValidConfianzaCode was called");

    return;
}

sub _test_emptyFormValues {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Servlet = $self->_createServlet();

    my $hCheckedValues = {
        'Values' => {
            'ConfianzaOnlineCode' => ''
        }
    };

    my $ConfianzaCode = $hCheckedValues->{'Values'}->{'ConfianzaOnlineCode'};

    my $ConfianzaRequest = $self->_createConfianzaRequest($ConfianzaCode);

    my $SUT = $self->_createSUT($hCheckedValues, $ConfianzaRequest);

    my $FakeObject = "FakeObject";
    my $FakeClass = "FakeClass";

    lives_ok(
        sub { $SUT->attributeValues($Servlet, $FakeObject, $FakeClass);},
        "$SubTestName - test if empty Form lives"
    );

    ok(!WasCalled($ConfianzaRequest, 'isValidConfianzaCode'), "$SubTestName - ConfianzaRequest isValidConfianzaCode was not called");

    return;
}

sub _test_undefinedFormValues {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Servlet = $self->_createServlet();

    my $hCheckedValues = {
        'Values' => {}
    };

    my $ConfianzaCode = $hCheckedValues->{'Values'}->{'ConfianzaOnlineCode'};

    my $ConfianzaRequest = $self->_createConfianzaRequest($ConfianzaCode);

    my $SUT = $self->_createSUT($hCheckedValues, $ConfianzaRequest);

    my $FakeObject = "FakeObject";
    my $FakeClass = "FakeClass";

    lives_ok(
        sub { $SUT->attributeValues($Servlet, $FakeObject, $FakeClass);},
        "$SubTestName - test if undefined Form lives"
    );

    ok(!WasCalled($ConfianzaRequest, 'isValidConfianzaCode'), "$SubTestName - ConfianzaRequest isValidConfianzaCode was not called");

    return;
}

sub _createSUT {
    my $self = shift;
    my ($hCheckedValues, $ConfianzaRequest) = @_;

    my $aParameterList = [];

    my $MockedSUT = Test::Mockify->new(
        'DE_EPAGES::ConfianzaOnline::UI::Shop',
        $aParameterList
    );

    $MockedSUT->mock('_checkSuperFormValues')->when(Object('DE_EPAGES::Presentation::Servlet::Permission'), String("FakeObject"), String("FakeClass"))->thenReturn($hCheckedValues);

    $MockedSUT->mock('_createConfianzaRequest')->when()->thenReturn($ConfianzaRequest);

    return $MockedSUT->getMockObject();
}

sub _createServlet {
    my $self = shift;

    my $Shop = $self->_createShop();

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $self->_createForm());

    return $MockifyServlet->getMockObject();
}

sub _createForm {
    my $self = shift;

    my $MockedForm = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');

    my $ExecuteFormErrorSub = sub {
        my ($hMessages) = @_;
        Error($hMessages->{'Reason'});
    };

    my $hFormError = {
        'Reason'=> 'ConfianzaOnlineCode',
        'Name'  => 'ConfianzaOnlineCode'
    };

    $MockedForm->mock('addFormError')->when(HashRef($hFormError))->thenCall($ExecuteFormErrorSub);

    return $MockedForm->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {};

    my $MockedShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    return $MockedShop->getMockObject();
}

sub _createConfianzaRequest {
    my $self = shift;
    my ($ConfianzaCode) = @_;

    my $aAttributes = [];

    my $MockedConfianzaRequest = Test::Mockify->new('DE_EPAGES::ConfianzaOnline::API::ConfianzaRequest', $aAttributes);

    $MockedConfianzaRequest->mock('isValidConfianzaCode')->when(String('invalidCode'))->thenReturnUndef();
    $MockedConfianzaRequest->mock('isValidConfianzaCode')->when(String('validCode'))->thenReturn(1);

    return $MockedConfianzaRequest->getMockObject();
}

__PACKAGE__->runScript();

1;
