#==============================================================================
# package      DE_EPAGES::Core::API::Net
# state        public
#----------------------------------------------------------------------------------------
# description  Functions for TCP/IP networking.
#==============================================================================
package DE_EPAGES::Core::API::Net;
use base qw( Exporter );

use strict;
use Config;
use File::Basename  qw ( dirname );
use HTTP::Request;
use HTTP::Status qw(RC_OK);
use IO::Socket::INET6;
use LWP::Simple qw(head);
use LWP::UserAgent;
use Net::IPv6Addr;

use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::File qw ( CreateDirectory SetAccessRights );
use DE_EPAGES::Core::API::PerlTools qw ( Memoize );
use DE_EPAGES::Core::API::IniConfig;

our @EXPORT_OK = qw(
    Ip2Int
    Int2Ip
    GetHttpResponse
    GetHttpContent
    localhostName
    SupportedIpVersions
    GetFile
    GetFastestEpagesMirror
    ExistsURLFile
    GetContentLength
);

sub ErrorDefinition {
    return {
        'InvalidIpAddress' => 'invalid IP address: #IP',
        'InvalidValue' => 'invalid value: #Value',
        'GET_FASTEST_MIRROR_FAILED' => 'can\'t get mirror URL for file: #File',
    };
}

#========================================================================================
# function     Ip2Int
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Int = Ip2Int($IP);
#----------------------------------------------------------------------------------------
# description  Converts an ip to an integer.
#----------------------------------------------------------------------------------------
# input        $IP | ip string | string
# return       $Int | integer representation of the ip | integer
#========================================================================================
sub Ip2Int {
    my ($IP) = @_;

    if ($IP =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/x) {
        my @InvalidAddressparts = grep { $_ > 255 } split(/\./, $IP);  ##no critic (ProhibitMagicNumbers)
        Error('InvalidIpAddress', { 'IP' => $IP }) if @InvalidAddressparts;
        return ($1 << 24) + ($2 << 16) + ($3 << 8) + $4; ##no critic (ProhibitCaptureWithoutTest ProhibitMagicNumbers)
    } else {
        my $IpObj = Net::IPv6Addr->new($IP);
        return $IpObj->to_bigint();
    }
    return;
}

#========================================================================================
# function     Int2Ip
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IP = Int2Ip($IP);
#----------------------------------------------------------------------------------------
# description  Converts an integer to an IP.
#----------------------------------------------------------------------------------------
# input        $Int | integer representation of the ip | integer
# return       $IP | ip string | string
#========================================================================================
sub Int2Ip {
    my ($Int) = @_;

    $Int =~ /^[0-9]+$/ or Error('InvalidValue', { 'Value' => "$Int !~ /^[0-9]+$/" });
    $Int < 2 ** 32 or Error('InvalidValue', { 'Value' => "$Int >= 2 ** 32" });  ##no critic (ProhibitMagicNumbers)
    my $IP = '';
    foreach my $count (qw(1 2 3)) {
        $IP = '.' . ($Int % 2 ** 8) . $IP;
        $Int = $Int >> 8;
    }
    return "$Int$IP";
}

#========================================================================================
# function     GetHttpResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Response = GetHttpResponse($Url);
# syntax       $Response = GetHttpResponse($Url, $Timeout);
# syntax       $Response = GetHttpResponse($Url, $Timeout, $UserAgentString);
#----------------------------------------------------------------------------------------
# description  Loads a document from internet via HTTP "GET" request.
#----------------------------------------------------------------------------------------
# input        $Url | location of document | string
# input        $Timeout | (optional) timeout in seconds | integer
# input        $UserAgentString | (optional) UserAgent header used in request | string
# return       $Response | http response object | HTTP::Response
#========================================================================================
sub GetHttpResponse {
    my ($Url, $timeout, $UserAgentString) = @_;

    my $Request = HTTP::Request->new(GET => $Url);
    my $UserAgent = LWP::UserAgent->new();
    $UserAgent->agent($UserAgentString) if $UserAgentString;
    $UserAgent->timeout($timeout) if defined $timeout;
    return $UserAgent->request($Request);
}

#========================================================================================
# function     GetHttpContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Content = GetHttpContent($Url);
#----------------------------------------------------------------------------------------
# description  Load document from internet.
#----------------------------------------------------------------------------------------
# input        $Url | location of document | string
# return       $Content | http response content (undef if http-return-code is not 200) | String
#========================================================================================
sub GetHttpContent {
    my ($Url) = @_;

    my $Response = GetHttpResponse($Url);
    return undef if $Response->code != RC_OK;
    return $Response->content;
}

#========================================================================================
# function     localhostName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $localhost = localhostName();
#----------------------------------------------------------------------------------------
# description  Returns the name of localhost:
#               - ipv6-localhost: if UNIX and IPv6 works
#               - localhost: otherwise
#----------------------------------------------------------------------------------------
# return       $localhost | localhost name | string
#========================================================================================
sub localhostName { return _localhostName(); }

*_localhostName = Memoize( sub {

    return 'localhost' if $Config{'osname'} eq 'MSWin32';

    if (my $Socket = IO::Socket::INET6->new(Listen => 1, LocalAddr => 'ipv6-localhost', Proto => 'tcp')) {
        $Socket->close or die "Can't close socket: $!";
        return 'ipv6-localhost';
    }

    return 'localhost';
});

#========================================================================================
# function     SupportedIpVersions
# state        public
#----------------------------------------------------------------------------------------
# syntax       @IpVersions = SupportedIpVersions();
#----------------------------------------------------------------------------------------
# description  Returns an array of address families containing one/more of:
#               6 (for IPv6), 4 (for IPv4)
#----------------------------------------------------------------------------------------
# return       @IpVersions | supported address families | array
#========================================================================================
sub SupportedIpVersions { return @{ _SupportedIpVersions() }; }

*_SupportedIpVersions = Memoize( sub {

    my @IpVersions;
    if (my $Socket = IO::Socket::INET->new( Listen => 1, LocalAddr => '127.0.0.1')) {
        $Socket->close or die "Can't close socket: $!";
        push @IpVersions, 4;
    }
    if (my $Socket = IO::Socket::INET6->new( Listen => 1, LocalAddr => '::1')) {
        $Socket->close or die "Can't close socket: $!";
        push @IpVersions, 6;
    }
    return \@IpVersions;
});

#========================================================================================
# function     GetFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ResponseCode = GetFile($Url, $FileName);
# syntax       my $ResponseCode = GetFile($Url, $FileName, $Timeout);
#----------------------------------------------------------------------------------------
# description  Loads a file from internet via HTTP "GET" request and saves it to filename.
#----------------------------------------------------------------------------------------
# input        $Url | location of file | string
# input        $Filename | name of file | string
# input        $timeout | (optional) timeout in seconds | integer
# return       $ResponseCode | if all is right 0, else http response code | string
#========================================================================================
sub GetFile {
    my ($Url, $FileName, $timeout) = @_;

    my $Request = HTTP::Request->new(GET => $Url);
    my $UserAgent = LWP::UserAgent->new;
    $UserAgent->timeout($timeout) if defined $timeout;

    # create Directory
    my $DirName = dirname ($FileName);
    CreateDirectory( $DirName, {'recursive'=>1});

    # get large files with direct writing to disk (otherwise it will take ~200 times longer to download the file)
    # see http://www.perl.com/lpt/a/661 (Getting Large Documents)
    # or see http://search.cpan.org/~gaas/libwww-perl-5.836/lib/LWP/UserAgent.pm (:content_file)
    my $Response = $UserAgent->request($Request,$FileName);
    SetAccessRights($FileName);

    return $Response->code == RC_OK ? 0 : $Response->code;
}

#========================================================================================
# function     GetContentLength
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hInfoContent = GetContentLength($Url,  $Timeout);
#----------------------------------------------------------------------------------------
# description  gets the content length via HTTP "HEAD" request.
#----------------------------------------------------------------------------------------
# input        $Url | location of file | string
# input        $timeout | (optional) timeout in seconds, default 10 | integer
# return       $hResult | Result with contentlength and response code
#               <ul>
#                   <li>ContentLength  - content length - string
#                   <li>ResponseCode - file name - string
#               </ul> | ref.hash
#========================================================================================
sub GetContentLength {
    my ($Url, $timeout) = @_;

    $timeout = 10 unless defined $timeout;

    # get file size with HTTP HEAD
    # in case of an error, use undef as file size
    my $UserAgent = LWP::UserAgent->new(
        timeout => $timeout,
    );

    my $Request = HTTP::Request->new('HEAD', $Url);
    my $Response = $UserAgent->request( $Request );
    my $ContentLength = $Response->content_length;

    return {'ContentLength' => $ContentLength, 'ResponseCode' => $Response->code };

}

#========================================================================================
# function     ExistsURLFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Exists = ExistsURLFile($Url, $FileName);
#----------------------------------------------------------------------------------------
# description  check if file exists
#----------------------------------------------------------------------------------------
# input        $Url | location of file | string
# return       $FileExists | true if file exists | bool
#========================================================================================
sub ExistsURLFile{
    my ($URL) = @_;

    my $UserAgent = LWP::UserAgent->new(max_redirect => 0);

    my $Response = $UserAgent->head($URL);

    #next if test request had no success
    return $Response->is_success;
}

#========================================================================================
# function     GetFastestEpagesMirror
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = GetFastestEpagesMirror($RelativePath);
# example      my $Url = GetFastestEpagesMirror('patch/6.14.3/epages-6.14.3-patch-data.tar.gz');
#               returns: http://epages.com/RPMS/patch/6.14.3/epages-6.14.3-patch-data.tar.gz
#----------------------------------------------------------------------------------------
# description  Returns the fastest URL to $RelativePath from a list of epages mirrors, e.g.:
#               http://epages.com/RPMS/$RelativePath.
#               If envvar EPAGES_PATCH_DATA_REPO is set, prefer that mirror (space-separated
#               URL or absolute path). If envvar EPAGES_REPO is set, prefer that mirror
#               (rather than EPAGES_PATCH_DATA_REPO).
#               Example:
#                 EPAGES_PATCH_DATA_REPO=/my/repos; EPAGES_REPO=http://epages.com/RPMS
#                 Result: http://epages.com/RPMS is used
#               Example:
#                 EPAGES_PATCH_DATA_REPO=/my/repos; EPAGES_REPO=
#                 Result: /my/repos is used
#               Otherwise, find mirrors in default places.
#----------------------------------------------------------------------------------------
# input        $RelativePath | find fastest mirror for file  | string
# return       $Url          | fastest mirror to file | string
#========================================================================================
sub GetFastestEpagesMirror ($) {
    my ($RelativePath) = @_;

    # repo priority is: fromEnv > inDev > yum > released
    my @RepoPriority = qw(fromEnv inDev yum released);

    my $hRepositories = _epages_mirrors();

    my %Speed;
    foreach my $Prio (@RepoPriority) {
        foreach my $UrlPrefix (@{$hRepositories->{$Prio}}) {
            my $Url = "$UrlPrefix/$RelativePath";
            if (-f $Url) {
                $Speed{$Url} = 0;
                next;
            }
            # we must use max_redirect => 0 because http pages may redirect
            # to an error page with RC == 200 if original page does not exist
            my $UserAgent = LWP::UserAgent->new(max_redirect => 0);
            my $time1 = time;
            my $Response = $UserAgent->head($Url);
            next unless $Response->is_success;
            $Speed{$Url} = time - $time1;
        }
        # stop if repository with specific priority found
        last if scalar keys %Speed;
    }

    Error('GET_FASTEST_MIRROR_FAILED', {File => $RelativePath})
        unless scalar keys %Speed;

    my $Fastest = (sort { $Speed{$a} <=> $Speed{$b} } keys %Speed)[0];
    return $Fastest;
}

#========================================================================================
# function     _epages_mirrors
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Repositories = _epages_mirrors();
#----------------------------------------------------------------------------------------
# description  Returns lists of epages repositories with priority.
#               If envvar EPAGES_PATCH_DATA_REPO is set, prefer that mirror (space-separated
#               URL or absolute path). If envvar EPAGES_REPO is set, prefer that mirror
#               (rather than EPAGES_PATCH_DATA_REPO).
#               Example:
#                 EPAGES_PATCH_DATA_REPO=/my/repos; EPAGES_REPO=http://epages.com/RPMS
#                 Result: http://epages.com/RPMS is used
#               Example:
#                 EPAGES_PATCH_DATA_REPO=/my/repos; EPAGES_REPO=
#                 Result: /my/repos is used
#               Otherwise, find mirrors in default places.
#----------------------------------------------------------------------------------------
# return       $Repositories | hash of repository arrays, key is priority | ref. hash
#========================================================================================
sub _epages_mirrors {

    # repo priority is fromEnv > inDev > yum > released

    my %Repositories = (
        inDev => [ 'http://ftp.epages.com/pub/epages/build/repo/epages',
        ],
        released => [ 'http://epages-software.de/repo/epages',
        ],
        );

    # repo from environment variables EPAGES_REPO / EPAGES_PATCH_DATA_REPO

    if ($ENV{EPAGES_REPO}) {
        $Repositories{fromEnv} = [split(/\s+/,$ENV{EPAGES_REPO})];
    } elsif ($ENV{EPAGES_PATCH_DATA_REPO}) {
        $Repositories{fromEnv} = [split(/\s+/,$ENV{EPAGES_PATCH_DATA_REPO})];
    }

    # repo from /etc/yum.repos.d/epages-arch.repo

    my $RepoDir = '/etc/yum.repos.d/';
    my $Repo = -f "$RepoDir/epages-arch.repo" ? 'epages-arch' : 'epages-kernel';

    if (-f "$RepoDir/$Repo.repo") {
        my $IniConfig = DE_EPAGES::Core::API::IniConfig->new(FileName => "$RepoDir/$Repo.repo",
                                                             CaseSensitive => 1);
        my $UrlPrefix;
        if ($UrlPrefix = $IniConfig->get($Repo, 'baseurl')) {
            $UrlPrefix =~ s,/kernel/.*,,;
            $UrlPrefix =~ s,^file://,,;
            $Repositories{yum} = [$UrlPrefix];
        }
    }
    return \%Repositories;
}

1;
