#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::AddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  do DHLIntraship related Address validation
#----------------------------------------------------------------------------------------
# example      my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $AddressObject )
#               if ($AddressValidation->hasWarningStates()) { DoSomething() };
#========================================================================================
package DE_EPAGES::DHLIntraship::API::AddressValidation;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::DHLIntraship::API::Constants qw (
    %ADDRESS_LENGTH_INTERVALS
);

use feature qw( switch );

use strict;

use constant MANDATORY_WARNING_STRING => 'is mandatory but not set';
use constant TOO_SHORT_WARNING_STRING => 'is too Short';
use constant TOO_LONG_WARNING_STRING => 'is too Long';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Address, $ExtraAttributDefinitions );
#----------------------------------------------------------------------------------------
# description  constructor of AddressValidation
#----------------------------------------------------------------------------------------
# input        $Address | ePages AddressObject | object
# input        $ExtraAttributDefinitions | extra Attributes definitions| hash_ref
# return       $self | AddressValidation object | object
#========================================================================================
sub new {
    my $class = shift;
    my ( $Address, $ExtraAttributDefinitions) = @_;
    TestObject('AddressValidation needs valid address object',$Address,'DE_EPAGES::Address::API::Object::Address');
    my $self  = bless {
                'Address' => $Address,
                'WarningStates' => [],
    } , $class;
    $self->_setAttributeDefinitions($ExtraAttributDefinitions);
    return $self;
}

#========================================================================================
# function     _setAttributeDefinitions
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setAttributeDefinitions( $ExtraAttributDefinitions );
#----------------------------------------------------------------------------------------
# description  defines the requirements for DHLIntraship Address attributes,
#               the $ExtraAttributDefinitions overrides and/or extends the defaults
#----------------------------------------------------------------------------------------
# input        $ExtraAttributDefinitions | extra definitions | ref_hash
#========================================================================================
sub _setAttributeDefinitions {
    my $self = shift;
    my ($ExtraAttributDefinitions) = @_;
    $self->{'AttributeNames'} = {
        'FirstName' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'max'}
        },
        'LastName' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'max'}
        },
        'FullName' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'name1'}->{'max'}
        },
        'City' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'city'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'city'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'city'}->{'max'}
        },
        'StreetOnly' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'streetName'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'streetName'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'streetName'}->{'max'}
        },
        'Company' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'name2'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'name2'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'name2'}->{'max'}
        },
        'Street2' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'addressAddition'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'addressAddition'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'addressAddition'}->{'max'}
        },
        'HouseNumber' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'streetNumber'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'streetNumber'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'streetNumber'}->{'max'}
        },
        'Zipcode' => {
            'DE' => {
                'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'zip'}->{'required'},
                'MinValue'=> 5,
                'MaxValue'=> 5},
            'EN' => {
                'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'zip'}->{'required'},
                'MinValue'=> 8,
                'MaxValue'=> 8
            },
            'Other' => {
                'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'zip'}->{'required'},
                'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'zip'}->{'min'},
                'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'zip'}->{'max'}
            },
        },
        'Phone' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'phone'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'phone'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'phone'}->{'max'}
        },
        'EMail' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'email'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'email'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'email'}->{'max'}
        },
        'Phone' => {
            'Mandatory'=> $ADDRESS_LENGTH_INTERVALS{'phone'}->{'required'},
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'phone'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'phone'}->{'max'}
        },
        'IsPickupAddress' => {'Mandatory'=> 0, 'MinValue'=> 1, 'MaxValue'=>1},
        'DHLPackstationId' => {
            'Mandatory'=> 0,
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'packstationNumber'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'packstationNumber'}->{'max'}
        },
        'DHLPostNumber' => {
            'Mandatory'=> 0,
            'MinValue'=> $ADDRESS_LENGTH_INTERVALS{'postNumber'}->{'min'},
            'MaxValue'=> $ADDRESS_LENGTH_INTERVALS{'postNumber'}->{'max'}
        }
    };
    if ( isValid($ExtraAttributDefinitions) ){
        $self->{'AttributeNames'} = {%{$self->{'AttributeNames'}}, %{$ExtraAttributDefinitions}};
    }
    return;
}

#========================================================================================
# function     hasWarningStates
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ValidationResult = $self->hasWarningStates( );
#----------------------------------------------------------------------------------------
# description  generates an internal list of warnings for not met mandatory as well as length
#               conditions and returns a boolean specifying if all conditions are met or not.
#----------------------------------------------------------------------------------------
# return       $ValidationResult | if all Attributes are fine | boolean
#========================================================================================
sub hasWarningStates{
    my $self = shift;
    $self->loadAttributes();
    return 1 if (scalar @{$self->{'WarningStates'}} > 0);
    return;
}

#========================================================================================
# function     loadAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       loadAttributes( );
#----------------------------------------------------------------------------------------
# description  load all Attributes to the class object
#========================================================================================
sub loadAttributes {
    my $self = shift;
    $self->_loadLanguageDependentAttributes();
    foreach my $AttributeName ( keys %{$self->{'AttributeNames'}}){
    my $IsMandatory = $self->{'AttributeNames'}->{$AttributeName}->{'Mandatory'};
        if ($IsMandatory){
            $self->_loadMandatoryAttribute($AttributeName);
        }else{
            $self->_getAttribute($AttributeName);
        }
    }
    return;
}

#========================================================================================
# function     getNotProcessableWarningStates
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $NotProcessableWarningStates = $self->getNotProcessableWarningStates( );
#----------------------------------------------------------------------------------------
# description  generates a list of warnings for DHLVersenden web service not processable
#               attributes (mandatory as well as length conditions for specific attributes).
#----------------------------------------------------------------------------------------
# return       $NotProcessableWarningStates | list of not processable states | arrayref
#========================================================================================
sub getNotProcessableWarningStates {
    my $self = shift;

    $self->loadAttributes();
    my @NotProcessableWarningStates = ();

    foreach my $WarningState (@{$self->{'WarningStates'}}) {
        push(@NotProcessableWarningStates, $WarningState) if ($WarningState =~ /@{[MANDATORY_WARNING_STRING]}/);
        push(@NotProcessableWarningStates, $WarningState) if ($WarningState =~ /Zipcode is too/);
        push(@NotProcessableWarningStates, $WarningState) if ($WarningState =~ /HouseNumber is too/);
    }

    return \@NotProcessableWarningStates;
}

#========================================================================================
# function     _loadLanguageDependentAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadLanguageDependentAttributes( $Input1 );
#----------------------------------------------------------------------------------------
# description  encapsulate the language dependent Attribute calls since they have a specific order
#========================================================================================
sub _loadLanguageDependentAttributes {
    my $self = shift;
    $self->_loadLanguageCode2();
    $self->_initZipCode();
    return;
}

#========================================================================================
# function     _getAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getAttribute( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Gets the Attribute with the Name $AttributeName from Shippinglabel->Shopaddress
#               and write it to the Class object
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
#========================================================================================
sub _getAttribute {
    my $self = shift;
    my ($AttributeName) = @_;
    $self->{$AttributeName} = $self->{'Address'}->get($AttributeName);
    $self->_validateAttributeLenght($AttributeName);
    return;
}

#========================================================================================
# function     _validateAttributeLenght
# state        private
#----------------------------------------------------------------------------------------
# syntax       _validateAttributeLenght( $AttributeName );
#----------------------------------------------------------------------------------------
# description  validate the Attribute lenght if wrong it creates warnings
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
#========================================================================================
sub _validateAttributeLenght {
    my $self = shift;
    my ($AttributeName) = @_;
    if ( isValid($self->{$AttributeName}) ){
        eval {
            my $MinValue = $self->{'AttributeNames'}->{$AttributeName}->{'MinValue'};
            my $MaxValue = $self->{'AttributeNames'}->{$AttributeName}->{'MaxValue'};
            TestString($AttributeName,$self->{$AttributeName},$MinValue,$MaxValue);
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ( $Error->code eq 'STRING_TOO_SHORT' ){
                $self->_setWarning($AttributeName, ' ' . TOO_SHORT_WARNING_STRING);
            } elsif ($Error->code eq 'STRING_TOO_LONG'){
                $self->_setWarning($AttributeName, ' ' . TOO_LONG_WARNING_STRING);
            }else {
                Error($Error);
            }
        }
    }
    return;
}

#========================================================================================
# function     _loadMandatoryAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadMandatoryAttribute( $AttributeName );
#----------------------------------------------------------------------------------------
# description  wrappes the _getAttribute method for mandatory check
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
#========================================================================================
sub _loadMandatoryAttribute {
    my $self = shift;
    my ( $AttributeName ) = @_;
    $self->_getAttribute( $AttributeName );
    if ( not isValid( $self->{$AttributeName} ) ) {
        $self->_setWarning($AttributeName);
    }
    return;
}

#========================================================================================
# function     _initZipCode
# state        private
#----------------------------------------------------------------------------------------
# syntax       _initZipCode( );
#----------------------------------------------------------------------------------------
# description  set the correct ZipCode Definition related to the shop address
#========================================================================================
sub _initZipCode {
    my $self = shift;
    my $ZipCodeDefinition  = {};
    given ($self->{'Code2'}) {
        when ('DE') {
            $ZipCodeDefinition = $self->{'AttributeNames'}{'Zipcode'}{'DE'};
        }
        when ('EN') {
            $ZipCodeDefinition = $self->{'AttributeNames'}{'Zipcode'}{'EN'};
        }
        default {
            $ZipCodeDefinition = $self->{'AttributeNames'}{'Zipcode'}{'Other'};
        }
    }
    $self->{'AttributeNames'}{'Zipcode'} = $ZipCodeDefinition;
    return;
}

#========================================================================================
# function     _loadLanguageCode2
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadLanguageCode2( $Input1 );
#----------------------------------------------------------------------------------------
# description  load the country code2
#========================================================================================
sub _loadLanguageCode2 {
    my $self = shift;
    $self->{'Country'} = $self->{'Address'}->get('Country');
    if ( isValid( $self->{'Country'} ) ){
        $self->{'Code2'} = $self->{'Country'}->{'Code2'};
        $self->_setWarning('Code2') unless isValid( $self->{'Code2'})
    }else{
        $self->_setWarning('Country');
    }
    return;
}

#========================================================================================
# function     _setWarning
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setWarning( $AttributeName );
#----------------------------------------------------------------------------------------
# description  writes a Attribute name to the classvar $self->{'WarningStates'} to show
#               which mandatory attribute is set
#----------------------------------------------------------------------------------------
# input        $AttributeName | Name of Attribute | String
# input        $Reason        | reason for warning | String
#========================================================================================
sub _setWarning{
    my $self = shift;
    my ($AttributeName, $Reason) = @_;

    $Reason = ' ' . MANDATORY_WARNING_STRING if ( not isValid($Reason));
    push( @{$self->{'WarningStates'}}, $AttributeName . $Reason );
    return ;
}

#========================================================================================
# function     getStatesAsWarning
# state        public
#----------------------------------------------------------------------------------------
# syntax       getStatesAsWarning( $Input1 );
#----------------------------------------------------------------------------------------
# description  Get the States and create epages warnings from this
#----------------------------------------------------------------------------------------
# input        $aWarningsToProcess | warnings array to process or undefined | refarray
# return       $Warnigs | epages warnings | refarray
#========================================================================================
sub getStatesAsWarning{
    my $self = shift;
    my ($aWarningsToProcess) = @_;

    $aWarningsToProcess //= $self->{'WarningStates'};

    my $aWarnings =  BlockWarnings (
        sub {
             foreach my $WarningState ( @{$aWarningsToProcess} ){
                eval { Warning("$WarningState") };
                if ( ExistsError() ) {
                    my $Error = GetError();
                    LogError($Error) if ( $Error->code =~ /Native/ms );
                }
            }
        },
    );

    return $aWarnings;
}
1;