#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship
# base         DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod
# state        public
#----------------------------------------------------------------------------------------
# description  Class for shipping fulfillment connection
#========================================================================================
package DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship;
use base DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod;
use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::AddressValidationMerchant;
use DE_EPAGES::DHLIntraship::API::BuildEndPoint;
use DE_EPAGES::DHLIntraship::API::BuildLoginData;
use DE_EPAGES::DHLIntraship::API::Constants qw (
    %DHLINTRASHIP_LABELSTATUS
    $DHLINTRASHIP_WRONG_POSTNUMMER_MESSAGE
    %SHIPPING_LABEL_STATUS
    @VISIBLE_SHIPPING_ADDRESS_FIELDS
    @MANDATORY_SHIPPING_ADDRESS_FIELDS
    %DHLVERSENDEN_STATUS_CODES
);
use DE_EPAGES::DHLIntraship::API::DeveloperAuthentication;
use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;
use DE_EPAGES::DHLIntraship::API::Tools;
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use constant WRONG_POSTNUMBER_ERROR_REASON => 'Customer ID Number is invalid';
use constant LOGIN_FAILED_ERROR_REASON => -1;
use constant OTHER_ERROR_REASON => 0;
use constant SUCCESS_PROCESSING => 1;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $ShippingFulfillmentMethod->featureName;
#----------------------------------------------------------------------------------------
# description   Returns undef as no feature is connected to this ShippingFulfillmentMethod.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'DHLIntraship'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethodDHLIntraship = DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship->insert($Package,$hInfo);
#----------------------------------------------------------------------------------------
# example      my $Class = LoadClassByAlias('ShippingFulfillmentMethodDHLIntraship');
#               my $ShippingFulfillmentMethodDHLIntraship = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook
#               'OBJ_InsertShippingFulfillmentMethodDHLIntraship' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingFulfillmentMethodDHLIntraship | new object | object
# hook         OBJ_InsertShippingFulfillmentMethodDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodDHLIntrashipID - shipping label id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertShippingFulfillmentMethodDHLIntraship',
        {'Object' => $self, 'ShippingFulfillmentMethodDHLIntrashipID' => $self->id() });

    $self->addVisibleAddressFields(@VISIBLE_SHIPPING_ADDRESS_FIELDS);
    $self->addMandatoryAddressFields(@MANDATORY_SHIPPING_ADDRESS_FIELDS);

    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethodDHLIntraship->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShippingFulfillmentMethodDHLIntraship' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingFulfillmentMethodDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodDHLIntrashipID - shipping label id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;
    TriggerHook('OBJ_DeleteShippingFulfillmentMethodDHLIntraship',
        {'Object' => $self, 'ShippingFulfillmentMethodDHLIntrashipID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethodDHLIntraship->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShippingFulfillmentMethodDHLIntraship'
#               and 'OBJ_AfterUpdateShippingFulfillmentMethodDHLIntraship'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShippingFulfillmentMethodDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodDHLIntrashipID - shipping label id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShippingFulfillmentMethodDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodDHLIntrashipID - shipping label id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;
    TriggerHook('OBJ_BeforeUpdateShippingFulfillmentMethodDHLIntraship',
        {'Object'=>$self,
            'Values'=> $hValues,
            'ShippingFulfillmentMethodDHLIntrashipID' => $self->id,
            'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingFulfillmentMethodDHLIntraship',
                {'Object'=>$self,
                    'Values'=> $hValues,
                    'ShippingFulfillmentMethodDHLIntrashipID' => $self->id,
                    'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     exportTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->exportTransaction( $Transaction );
#----------------------------------------------------------------------------------------
# description  Does a csv export if ShippingFulfillment integration uses an export file
#               for generating shipping labels.
#               Has to be implemented in child class, if
#               ShippingFulfillmentType.IntegrationType =~ m/Export/
#----------------------------------------------------------------------------------------
# input        $Transaction | shipping transaction | object
#========================================================================================
sub exportTransaction {
    my $self = shift;
    my ($Transaction) = @_;
    Error('NotImplemented');
}

#========================================================================================
# function     importTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->importTransaction( $Transaction );
#----------------------------------------------------------------------------------------
# description  Does an import if ShippingFulfillment integration uses an import file
#               for assigning tracking numbers to orders.
#               Has to be implemented in child class, if
#               ShippingFulfillmentType.IntegrationType =~ m/Import/
#----------------------------------------------------------------------------------------
# input        $Transaction | shipping transaction | object
# input        $ImportFile | reference to an import file | object
#========================================================================================
sub importTransaction {
    my $self = shift;
    my ($Transaction, $ImportFile) = @_;
    Error('NotImplemented');
}

#========================================================================================
# function     isUsable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isUsable = $ShippingFulfillmentMethod->isUsable( );
#----------------------------------------------------------------------------------------
# description  Checks if ShippingFulfillmentMethod is usable
#----------------------------------------------------------------------------------------
# return       $IsUsable | True if ShippingFulfillmentMethod is usable | boolean
#========================================================================================
sub isUsable {
    my $self = shift;
    return $self->get('WebserviceUsername') &&
           $self->get('WebservicePassword') &&
           $self->get('AreMandatoryShippingFieldsSet') &&
           $self->_isUsableWithShopAddress() &&
           $self->SUPER::isUsable();
}

#========================================================================================
# function     _isUsableWithShopAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isUsableWithShopAddress( );
#----------------------------------------------------------------------------------------
# description  checks if the ShopAddress contains all needed Values
#----------------------------------------------------------------------------------------
# return       $IsUsable | is it usable with shopaddress? | boolean
#========================================================================================
sub _isUsableWithShopAddress {
    my $self = shift;

    my $IsUsable;
    my $Shop = $self->getSite();
    my $ShopAddress = $Shop->get('Address');
    if ( isValid($ShopAddress) ){
        my $AddressValidationMerchant = DE_EPAGES::DHLIntraship::API::AddressValidationMerchant->new( $ShopAddress );
        my $aWarningStates = $AddressValidationMerchant->getNotProcessableWarningStates();
        $IsUsable = 1 if ( scalar(@{$aWarningStates}) == 0 );
    }

    return $IsUsable;
}

#========================================================================================
# function     areMandatoryShippingFieldsSet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $areMandatoryShippingFieldsSet = $ShippingFulfillmentMethod->areMandatoryShippingFieldsSet( );
#----------------------------------------------------------------------------------------
# description  Checks if mandatory shipping address fields are set into basket registration settings
#----------------------------------------------------------------------------------------
# return       $areMandatoryShippingFieldsSet | True if mandatory shipping address fields are set | boolean
#========================================================================================
sub areMandatoryShippingFieldsSet {
    my $self = shift;

    my $Shop = $self->getSite();
    my $ShippingAddressMandatoryFields = ',' . $Shop->get('ShippingAddressSetMandatoryFields') . ',';

    # set mandatory shipping fields list as required
    foreach my $field ( qw( FirstName LastName City Street Zipcode EMail) ) {
        return 0 if not ($ShippingAddressMandatoryFields =~ /,$field,/);
    }
    return 1;
}
#========================================================================================
# function     generateShippingLabels
# state        public
#----------------------------------------------------------------------------------------
# syntax       generateShippingLabels( $ShippingTransaction, $LanguageID );
#----------------------------------------------------------------------------------------
# description  operates a stack of Shippinglabels and set the transstatus
#----------------------------------------------------------------------------------------
# input        $ShippingTransaction | Shipping transaction object | object_ref
# input        $LanguageID | the language ID | String
#========================================================================================
sub generateShippingLabels{
    my $self = shift;
    my ($ShippingTransaction, $LanguageID) = @_;


    my $aShippingLabels = $ShippingTransaction->get('ShippingLabels');
    my $countFaultyShippinglabels = 0;
    my $transStatus;

    my $MethodName = (caller(0))[3];
    DE_EPAGES::DHLIntraship::API::Tools->LogShopContext(
            $ShippingTransaction,
            $MethodName,
            sub {
                foreach my $ShippingLabel (@{$aShippingLabels}) {
                    eval{
                         $self->_generateOneShippingLabel($ShippingLabel);
                    };
                    if ( ExistsError() ) {
                        my $Error = GetError();
                        $self->_reportAndUpdateShippingLabelOnError($Error, $ShippingLabel);

                        if ( $Error->code eq 'LOGIN_FAILED_ERROR_REASON') {
                            $transStatus = $SHIPPING_LABEL_STATUS{'WRONG_CREDENTIALS'};
                            last;
                        } elsif ($Error->code eq 'OTHER_ERROR_REASON') {
                            $countFaultyShippinglabels++;
                        }

                        Error($Error) if ( $Error->code eq 'NativDie' );
                    }
                }
            }
    );

    if (!isValid($transStatus)) {
        $transStatus = $SHIPPING_LABEL_STATUS{'ALL_OK'};
        if ( $countFaultyShippinglabels == scalar @{$aShippingLabels} ) {
            $transStatus = $SHIPPING_LABEL_STATUS{'ALL_KO'};
        } elsif ( $countFaultyShippinglabels >= 1 ) {
            $transStatus = $SHIPPING_LABEL_STATUS{'SOME_OK'};
        }
    }

    $ShippingTransaction->set({'TransStatus' => $transStatus});

    return;
}
#========================================================================================
# function     _generateOneShippingLabel
# state        private
#----------------------------------------------------------------------------------------
# syntax       _generateOneShippingLabel( $ShippingLabel );
#----------------------------------------------------------------------------------------
# description  This private Function encapsulate the generation process for shipping labels.
#               Its writes the pdf-file to a static folder and set the flags 'LabelStatus'
#               and 'Issued' and also the 'TrackingID'
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | a shipping lable object | object
#========================================================================================
sub _generateOneShippingLabel {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $IsTestLabel = $ShippingLabel->get('IsTestLabel');
    my $DHLWSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder',
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser',
        $IsTestLabel);
    my $aResponse = $DHLWSClient->createShipmentOrder($ShippingLabel);

    #$aResponse is an array since several shipping labels in one request.
    #In this function we only handle one at the time
    my $Response = @$aResponse[0];


    if ($self->_isErrorResponse($Response)) {
        my $ErrorCode;
        if ($self->_isLoginFailedErrorResponse($Response)) {
            $ErrorCode = 'LOGIN_FAILED_ERROR_REASON';
        } else {
            $ErrorCode = 'OTHER_ERROR_REASON';
        }

        my $StatusMessage = @{$Response->{'StatusMessage'}}[0];

        Error($ErrorCode,
                {
                    'StatusCode' => $Response->{'StatusCode'},
                    'StatusMessage' => $StatusMessage,
                    'StatusText' => $Response->{'StatusText'}
                }
        );
    }

    $ShippingLabel->set({
            'LabelStatus'      => $DHLINTRASHIP_LABELSTATUS{'CREATED'},
            'Issued'           => 1,
            'TrackingID'       => $Response->{'ShipmentNumber'},
            'LabelDownloadURL' => $Response->{'Labelurl'},
            'InvalidAddress'   => $Response->{'InvalidAddress'},
    });

    LogShippingFulfillment('DHLIntraship', 'Shipping Label dispatched successfully: ',
        {
            'ShippingLabel' => $ShippingLabel->alias,
            'ShippingLabelID' => $ShippingLabel->id,
            'TrackingID' => $Response->{'ShipmentNumber'},
        }
    );

    return;
}

#========================================================================================
# function     _reportAndUpdateShippingLabelOnError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_reportAndUpdateShippingLabelOnError($Error, $ShippingLabel);
#----------------------------------------------------------------------------------------
# description  report error and update shipping label information accordingly
#----------------------------------------------------------------------------------------
# input        $Error | Error Object | object
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _reportAndUpdateShippingLabelOnError {
    my $self = shift;
    my ($Error, $ShippingLabel) = @_;

    my $StatusText = $Error->vars->{'StatusText'};
    my $StatusMessage = $Error->vars->{'StatusMessage'};

    my $IsPostnummerError = $self->_isPostnummerErrorMessage($StatusMessage);

    $ShippingLabel->set({
            'Issued' => '0',
            'LabelStatus' => ($IsPostnummerError) ?
                    $DHLINTRASHIP_LABELSTATUS{'WRONG_POSTNUMBER'} : $DHLINTRASHIP_LABELSTATUS{'ERROR'},
            'ShippingLabelErrorDescription' => $StatusText . ' / ' . $StatusMessage
    });

    LogShippingFulfillment('DHLIntraship', 'Error at Shipping Label dispatch phase: ',
        {
            'ShippingLabel' => $ShippingLabel->alias(),
            'ShippingLabelID' => $ShippingLabel->id(),
            'ErrorCode' => $StatusText,
            'ErrorReason' => $StatusMessage,
        }
    );

    return;
}

#========================================================================================
# function     _isPostnummerErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isPostnummerError = $self->_isPostnummerErrorMessage($StatusMessage);
#----------------------------------------------------------------------------------------
# description  return boolean stating if error message is a postnummer error message
#----------------------------------------------------------------------------------------
# input        $StatusMessage | status message string | string
# return       $isPostnummerError |is a postnummer error | boolean
#========================================================================================
sub _isPostnummerErrorMessage {
    my $self = shift;
    my ($StatusMessage) = @_;

    return $StatusMessage ~~ /$DHLINTRASHIP_WRONG_POSTNUMMER_MESSAGE/;
}

#========================================================================================
# function     _isErrorResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isErrorResponse = $self->_isErrorResponse($Response);
#----------------------------------------------------------------------------------------
# description  return boolean stating if response is an error
#----------------------------------------------------------------------------------------
# input        $Response | response hash | hash.ref
# return       $isErrorResponse |is an erroneus response | boolean
#========================================================================================
sub _isErrorResponse {
    my $self = shift;
    my ($Response) = @_;

    return 1 if ($Response->{'IsError'} || ($Response->{'StatusCode'} != 0) );
    return 0;
}

#========================================================================================
# function     _isLoginFailedErrorResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isErrorResponse = $self->_isLoginFailedErrorResponse($Response);
#----------------------------------------------------------------------------------------
# description  return boolean stating if response is a login failure error
#----------------------------------------------------------------------------------------
# input        $Response | response hash | hash.ref
# return       $isErrorResponse |is an erroneus response | boolean
#========================================================================================
sub _isLoginFailedErrorResponse {
    my $self = shift;
    my ($Response) = @_;

    return 1 if ( $Response->{'StatusCode'} ~~ $DHLVERSENDEN_STATUS_CODES{'unvalid_login'});
    return 0;
}

1;