#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address
# state        public
# description  Address element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Origin;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Address = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Address | DHL Versenden address element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAddress = $self->assemble($Address);
#----------------------------------------------------------------------------------------
# description  Create the hash with address data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $Address | Address object | object
# input        $Name2 | Second name (only for Receivers) | string
# return       $hAddress | DHL Versenden address element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($Address, $Name2) = @_;

    my $hAddressData = $Address->get( [ 'StreetOnly', 'HouseNumber', 'Street2', 'Zipcode', 'City' ] );

    my $LeafNodeHelper = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper->new();
    $LeafNodeHelper->addLeaf('streetName', $hAddressData->{'StreetOnly'});
    $LeafNodeHelper->addLeaf('streetNumber', $hAddressData->{'HouseNumber'});
    $LeafNodeHelper->addLeaf('addressAddition', $self->_concatenateAddressExtensionAndStreet2($hAddressData));
    $LeafNodeHelper->addLeaf('zip', $hAddressData->{'Zipcode'});
    $LeafNodeHelper->addLeaf('city', $hAddressData->{'City'});

    my $hAddress = {};

    if (defined($Name2)) {
        $LeafNodeHelper->addLeaf('name2', $Name2);
        $hAddress->{'name2'} = $LeafNodeHelper->getLeaf('name2');
    }
    $hAddress->{'streetName'} = $LeafNodeHelper->getLeaf('streetName');
    $hAddress->{'streetNumber'} = $LeafNodeHelper->getLeaf('streetNumber');
    $hAddress->{'addressAddition'} = $LeafNodeHelper->getLeaf('addressAddition');
    $hAddress->{'zip'} = $LeafNodeHelper->getLeaf('zip');
    $hAddress->{'city'} = $LeafNodeHelper->getLeaf('city');

    my @Warnings = @{$LeafNodeHelper->getWarnings()};
    my $hOrigin = $self->_assembleOrigin($Address);
    push(@Warnings, @{$hOrigin->{'nodeWarnings'}});

    return {
        'nodeData' => {
            'Address' => {
                %{$hAddress},
                # no optional dispatchingInformation (not used)
                %{$hOrigin->{'nodeData'}},
            }
        },
        'nodeWarnings' => \@Warnings
    };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleOrigin {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Origin->assemble($Address);
}

sub _concatenateAddressExtensionAndStreet2 {
    my $self = shift;
    my ($hAddressData) = @_;

    my @AddressAddition = ();

    if (isValid($hAddressData->{'AddressExtension'})) {
        push(@AddressAddition, $hAddressData->{'AddressExtension'});
    }

    if (isValid($hAddressData->{'Street2'})) {
        push(@AddressAddition, $hAddressData->{'Street2'});
    }

    return join(' ', @AddressAddition);
}

1;