#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver
# state        public
# description  Receiver element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Packstation;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Receiver = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Receiver | DHL Versenden receiver element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hReceiver = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with receiver data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hReceiver | DHL Versenden receiver element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $CustomerAddress = $ShippingLabel->get('Address');

    my $LeafNodeHelper = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper->new();

    my $hCommunication = $self->_assembleCommunication($CustomerAddress);
    my $hReceiver = {%{$hCommunication->{'nodeData'}}};

    my @Warnings = ();
    push(@Warnings, @{$hCommunication->{'nodeWarnings'}});

    if ( defined($CustomerAddress->get('DHLPackstationId')) ) {
        $LeafNodeHelper->addLeaf('name1', $CustomerAddress->get('FullName'));

        my $hPackstation = $self->_assemblePackstation($CustomerAddress);

        $hReceiver = { %{$hReceiver}, %{$hPackstation->{'nodeData'}} };
        push(@Warnings, @{$hPackstation->{'nodeWarnings'}});
    } else {
        my $hReceiverNames = $self->_getNamesPrioritizingCompanyOverFullName($CustomerAddress);
        $LeafNodeHelper->addLeaf('name1', $hReceiverNames->{'name1'});

        my $hAddress = $self->_assembleAddressReceiver($CustomerAddress, $hReceiverNames->{'name2'});

        $hReceiver = { %{$hReceiver}, %{$hAddress->{'nodeData'}} };
        push(@Warnings, @{$hAddress->{'nodeWarnings'}});
    }

    $hReceiver->{'name1'} = $LeafNodeHelper->getLeaf('name1');
    push(@Warnings, @{$LeafNodeHelper->getWarnings()});

    return {
        'nodeData' => {
            'Receiver' => {
                %{$hReceiver},
                # Postfiliate not used
                # ParcelShop not used
            }
        },
        'nodeWarnings' => \@Warnings
    };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleCommunication {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication->assemble($Address);
}

sub _assembleAddressReceiver {
    my $self = shift;
    my ($Address, $ReceiverName2) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address->assemble($Address, $ReceiverName2);
}

sub _assemblePackstation {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Packstation->assemble($Address);
}

sub _getNamesPrioritizingCompanyOverFullName {
    my $self = shift;
    my ($Address) = @_;

    my $hNames = {};
    my $Company = $Address->get('Company');
    if (isValid($Company)) {
        $hNames->{'name1'} = $Company;
        $hNames->{'name2'} = $Address->get('FullName');
    } else {
        $hNames->{'name1'} = $Address->get('FullName');
        $hNames->{'name2'} = undef;
    }

    return $hNames;
}

1;