#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products
# state        public
#----------------------------------------------------------------------------------------
# description  validate and get the product based informations from Shippinglabel
#----------------------------------------------------------------------------------------
# example      my $ProductData = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $ShippingLabel );
#               if ($ProductData->isComplete();){print "all is fine!\n";}
#               or:
#               $ProductData->loadAttributes();
#               my $TotalWeight = $ProductData->{'TotalWeight'};
#========================================================================================
package DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );

use constant MAX_WEIGHT => 31.5;
use constant MIN_WEIGHT => 0.1;

use strict;

sub new {
    my $class = shift;
    my ( $ShippingLabel, $hPackageTypeConditions )  = @_;
    my $self  = bless {
        'ShippingLabel' => $ShippingLabel,
        'WarningStates' => [],
        'PackageTypeConditions' => $hPackageTypeConditions // {
            'Weltpaket' => {
                'MaxLenght'         => '0',
                'MaxWidth'          => '0',
                'MaxHeight'         => '0',
            },
            'Europaket' => {
                'MaxLenght'         => '120',
                'MaxWidth'          => '60',
                'MaxHeight'         => '60',
            },
            'Paket' => {
                'MaxLenght'         => '200',
                'MaxWidth'          => '200',
                'MaxHeight'         => '200',
            }
        },
    }, $class;
    return $self;
}

#========================================================================================
# function     isComplete
# state        public
#----------------------------------------------------------------------------------------
# syntax       isComplete( );
#----------------------------------------------------------------------------------------
# description  checks if all mandatory Atrributes are ok
#----------------------------------------------------------------------------------------
# return       $ReturnVariable | if all Attributes are fine | boolean
#========================================================================================
sub isComplete{
    my $self = shift;
    $self->loadAttributes();
    return unless (scalar @{$self->{'WarningStates'}} == 0);
    return 1;
}

#========================================================================================
# function     loadAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       loadAttributes( );
#----------------------------------------------------------------------------------------
# description  load all Attributes to the class object
#========================================================================================
sub loadAttributes {
    my $self = shift;
    $self->_loadWeight();
    $self->_loadLineItemsSum();
    $self->_loadDimensions();
    return;
}
#========================================================================================
# function     _isValidAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isValidAttribute( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Check if in the current instanste of this class is a Attribute with the
#               AttributeName. If not it will store the Name for warnings.
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
# return       $ReturnVariable | true if the Attribute is valid otherwise false | boolean
#========================================================================================
sub _isValidAttribute {
    my $self = shift;
    my ( $AttributeName ) = @_;
    unless ( isValid( $self->{$AttributeName} ) ) {
        my $Warning = "Parameter: $AttributeName is mandatory but not set";
        push( @{$self->{'WarningStates'}}, $Warning );
        return 0;
    }
    return 1;
}


#========================================================================================
# function     _loadWeight
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadWeight( );
#----------------------------------------------------------------------------------------
# description  load the Weight from Shippinglabel
#========================================================================================
sub _loadWeight {
    my $self = shift;
    $self->{'TotalWeight'} = $self->{'ShippingLabel'}->get('TotalWeight');
    if ( defined $self->{'TotalWeight'} ){
        if ( $self->{'TotalWeight'} > MAX_WEIGHT ){
            GetLog->warn(sprintf('Total weight is %.2f kg but maximum weight is %.1f kg.', $self->{'TotalWeight'}, MAX_WEIGHT));
            push( @{$self->{'WarningStates'}}, sprintf('Gesamtgewicht ist zu hoch. Hchstens %.1f kg', MAX_WEIGHT));
        }
        elsif ( $self->{'TotalWeight'} < MIN_WEIGHT ){
            GetLog->warn(sprintf('Total weight is %.2f kg but minimum weight is %.1f kg.', $self->{'TotalWeight'}, MIN_WEIGHT));
            push( @{$self->{'WarningStates'}}, 'TotalWeightTooLow' );
        }
    }

    return;
}

#========================================================================================
# function     _loadAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadAmount( );
#----------------------------------------------------------------------------------------
# description  load the Amount of the Shipping label as class var
#========================================================================================
#sub _loadAmount {
#    my $self = shift;
#    $self->{'TotalAmount'} = $self->{'ShippingLabel'}->get('TotalAmount');
#    $self->_isValidAttribute('TotalAmount');
#    return;
#}

#========================================================================================
# function     _loadLineItemsSum
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadLineItemsSum( );
#----------------------------------------------------------------------------------------
# description  load the LineItemsSum of the Shipping label as class var
#========================================================================================
sub _loadLineItemsSum {
    my $self = shift;
    $self->{'LineItemContainer'} = $self->{'ShippingLabel'}->get('LineItemContainer');
    if ( $self->_isValidAttribute('LineItemContainer') ){
        $self->{'LineItemsSum'} = $self->{'LineItemContainer'}->get('LineItemsSum');
        $self->_isValidAttribute('LineItemsSum');
    }
    return;
}

#========================================================================================
# function     _loadDimensions
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadDimensions( );
#----------------------------------------------------------------------------------------
# description  load the dimension of the ShippingLabel
#========================================================================================
sub _loadDimensions {
    my $self = shift;
    my $aParcelDimensions = ['ParcelDepth', 'ParcelLength', 'ParcelWidth'];
    foreach my $ParcelDimension (@{$aParcelDimensions}){
        $self->{$ParcelDimension} = $self->{'ShippingLabel'}->get($ParcelDimension);
        if ( defined $self->{$ParcelDimension} ){
            $self->_convertToCentimetre($ParcelDimension);
        }
    }
    return;
}

#========================================================================================
# function     _convertToCentimetre
# state        private
#----------------------------------------------------------------------------------------
# syntax       _convertToCentimetre( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Takes a Value from class var and convert it to centimetre
#----------------------------------------------------------------------------------------
# input        $AttributeName | name of var in self | String
#========================================================================================
sub _convertToCentimetre {
    my $self = shift;
    my ( $AttributeName ) = @_;
        my $OriginalFactor = 1; #TODO Need original from the Shipping Label
        my $GoalFactor = LoadObjectByPath( '/Units/metre/centimetre' )->get( 'Factor' );
        my $ConversionFactor = $OriginalFactor / $GoalFactor;
        $self->{$AttributeName}  *= $ConversionFactor;
    return;
}

#========================================================================================
# function     getStatesAsWarnig
# state        public
#----------------------------------------------------------------------------------------
# syntax       getStatesAsWarnig( $Input1 );
#----------------------------------------------------------------------------------------
# description  Get the States and create epages warnings from this
#----------------------------------------------------------------------------------------
# return       $Warnigs | epages warnings | refarray
#========================================================================================
sub getStatesAsWarning{
    my $self = shift;
    my $aWarnings =  BlockWarnings (
        sub {
             foreach my $WarningState ( @{$self->{'WarningStates'}} ){
                eval {Warning($WarningState)};
                if ( ExistsError() ) {
                    my $Error = GetError();
                    LogError($Error) if ( $Error->code =~ /Native/ );
                }
            }
        },
    );
    return $aWarnings;
}
1;