#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Receiver
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Receiver module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Receiver;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

use constant TEST_ADDRESS_FULL_NAME => 'John Doe';
use constant TEST_PACKSTATION_ID => '110';
use constant TEST_ADDRESS_PACKSTATION_FULL_NAME => 'Packstation 110';
use constant TEST_ADDRESS_COMPANY_NAME => 'My Company Name';


#========================================================================================
sub test {
    my $self = shift;

    $self->receiver_assemble_with_address_test();
    $self->receiver_assemble_with_address_no_company_test();
    $self->receiver_assemble_with_packstation_test();
    $self->receiver_assemble_with_address_subnodes_with_warnings_test();
    $self->receiver_assemble_with_packstation_subnodes_with_warnings_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub receiver_assemble_with_address_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => TEST_ADDRESS_COMPANY_NAME,
        'DHLPackstationId' => undef,
        'FullName' => TEST_ADDRESS_FULL_NAME
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $ReceiverWithWarnings = 0;
    my $mockedReceiverElement = $self->_createReceiver($ReceiverWithWarnings);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver->{'nodeData'},
        { 'Receiver' => {
              'name1' => TEST_ADDRESS_COMPANY_NAME,
              'Address' => {},
              'Communication' => {},
          }
        },
        "$SubTestName - Receiver hash assembled with an Address sub-node"
    );
}
sub receiver_assemble_with_address_no_company_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => undef,
        'DHLPackstationId' => undef,
        'FullName' => TEST_ADDRESS_FULL_NAME
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $ReceiverWithWarnings = 0;
    my $mockedReceiverElement = $self->_createReceiver($ReceiverWithWarnings);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver->{'nodeData'},
        { 'Receiver' => {
              'name1' => TEST_ADDRESS_FULL_NAME,
              'Address' => {},
              'Communication' => {},
          }
        },
        "$SubTestName - Receiver hash assembled with an Address sub-node"
    );
}

sub receiver_assemble_with_packstation_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => TEST_ADDRESS_COMPANY_NAME,
        'DHLPackstationId' => TEST_PACKSTATION_ID,
        'FullName' => TEST_ADDRESS_PACKSTATION_FULL_NAME
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $ReceiverWithWarnings = 0;
    my $mockedReceiverElement = $self->_createReceiver($ReceiverWithWarnings);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver->{'nodeData'},
        { 'Receiver' => {
              'name1' => TEST_ADDRESS_PACKSTATION_FULL_NAME,
              'Packstation' => {},
              'Communication' => {},
          }
        },
        "$SubTestName - Receiver hash assembled with a Packstation sub-node"
    );
}

sub receiver_assemble_with_address_subnodes_with_warnings_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => TEST_ADDRESS_COMPANY_NAME,
        'DHLPackstationId' => undef,
        'FullName' => TEST_ADDRESS_FULL_NAME
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $ReceiverWithWarnings = 1;
    my $mockedReceiverElement = $self->_createReceiver($ReceiverWithWarnings);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver->{'nodeWarnings'},
        [ { 'subnode1' => 'warning' }, { 'subnode2' => 'warning' } ],
        "$SubTestName - warnings array for Receiver coming from subnodes (not a Packstation address)"
    );
}

sub receiver_assemble_with_packstation_subnodes_with_warnings_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => TEST_ADDRESS_COMPANY_NAME,
        'DHLPackstationId' => TEST_PACKSTATION_ID,
        'FullName' => TEST_ADDRESS_PACKSTATION_FULL_NAME
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $ReceiverWithWarnings = 1;
    my $mockedReceiverElement = $self->_createReceiver($ReceiverWithWarnings);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver->{'nodeWarnings'},
        [ { 'subnode1' => 'warning' }, { 'subnode3' => 'warning' } ],
        "$SubTestName - warnings array for Receiver coming from subnodes (with a Packstation address)"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createReceiver {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver',
        $aParameterList
    );

    my $hCommunicationReturn = {
            'nodeData' => { 'Communication' => {} },
            'nodeWarnings' => $withWarning? [{'subnode1' => 'warning'}] : []
    };
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleCommunication',
            $hCommunicationReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );

    my $hAddressReturn = {
            'nodeData' => { 'Address' => {} },
            'nodeWarnings' => $withWarning? [{'subnode2' => 'warning'}] : []
    };
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleAddressReceiver',
            $hAddressReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' }, 'string' ]
    );

    my $hPackstationReturn = {
            'nodeData' => { 'Packstation' => {} },
            'nodeWarnings' => $withWarning? [{'subnode3' => 'warning'}] : []
    };
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assemblePackstation',
            $hPackstationReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;
    my ($hAddress) = @_;

    my $aParameterList = {
        'Address' => $self->_createAddress($hAddress),
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hAddress) = @_;

    my $hParameterList = {
        'Company' => $hAddress->{'Company'},
        'DHLPackstationId' => $hAddress->{'DHLPackstationId'},
        'FullName' => $hAddress->{'FullName'},
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $hParameterList
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
