﻿#!/usr/bin/perl

#============================================================================
# Author: Friedrich Gehring
# Description:
#     Move translations from template file to dictionary
#============================================================================

use utf8;
use strict;
use Getopt::Long;
use DE_EPAGES::Core::API::File qw ( RealPath AbsolutePath fsplit GetFileNames ExistsFile DeleteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Dictionary::API::Compare qw ( CompareDictionaryFile );
use DE_EPAGES::Dictionary::API::XML::DictionaryImportHandler qw ( ImportDictionary );
use Tie::File;
use XML::Simple;

# package variable for warning messages
our @Warnings = ();

sub Main {
    my $LanguagePrepared = "de";
    my $InputLanguageCodes;
    my $TemplatePath;
    my $Help;
    my $Verbose = 1;
    my $MoveAll = 0;
    my $MoveToGlobal = 0;
    my $NoSort = 0;

    GetOptions(
        'tmpl=s'  => \$TemplatePath,
        'pre=s'   => \$LanguagePrepared,
        'lang=s'  => \$InputLanguageCodes,
        'output!' => \$Verbose,
        'help'    => \$Help,
        'all'     => \$MoveAll,
        'global'  => \$MoveToGlobal,
        'nosort'  => \$NoSort
    );

    Usage() if $Help;
    Usage( 'Missing required parameter -tmpl' ) unless defined $TemplatePath;

    # clean plossible extensions of TemplatePath
    $TemplatePath = $1 if $TemplatePath =~ m/(.+)\.$/;
    $TemplatePath = $1 if $TemplatePath =~ m/(.+)\.xml$/;
    $TemplatePath = $1 if $TemplatePath =~ m/(.+)\.$LanguagePrepared$/;
    $TemplatePath = $1 if $TemplatePath =~ m/(.+)\.html$/;
    $TemplatePath = $1 if $TemplatePath =~ m/(.+)\.js$/;
    $TemplatePath = $1 if $TemplatePath =~ m/(.+)\.json$/;
    $TemplatePath = RealPath(AbsolutePath($TemplatePath));

    my $LanguageCodes;
    if (defined $InputLanguageCodes && !(lc $InputLanguageCodes eq 'all')) {
        if (lc $InputLanguageCodes eq 'unity') {
            $LanguageCodes = ['de', 'en', 'en-US'];
        } else {
            $LanguageCodes = [split(/,/, $InputLanguageCodes)];
        }
    } else {
        # get language codes for moving
        $LanguageCodes = _getLanguageCodes($TemplatePath, $LanguagePrepared, $MoveAll);
    }

    die "Error: No files found for moving. Please check file path: $TemplatePath"
        unless scalar @$LanguageCodes;

    # compare dictionaries in case the dictionary of the prepared language exists and not all tags should be moved
    my $CompareDictionaries = (not $MoveAll and ExistsFile($TemplatePath.'.'.$LanguagePrepared.'.xml')) ? 1 : 0;

    my $aTagsToMove;
    foreach my $LanguageToMove (@$LanguageCodes) {
        $aTagsToMove = _findTagsToMove($LanguagePrepared, $LanguageToMove, $TemplatePath,
            $CompareDictionaries, $MoveAll, $MoveToGlobal);
        _moveTranslations($aTagsToMove, $LanguageToMove, $TemplatePath, $Verbose, $MoveToGlobal, $NoSort);
        unless ($CompareDictionaries or scalar @Warnings) {
            my $TemplateDictionaryPathToDelete = $TemplatePath.'.'.$LanguageToMove.'.xml';
            print "-------------------------------------------------------------------\n" if $Verbose;
            print "Delete File: $TemplateDictionaryPathToDelete\n" if $Verbose;
            DeleteFile($TemplateDictionaryPathToDelete);
        }
    }

    # print warnings if any
    print "===================================================================\n";
    if (scalar @Warnings) {
        print "Finished moving with the following warnings:\n";
        print $_ . "\n" foreach @Warnings;
    } else {
        print "Moving finished sucessfully.\n"
    }
}

#========================================================================================
# §function     _getLanguageCodes
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       @LanguageCodes = _getLanguageCodes($TemplatePath, $LanguagePrepared);
#----------------------------------------------------------------------------------------
# §description  Scans the file list of the directory of the template dictionary and
#               returns all language codes for which a template dictionary exists
#               except the prepared language (unless $MoveAll = 1)
#----------------------------------------------------------------------------------------
# §input        $LanguagePrepared | language code (en,de ...) | string
# §input        $TemplatePath | (absolute) path of the template | string
# §input        $MoveAll | true if all languages should be moved | boolean
# §return       $aLanguageCodes | list language codes | array.ref
#========================================================================================
sub _getLanguageCodes {
    my ($TemplatePath, $LanguagePrepared, $MoveAll) = @_;

    my ($TemplateDictionaryPath) = fsplit($TemplatePath);

    my @LanguageCodes;
    foreach my $File (@{GetFileNames($TemplateDictionaryPath)}) {
        # get language codes for all files matching the given template path
        if ($File =~ m/$TemplatePath\.(.+)\.xml$/) {
            push @LanguageCodes, $1 if (($1 ne $LanguagePrepared) or $MoveAll);
        }
    }

    return \@LanguageCodes;
}

#========================================================================================
# §function     _findTagsToMove
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $aTagsToMove = _findTagsToMove($LanguagePrepared, $LanguageToMove,
#                   $TemplatePath, $CompareDictionaries)
#----------------------------------------------------------------------------------------
# §description  Compares the dictionary files of two languages and returns an array
#               of language tags that have been removed in the prepared file but are still
#               present in the file to move and occure in the dictionary
#               of the prepared language, too
#----------------------------------------------------------------------------------------
# §input        $LanguagePrepared | language code (en,de ...) | string
# §input        $LanguageToMove | language code (en,de ...) | string
# §input        $TemplatePath | (absolute) path of the template | string
# §input        $CompareDictionaries | dictionaries should be compared | boolean
# §input        $MoveAll | true if all languages should be moved | boolean
# §input        $MoveToGlobal | true if moving to the global dictionary | boolean
# §return       $aTagsToMove | list of tags to move | array.ref
#========================================================================================
sub _findTagsToMove {
    my ($LanguagePrepared, $LanguageToMove, $TemplatePath,
        $CompareDictionaries, $MoveAll, $MoveToGlobal) = @_;

    my $DictionaryPreparedPath
        = _dictionaryPath($LanguagePrepared, $TemplatePath, $MoveToGlobal);
    my $DictionaryToMovePath
        = _dictionaryPath($LanguageToMove, $TemplatePath, $MoveToGlobal);

    my $hDictionaryPrepared =
        ImportDictionary($DictionaryPreparedPath);
    my $hDictionaryToMove =
        ImportDictionary($DictionaryToMovePath);

    my ($LanguageCodePrepared) = split /-/, $LanguagePrepared;
    my ($LanguageCodeToMove) = split /-/, $LanguageToMove;

    my @TagsToMove;
    if ($CompareDictionaries) {
        # find differences when comparing the dictionaries of the two languages
        my $aErrors = CompareDictionaryFile($LanguagePrepared, $LanguageToMove, $TemplatePath);
        foreach my $Error ( @$aErrors ){
            my @Split = split ' ', $Error;
            # get reason for the error message
            my $ErrorCode = $Split[1] . ' ' . $Split[2];
            # get keyword from the error message
            my $Keyword = $Split[3];
            # show warning if keyword already exists in dictionary
            if (exists $hDictionaryToMove->{$LanguageCodeToMove}->{$Keyword}) {
                push (@Warnings, "Keyword \"$Keyword\" already exists in $DictionaryToMovePath.");
                next;
            }
            # keep keyword if it is extra in the file to move and exists in the dictionary
            if ($ErrorCode eq 'extra keyword'
                and exists $hDictionaryPrepared->{$LanguageCodePrepared}->{$Keyword}) {
                push (@TagsToMove, $Keyword);
            } else {
                # Tag has been deleted in prepared language
                # TODO: delete tag in other languages, too
            }
        }
    } else {
        my $hTemplateDictionary = ImportDictionary($TemplatePath.'.'.$LanguageToMove.'.xml');
        foreach my $Keyword (sort keys %{$hTemplateDictionary->{$LanguageCodeToMove}}) {
            # show warning if keyword already exists in dictionary
            if (exists $hDictionaryToMove->{$LanguageCodeToMove}->{$Keyword}) {
                push (@Warnings, "Keyword \"$Keyword\" already exists in $DictionaryToMovePath.");
                next;
            }
            if ($MoveAll or exists $hDictionaryPrepared->{$LanguageCodePrepared}->{$Keyword}) {
                push (@TagsToMove, $Keyword);
            } else {
                push (@Warnings, "Keyword \"$Keyword\" should be moved, " .
                "but does not exist in $DictionaryPreparedPath.");
                next;
                # Tag has been deleted in prepared language
                # TODO: delete tag in other languages, too
            }
        }
    }

    return \@TagsToMove;
}

#========================================================================================
# §function     _dictionaryPath
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $DictionaryPath
#                   = _dictionaryPath($LanguageCode, $TemplatePath);
#----------------------------------------------------------------------------------------
# §description  returns the absolute path of the dictionary
#               for the given language
#----------------------------------------------------------------------------------------
# §input        $LanguageCode | language code (en,de ...) | string
# §input        $TemplatePath | (absolute) path of the template | string
# §input        $MoveToGlobal | true if moving to the global dictionary | boolean
# §return       $DictionaryPath | Path of the dictionary | string
#========================================================================================
sub _dictionaryPath {
    my ($LanguageCode, $TemplatePath, $MoveToGlobal) = @_;

    my $DictionaryPath;
    if ($TemplatePath =~ m:(^.+/Templates/):) {
        if ($MoveToGlobal) {
            $DictionaryPath =
                RealPath($1."../../Dictionary/Templates/Dictionary.".$LanguageCode.".xml");
        } else {
            $DictionaryPath = $1."Dictionary.".$LanguageCode.".xml";
        }
    }
    else {
        die ("Error: No cartridge dictionary found for file " .
            "$TemplatePath and language $LanguageCode.");
    }

    return $DictionaryPath;
}


#========================================================================================
# §function     _moveTranslations
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       _moveTranslations($aTagsToMove, $LanguageCode, $TemplatePath);
#----------------------------------------------------------------------------------------
# §description  Moves the translations of the tags to move from the template dictionary
#               of the given language to the according cartridge dictionary
#----------------------------------------------------------------------------------------
# §input        $aTagsToMove | list of tags to move | array.ref
# §input        $LanguageCode | language code (en,de ...) | string
# §input        $TemplatePath | (absolute) path of the template | string
# §input        $MoveToGlobal | true if moving to the global dictionary | boolean
# §input        $NoSort | true if no sorting of the dictionary | boolean
# §input        $Verbose | if true output is printed, if false not | boolean
#========================================================================================
sub _moveTranslations {
    my ($aTagsToMove, $LanguageCode, $TemplatePath, $Verbose, $MoveToGlobal, $NoSort) = @_;

    my $TranslationFilePath = $TemplatePath.'.'.$LanguageCode.'.xml';
    my $DictionaryPath =
        _dictionaryPath($LanguageCode, $TemplatePath, $MoveToGlobal);
    my @TranslationFileLines;
    my @DictonaryLines;

    # tie the lines of the translation file to an array
    tie @TranslationFileLines, 'Tie::File', $TranslationFilePath
        or die "Error: Unable to open file $TranslationFilePath.";
    my $hKeywordLines = _assignLineNumberToKeyword(\@TranslationFileLines);
    # sort tags to move in order of the line numbers
    @$aTagsToMove = sort { $hKeywordLines->{$a}->{'Start'}
        <=> $hKeywordLines->{$b}->{'Start'} } @$aTagsToMove;

    # tie the lines of the dictionary to an array
    tie @DictonaryLines, 'Tie::File', $DictionaryPath
        or die "Error: Unable to open file $DictionaryPath.";

    my $LinesToFileEnding = 1;
    my $LinesInFile = scalar @DictonaryLines;
    # scan the end of the dictionary
    # until it is possible to add translation lines above the </Language> tag
    while ($DictonaryLines[-$LinesToFileEnding] !~ m:</Language>:) {
        $LinesToFileEnding++;
        die "Error: No closing </Language> tag in file $DictionaryPath"
            if $LinesToFileEnding > $LinesInFile;
    };

    if ($Verbose) {
        print "===================================================================\n";
        print "Language: $LanguageCode\n";
        print "-------------------------------------------------------------------\n";
        print "$TranslationFilePath: Move translations to $DictionaryPath\n";
        print "-------------------------------------------------------------------\n";
    }
    my $Counter=0;
    my $LineToMove;
    # moving all tags from the beginning of the file to the end
    foreach my $TagToMove ( @$aTagsToMove ) {
        if (exists $hKeywordLines->{$TagToMove}) {
            print "Keyword: $TagToMove\n" if $Verbose;
            my $LineToStart = $hKeywordLines->{$TagToMove}->{'Start'};
            my $LineToEnd = $hKeywordLines->{$TagToMove}->{'End'};
            for (my $KeywordLine = $LineToStart; $KeywordLine <= $LineToEnd; $KeywordLine++) {
                # remove the line from the template dictionary,
                # thereby take into account that previously lines above have been removed already
                $LineToMove = splice @TranslationFileLines, $KeywordLine - $Counter, 1;
                # add the line to the end of the dictonary,
                # move one line up per step to preserve the order of the tags
                # offset $LinesToFileEnding the closing </Language> </epages> tags
                splice @DictonaryLines, -$LinesToFileEnding, 0, $LineToMove;
                $Counter++;
            }
        }
    }
    print "Nothing moved.\n" if ($Counter == 0 and $Verbose);

    @DictonaryLines = _sortKeywords(\@DictonaryLines) unless $NoSort;

    untie @TranslationFileLines;
    untie @DictonaryLines;
}

#========================================================================================
# §function     _assignLineNumberToKeyword
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hKeywordLines = _assignLineNumberToKeyword(\@TranslationFileLines);
#----------------------------------------------------------------------------------------
# §description  For an array that contains the lines of a translation file a hash
#               is created whose keys are the keywords that are mapped
#               to the corresponding line number in the file (starting with 0)
#----------------------------------------------------------------------------------------
# §input        $aLineArray | Array of lines from a translation file | array.ref
# §return       $hKeywords | keyword to linenumber map | hash.ref
#========================================================================================
sub _assignLineNumberToKeyword {
    my ($aLineArray) = @_;

    my $XML;
    my %Keywords;

    my $NumberOfLines = scalar @$aLineArray;
    for (my $LineNumber = 0; $LineNumber < $NumberOfLines;  $LineNumber++) {
        my $element = $$aLineArray[$LineNumber];
        # take only lines with a keyword into account
        if ($element =~ m/Keyword=/) {
            my $LineNumberStart = $LineNumber;
            while ($element !~ m:</Translation>: and $LineNumber < $NumberOfLines) {
                $LineNumber++;
                $element .= $$aLineArray[$LineNumber];
            }
            if ($element =~ m:</Translation>:) {
                # parse XML to filter out the keyword
                $XML = XMLin($element);
                # store line area of the translation for the keyword
                $Keywords{$XML->{'Keyword'}}{'Start'} = $LineNumberStart;
                $Keywords{$XML->{'Keyword'}}{'End'} = $LineNumber;
            } else {
                $element =~ s/\R//g;
                push (@Warnings,
                    "Translation \'".$element."\' in Line ".($LineNumber+1).
                    " has no closing </Translation> tag. Error in XML file.");
            }
        }
    }

    return \%Keywords;
}

#========================================================================================
# §function     _assignKeywordToLine
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hLines = _assignLineNumberToKeyword(\@TranslationFileLines);
#----------------------------------------------------------------------------------------
# §description  For an array that contains the lines of a translation file a hash
#               is created whose keys are the translation lines itself that are mapped
#               to the corresponding keyword
#----------------------------------------------------------------------------------------
# §input        $aLineArray | Array of lines from a translation file | array.ref
# §return       $hLines| line(s) keyword map | hash.ref
#========================================================================================
sub _assignKeywordToLine {
    my ($aLineArray) = @_;

    my $XML;
    my %Lines;

    my $NumberOfLines = scalar @$aLineArray;
    for (my $LineNumber = 0; $LineNumber < $NumberOfLines;  $LineNumber++) {
        my $element = $$aLineArray[$LineNumber];
        # take only lines with a keyword into account
        if ($element =~ m/Keyword=/) {
            while ($element !~ m:</Translation>: and $LineNumber < $NumberOfLines) {
                $LineNumber++;
                $element .= $$aLineArray[$LineNumber];
            }
            if ($element =~ m:</Translation>:) {
                # parse XML to filter out the keyword
                $XML = XMLin($element);
                # store line area of the translation for the keyword
                $Lines{$element} = $XML->{'Keyword'};
            } else {
                $element =~ s/\R//g;
                push (@Warnings,
                    "Translation \'".$element."\' in Line ".($LineNumber+1).
                    " has no closing </Translation> tag. Error in XML file.");
            }
        }
    }

    return \%Lines;
}

#========================================================================================
# §function     _sortKeywords
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       @SortedDictionary = _sortKeywords(\@TranslationFileLines);
#----------------------------------------------------------------------------------------
# §description  For an array that contains the lines of a translation file all lines
#               containing language tags are sorted alphabetically regarding
#               to their keywords
#----------------------------------------------------------------------------------------
# §input        $aLineArray | Array of lines from a translation file | array.ref
# §input        $aSortedLineArray | Array of sorted lines from a translation file | array.ref
#========================================================================================
sub _sortKeywords {
    my ($aLineArray) = @_;

    my $hLinesToSort = _assignKeywordToLine($aLineArray);
    my @LinesToSort = keys %$hLinesToSort;

    #find beginning and end of the translation file
    my $LinesInFile = scalar @$aLineArray;
    my $StartLine = 0;
    my $LinesToEnd = 1;

    while ($aLineArray->[$StartLine] !~ m:<Translation:) {
        $StartLine++;
        die "Error: No <Translation> tag in file"
            if $StartLine >= $LinesInFile;
    };

    my @FileStart = splice (@$aLineArray, 0, $StartLine);

    while ($aLineArray->[-$LinesToEnd] !~ m:</Translation>:) {
        $LinesToEnd++;
        die "Error: No closing </Translation> tag in file"
            if $LinesToEnd >= $LinesInFile;
    };

    my @FileEnd = splice (@$aLineArray, -$LinesToEnd + 1);

    my @SortedLines = sort { $hLinesToSort->{$a} cmp $hLinesToSort->{$b} } @LinesToSort;

    return (@FileStart, @SortedLines, @FileEnd);
}


#========================================================================================
# §function     usage
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       usage ()
#----------------------------------------------------------------------------------------
# §description  Prints the required parameters of the script
#========================================================================================
sub Usage() {
    my ($Message) = @_;

    print "\n$Message\n\n" if $Message;
    print <<END_USAGE;
Description:
    For a given template, this script compares each template dictionary to the one
    of a language for which the movement of translations to the dictionary
    has been already done (default: German).
    In case of diffences it checks, if the translations for the given language
    have been moved to the dictionary and moves the translations
    for all other languages accordingly. Afterwards the dictionaries will be sorted
    alphabetically.
    In case there is no prepared language file present anymore (because all
    translations have been moved) but template dictionaries of other
    languages still exist, their tags will be moved and the files will be deleted.

Usage:
    perl $0 [options] [flags]

options:
    -tmpl       path of the template for which the translations should be moved
                without file extension (.html, .js, .json will be detected automatically)
    -pre        language for which the moving has already been done (default = de)
    -lang       languages that should be moved (comma separated, without spaces),
                unity = de,en,en-US (default = all)

flags:
    -help       show the command line options
    -nooutput   do not print any output
    -all        move all translations to dictionary and delete template dictionaries
                (no preparation needed; -pre option will be ignored)
    -global     move to the global dictionary
    -nosort     no (alphabetical) sorting after moving

Examples:
    perl $0 -tmpl ../../ShippingTypes/Templates/MBO/MBO-ShippingMethodProductsInBasketSettings.TabPage
    perl $0 -tmpl MBO-ShippingMethodProductsInBasketSettings.TabPage -pre en -nooutput
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;