#========================================================================================
# package       DE_EPAGES::Klarna::API::KlarnaOrderStatus
# state         public
# description   Klarna order status management
#========================================================================================
package DE_EPAGES::Klarna::API::KlarnaOrderStatus;
use base qw ( Exporter );
use strict;
our @EXPORT_OK = qw(
    CheckKlarnaOrderStatus
    CheckKlarnaOrderStatusViaScheduler
    UpdateOrderStatus
);

use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Klarna::API::Constants qw (
    KLARNA_STATUS_DENIED
    KLARNA_STATUS_ACCEPTED
    KLARNA_STATUS_PENDING
    KLARNA_STATUS_ERROR
);
use DE_EPAGES::Klarna::API::MerchantEvents qw (
    SendPaymentAcceptedMail
    SendPaymentDeniedMail
);
use DE_EPAGES::Klarna::API::KlarnaOperations::CheckOrderStatus;

#========================================================================================
# function        _setKlarnaAccepted
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $result = _setKlarnaAccepted($Order);
#----------------------------------------------------------------------------------------
# description     Marks an order as accepted (approved by Klarna)
#                  Sets the PaidOn flag if not already set. Resets the PendingOn flag.
#----------------------------------------------------------------------------------------
# input           $Order | the order to mark as accepted | object
# return          $result  | KLARNA_STATUS_ACCEPTED  | int
#========================================================================================
sub _setKlarnaAccepted {
    my ($Order) = @_;
    $Order->set({'PendingOn' => undef}) if(defined $Order->get('PendingOn'));
    $Order->set({'PaidOn' => GetCurrentDBHandle()->currentDateTime}) unless defined($Order->get('PaidOn'));

    SendPaymentAcceptedMail($Order);

    return;
}

#========================================================================================
# function        _setKlarnaPending
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $result = _setKlarnaPending($Order);
#----------------------------------------------------------------------------------------
# description     Marks an order as pending (awaiting approval by Klarna)
#                  Sets the PendingOn flag if not already set.
#----------------------------------------------------------------------------------------
# input           $Order | the order to mark as pending | object
# return          $result  | STATUS_PENDING  | int
#========================================================================================
sub _setKlarnaPending {
    my ($Order) = @_;
    $Order->set({'PendingOn' => GetCurrentDBHandle()->currentDateTime}) unless defined($Order->get('PendingOn'));

    return;
}

#========================================================================================
# function        _setKlarnaDenied
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $result = _setKlarnaDenied($Order);
#----------------------------------------------------------------------------------------
# description     Marks an order as denied (approvement denied by Klarna)
#                  Sets the CancelledOn flag if not already set. Resets the PendingOn flag.
#----------------------------------------------------------------------------------------
# input           $Order | the order to mark as denied | object
# return          $result  | STATUS_DENIED  | int
#========================================================================================
sub _setKlarnaDenied {
    my ($Order) = @_;
    $Order->set({'PendingOn' => undef}) if(defined $Order->get('PendingOn'));
    $Order->set({'CancelledOn' => GetCurrentDBHandle()->currentDateTime}) unless defined($Order->get('CancelledOn'));

    SendPaymentDeniedMail($Order);

    return;
}

#========================================================================================
# function        _updateKlarnaOrderStatus
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $result = _updateKlarnaOrderStatus($Order, $KlarnaOrderStatus);
#----------------------------------------------------------------------------------------
# description     Updates an order depending on the order status returned by Klarna
#----------------------------------------------------------------------------------------
# input           $Order | the order to update | object
# input           $KlarnaOrderStatus | the status returned by Klarna for this order | int
# return          $result | a STATUS_* constant  | int
#========================================================================================
sub _updateKlarnaOrderStatus {
    my ($Order, $KlarnaOrderStatus) = @_;

    if ($KlarnaOrderStatus == KLARNA_STATUS_ACCEPTED){
        _setKlarnaAccepted($Order);
    }
    elsif ($KlarnaOrderStatus == KLARNA_STATUS_PENDING){
        _setKlarnaPending($Order);
    }
    elsif ($KlarnaOrderStatus == KLARNA_STATUS_DENIED) {
        _setKlarnaDenied($Order);
    }

    return;
}

#========================================================================================
# function        UpdateOrderStatus
# state           public
#----------------------------------------------------------------------------------------
# syntax          my $result = UpdateOrderStatus($Servlet);
#----------------------------------------------------------------------------------------
# description     Request order status updates from Klarna and update the order flags
#                  accordingly. For use in the MBO
#----------------------------------------------------------------------------------------
# input           $Order | the order to update | object
# input           $hResponse | the response from Klarna webservices | hash reference
# return          $hResult | result of the operation
#                    - Message     Status message
#                    - StatusCode  Status code  | hash reference
#========================================================================================
sub UpdateOrderStatus {
    my ($Order,$hResponse) = @_;
    my $hResult;

    if(uc($hResponse->{'StatusCode'}) eq 'OK') {
        my $StatusMap = {
            (KLARNA_STATUS_ACCEPTED) => { 'StatusCode' => 'OK',  'Message' => 'STATUS_ACCEPTED' },
            (KLARNA_STATUS_DENIED)   => { 'StatusCode' => 'OK',  'Message' => 'STATUS_DENIED' },
            (KLARNA_STATUS_PENDING)  => { 'StatusCode' => 'OK',  'Message' => 'STATUS_PENDING' },
            (KLARNA_STATUS_ERROR)    => { 'StatusCode' => 'ERR', 'Message' => 'STATUS_ERROR' },
        };
        my $KlarnaOrderStatus = $hResponse->{'Message'}{'int'};
        $hResult = $StatusMap->{$KlarnaOrderStatus};

        _updateKlarnaOrderStatus($Order, $KlarnaOrderStatus);
    }
    else {
        $hResult = { 'StatusCode' => $hResponse->{'StatusCode'}, 'Message' => $hResponse->{'Message'} };
    }

    return $hResult;
}

#========================================================================================
# function        CheckKlarnaOrderStatusViaScheduler
# state           public
#----------------------------------------------------------------------------------------
# syntax          my $result = CheckKlarnaOrderStatusViaScheduler($Order);
#----------------------------------------------------------------------------------------
# description     Request order status updates from Klarna and update the order flags
#                  accordingly. Does not need a Servlet object. For use with the scheduler
#----------------------------------------------------------------------------------------
# input           $Order | The order to update | object
# return          $result | result of the operation
#                    - Message     Status message
#                    - StatusCode  Status code  | hash reference
#========================================================================================
sub CheckKlarnaOrderStatusViaScheduler {
    my ($Order) = @_;
    my $LineItemPaymentKlarna = $Order->get('LineItemContainer')->get('Payment');

    my $OrderStatus = DE_EPAGES::Klarna::API::KlarnaOperations::CheckOrderStatus->new( $LineItemPaymentKlarna );
    my $hResponse = $OrderStatus->call();

    my $hResult = UpdateOrderStatus($Order, $hResponse);

    return $hResult;
}

1;