#========================================================================================
# package      DE_EPAGES::Klarna::DAL::Klarna
# state        private
# description  database interface for tables:
#               <ul>
#                   <li>klarnapclass
#                   <li>klarnaperiodiccost
#                   <li>lineitempaymentklarna
#                   <li>lineitempayklarnaperiodcost
#               </ul>
#========================================================================================
package DE_EPAGES::Klarna::DAL::Klarna;
use base qw (Exporter);

use strict;
our @EXPORT = qw (
    DAL_InsertLineItemPayKlarnaPeriodCost
    DAL_DeleteLineItemPayKlarnaPeriodCost
    DAL_GetAllPKeyLineItemPayKlarnaPeriodCosts
    DAL_GetAllInfoLineItemPayKlarnaPeriodCosts
    DAL_GetInfoLineItemPayKlarnaPeriodCost
    DAL_GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID
    DAL_GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID
    DAL_GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID
    DAL_GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID
    DAL_GetPendingKlarnaOrdersForOpenShops
    DAL_GetPendingKlarnaOrdersByShopID
);

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::DataCache::API::Cache qw ( GetCache UpdateCache GetCacheSize );

my $DAL_PACKAGE = 'DE_EPAGES::Database::API::DAL';
my $DO_CACHE = 1;

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Klarna::DAL::'.$dbi->driverName.'::Klarna';
    UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

sub _DoFillInfoCache ($$) {
    my ($CacheName, $CountRows) = @_;
    my $dbi = GetCurrentDBHandle();
    my $CacheSize = GetCacheSize($dbi->cacheKey, $CacheName);
    return $CountRows < $CacheSize ? 1 : 0;
}

#========================================================================================
# function     DAL_InsertLineItemPayKlarnaPeriodCost
#----------------------------------------------------------------------------------------
# syntax       DAL_InsertLineItemPayKlarnaPeriodCost($hParameter);
#----------------------------------------------------------------------------------------
# description  insert entry in table lineitempayklarnaperiodcost
#----------------------------------------------------------------------------------------
# input        $hParameter | with following keys:
#               <ul>
#                  <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                  <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul> | reference to hash
#========================================================================================
sub DAL_InsertLineItemPayKlarnaPeriodCost ($) {
    my ($hParameter) = @_;
    _CallDriver('InsertLineItemPayKlarnaPeriodCost', $hParameter);
    if ($DO_CACHE) {
        my $dbi = GetCurrentDBHandle();
        foreach my $key (keys %$hParameter) {
            $hParameter->{$key} = $dbi->encodeSpaces($hParameter->{$key});
        }
        UpdateCache($dbi->cacheKey,'AllLineItemPayKlarnaPeriodCost');
        UpdateCache($dbi->cacheKey,'InfoLineItemPayKlarnaPeriodCost', join('<||>',
            @$hParameter{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID)}),$hParameter );
        UpdateCache($dbi->cacheKey,'LineItemPayKlarnaPeriodCostByLineItemPaymentKlarnaID',
            $hParameter->{'LineItemPaymentKlarnaID'});
        UpdateCache($dbi->cacheKey,'LineItemPayKlarnaPeriodCostByKlarnaPeriodicCostID',
            $hParameter->{'KlarnaPeriodicCostID'});
    }
    return;
}

#========================================================================================
# function     DAL_DeleteLineItemPayKlarnaPeriodCost
#----------------------------------------------------------------------------------------
# syntax       DAL_DeleteLineItemPayKlarnaPeriodCost($LineItemPaymentKlarnaID, $KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  removes an entry of table lineitempayklarnaperiodcost
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarnaID | line item payment klarna id (part of primary key) | int
# input        $KlarnaPeriodicCostID | klarna periodic cost id (part of primary key) | int
#========================================================================================
sub DAL_DeleteLineItemPayKlarnaPeriodCost ($$) {
    my ($LineItemPaymentKlarnaID, $KlarnaPeriodicCostID) = @_;
    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    my $hInfo = undef;
    if ($DO_CACHE) {
        my $InfoCache = GetCache($cacheKey,'InfoLineItemPayKlarnaPeriodCost');
        $hInfo = $InfoCache->{ join('<||>',$LineItemPaymentKlarnaID, $KlarnaPeriodicCostID)};
    }
    _CallDriver('DeleteLineItemPayKlarnaPeriodCost', $LineItemPaymentKlarnaID, $KlarnaPeriodicCostID);
    if ($DO_CACHE) {
        UpdateCache($cacheKey,'InfoLineItemPayKlarnaPeriodCost',  join('<||>',$LineItemPaymentKlarnaID,
            $KlarnaPeriodicCostID));
        UpdateCache($cacheKey,'AllLineItemPayKlarnaPeriodCost');
        UpdateCache($cacheKey,'LineItemPayKlarnaPeriodCostByLineItemPaymentKlarnaID');
        if (defined $hInfo) {
            UpdateCache($cacheKey,'LineItemPayKlarnaPeriodCostByKlarnaPeriodicCostID', $hInfo->{'KlarnaPeriodicCostID'});
        } else {
            UpdateCache($cacheKey,'LineItemPayKlarnaPeriodCostByKlarnaPeriodicCostID');
        }
    }
    return;
}

#========================================================================================
# function     DAL_GetAllPKeyLineItemPayKlarnaPeriodCosts
#----------------------------------------------------------------------------------------
# syntax       $rahLineItemPayKlarnaPeriodCostIDs = DAL_GetAllPKeyLineItemPayKlarnaPeriodCosts();
#----------------------------------------------------------------------------------------
# description  returns all primary keys of table lineitempayklarnaperiodcost
#----------------------------------------------------------------------------------------
# return       $rahLineItemPayKlarnaPeriodCostIDs | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllPKeyLineItemPayKlarnaPeriodCosts () {
    return _CallDriver('GetAllPKeyLineItemPayKlarnaPeriodCosts') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllLineItemPayKlarnaPeriodCost');
    $CacheRef->{'1'} = _CallDriver('GetAllPKeyLineItemPayKlarnaPeriodCosts') unless exists $CacheRef->{'1'};
    return [ @{$CacheRef->{'1'}} ];
}

#========================================================================================
# function     DAL_GetAllInfoLineItemPayKlarnaPeriodCosts
#----------------------------------------------------------------------------------------
# syntax       $rahLineItemPayKlarnaPeriodCostIDs = DAL_GetAllInfoLineItemPayKlarnaPeriodCosts();
#----------------------------------------------------------------------------------------
# description  returns all rows of table lineitempayklarnaperiodcost
#----------------------------------------------------------------------------------------
# return       $rahLineItemPayKlarnaPeriodCostIDs | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllInfoLineItemPayKlarnaPeriodCosts ();
sub DAL_GetAllInfoLineItemPayKlarnaPeriodCosts () {
    return _CallDriver('GetAllInfoLineItemPayKlarnaPeriodCosts') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllLineItemPayKlarnaPeriodCost');
    if (not exists $CacheRef->{'1'}) {
        my $rResultSet = _CallDriver('GetAllInfoLineItemPayKlarnaPeriodCosts');
        # make copy to primary key cache
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )} =
                @$hRow{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{'1'} = \@PrimaryKeySet;
        # make copy to info cache
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoLineItemPayKlarnaPeriodCost');
        if (_DoFillInfoCache('InfoLineItemPayKlarnaPeriodCost', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join('<||>', @$Row{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoLineItemPayKlarnaPeriodCost', @{$CacheRef->{'1'}})) {
        my @ResultSet;
        foreach my $PrimaryKey (@{$CacheRef->{'1'}}) {
            my $Info = DAL_GetInfoLineItemPayKlarnaPeriodCost($PrimaryKey->{'LineItemPaymentKlarnaID'},
                $PrimaryKey->{'KlarnaPeriodicCostID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{'1'};
    return DAL_GetAllInfoLineItemPayKlarnaPeriodCosts();
}

#========================================================================================
# function     DAL_GetInfoLineItemPayKlarnaPeriodCost
#----------------------------------------------------------------------------------------
# syntax       $hResult = DAL_GetInfoLineItemPayKlarnaPeriodCost($LineItemPaymentKlarnaID, $KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  returns one dataset
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarnaID | line item payment klarna id (part of primary key) | int
# input        $KlarnaPeriodicCostID | klarna periodic cost id (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>| reference to hash
#========================================================================================
sub DAL_GetInfoLineItemPayKlarnaPeriodCost ($$) {
    my ($LineItemPaymentKlarnaID, $KlarnaPeriodicCostID) = @_;
    return _CallDriver('GetInfoLineItemPayKlarnaPeriodCost', $LineItemPaymentKlarnaID,
        $KlarnaPeriodicCostID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef    = GetCache($dbi->cacheKey,'InfoLineItemPayKlarnaPeriodCost');
    my $CacheString =  join('<||>',$LineItemPaymentKlarnaID, $KlarnaPeriodicCostID);
    $CacheRef->{$CacheString} = _CallDriver('GetInfoLineItemPayKlarnaPeriodCost', $LineItemPaymentKlarnaID,
        $KlarnaPeriodicCostID) unless exists $CacheRef->{$CacheString};
    return undef unless defined $CacheRef->{$CacheString};
    return {%{$CacheRef->{$CacheString}}};
}

#========================================================================================
# function     DAL_GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID($LineItemPaymentKlarnaID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table lineitempayklarnaperiodcost by primary key
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarnaID | line item payment klarna id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID ($) {
    my ($LineItemPaymentKlarnaID) = @_;
    return _CallDriver('GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID',
        $LineItemPaymentKlarnaID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'LineItemPayKlarnaPeriodCostByLineItemPaymentKlarnaID');
    my $CacheString = $LineItemPaymentKlarnaID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID',
        $LineItemPaymentKlarnaID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID($LineItemPaymentKlarnaID);
#----------------------------------------------------------------------------------------
# description  get full row of table lineitempayklarnaperiodcost by primary key
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentKlarnaID | line item payment klarna id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID ($);
sub DAL_GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID ($) {
    my ($LineItemPaymentKlarnaID) = @_;
    return _CallDriver('GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID',
        $LineItemPaymentKlarnaID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'LineItemPayKlarnaPeriodCostByLineItemPaymentKlarnaID');
    my $CacheString = $LineItemPaymentKlarnaID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID',
            $LineItemPaymentKlarnaID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )} =
                @$hRow{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoLineItemPayKlarnaPeriodCost');
        if (_DoFillInfoCache('InfoLineItemPayKlarnaPeriodCost', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoLineItemPayKlarnaPeriodCost', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoLineItemPayKlarnaPeriodCost($PrimaryKey->{'LineItemPaymentKlarnaID'},
                $PrimaryKey->{'KlarnaPeriodicCostID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID($LineItemPaymentKlarnaID);
}

#========================================================================================
# function     DAL_GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID($KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table lineitempayklarnaperiodcost by index i_lipk_periodidcostid
#----------------------------------------------------------------------------------------
# input        $KlarnaPeriodicCostID | klarna periodic cost id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID ($) {
    my ($KlarnaPeriodicCostID) = @_;
    return _CallDriver('GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID',
        $KlarnaPeriodicCostID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'LineItemPayKlarnaPeriodCostByKlarnaPeriodicCostID');
    my $CacheString = $KlarnaPeriodicCostID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID',
        $KlarnaPeriodicCostID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID($KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  get full row of table lineitempayklarnaperiodcost by index i_lipk_periodidcostid
#----------------------------------------------------------------------------------------
# input        $KlarnaPeriodicCostID | klarna periodic cost id | int
# return       $rahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID ($);
sub DAL_GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID ($) {
    my ($KlarnaPeriodicCostID) = @_;
    return _CallDriver('GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID',
        $KlarnaPeriodicCostID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'LineItemPayKlarnaPeriodCostByKlarnaPeriodicCostID');
    my $CacheString = $KlarnaPeriodicCostID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID',
            $KlarnaPeriodicCostID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )} =
                @$hRow{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoLineItemPayKlarnaPeriodCost');
        if (_DoFillInfoCache('InfoLineItemPayKlarnaPeriodCost', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (LineItemPaymentKlarnaID KlarnaPeriodicCostID )});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoLineItemPayKlarnaPeriodCost', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoLineItemPayKlarnaPeriodCost($PrimaryKey->{'LineItemPaymentKlarnaID'},
                $PrimaryKey->{'KlarnaPeriodicCostID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID($KlarnaPeriodicCostID);
}

#========================================================================================
# function     DAL_GetPendingKlarnaOrdersForOpenShops();
#----------------------------------------------------------------------------------------
# syntax       $aOrderIDs = DAL_GetPendingKlarnaOrdersForOpenShops();
#----------------------------------------------------------------------------------------
# description  Returns pending klarna orderids, shopids for open shops
#----------------------------------------------------------------------------------------
# return       $aOrderIDs | list of order identifier | ref.array.integer
#========================================================================================
sub DAL_GetPendingKlarnaOrdersForOpenShops ();
sub DAL_GetPendingKlarnaOrdersForOpenShops () {
    return _CallDriver('GetPendingKlarnaOrdersForOpenShops');
}

#========================================================================================
# function     DAL_GetPendingKlarnaOrdersByShopID
#----------------------------------------------------------------------------------------
# syntax       $aOrderIDs = DAL_GetPendingKlarnaOrdersByShopID($ShopID);
#----------------------------------------------------------------------------------------
# description  Returns orderids of pending orders paid with klarna by shopid
#----------------------------------------------------------------------------------------
# input        $ShopID | shop id | int
# return       $aOrderIDs | list of order identifier | ref.array.integer
#========================================================================================
sub DAL_GetPendingKlarnaOrdersByShopID ($);
sub DAL_GetPendingKlarnaOrdersByShopID ($) {
    my ($ShopID) = @_;
    return _CallDriver('GetPendingKlarnaOrdersByShopID',$ShopID);
}

1;
