#========================================================================================
# package      DE_EPAGES::Klarna::DAL::SQL::Klarna
# state        private
# description  Generic SQL database interface for tables:
#               <ul>
#                   <li>klarnapclass
#                   <li>klarnaperiodiccost
#                   <li>lineitempaymentklarna
#                   <li>lineitempayklarnaperiodcost
#               </ul>
#========================================================================================
package DE_EPAGES::Klarna::DAL::SQL::Klarna;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);

## no critic qw(ProhibitLongLines)

my %TABLE_DEF = (
    'LineItemPayKlarnaPeriodCost' => {
        'LineItemPaymentKlarnaID' => ['lineitempaymentklarnaid','Integer'],
        'KlarnaPeriodicCostID' => ['klarnaperiodiccostid','Integer'],
    },
);

sub ErrorDefinition {
    return {
        'TableNotExists'  => 'table #Table does not exist at package DE_EPAGES::Klarna::DAL::SQL::Klarna',
        'ColumnNotExists' => 'column #Column does not exist at table #Table at package DE_EPAGES::Klarna::DAL::SQL::Klarna',
    };
}

sub TableDef {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[0];
}

sub TableType {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[1];
}

#========================================================================================
# function     InsertLineItemPayKlarnaPeriodCost
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertLineItemPayKlarnaPeriodCost($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table lineitempayklarnaperiodcost.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                  <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul> | reference to hash
#========================================================================================
sub InsertLineItemPayKlarnaPeriodCost {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO lineitempayklarnaperiodcost (lineitempaymentklarnaid, klarnaperiodiccostid) VALUES (?, ?)',
        [ @{$hParameter}{qw( LineItemPaymentKlarnaID KlarnaPeriodicCostID )} ]
    );
}

#========================================================================================
# function     DeleteLineItemPayKlarnaPeriodCost
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteLineItemPayKlarnaPeriodCost($dbi, $LineItemPaymentKlarnaID, $KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table lineitempayklarnaperiodcost.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $LineItemPaymentKlarnaID | line item payment klarna id (part of primary key) | int
# input        $KlarnaPeriodicCostID | klarna periodic cost id (part of primary key) | int
#========================================================================================
sub DeleteLineItemPayKlarnaPeriodCost {
    my $self = shift;
    my ($dbi, $LineItemPaymentKlarnaID, $KlarnaPeriodicCostID) = @_;
    $dbi->do('DELETE FROM lineitempayklarnaperiodcost WHERE lineitempaymentklarnaid=? AND klarnaperiodiccostid=?', [$LineItemPaymentKlarnaID, $KlarnaPeriodicCostID]);
    return;
}

#========================================================================================
# function     GetAllPKeyLineItemPayKlarnaPeriodCosts
#----------------------------------------------------------------------------------------
# syntax       $ahLineItemPayKlarnaPeriodCostIDs = $Package->GetAllPKeyLineItemPayKlarnaPeriodCosts($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table lineitempayklarnaperiodcost.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahLineItemPayKlarnaPeriodCostIDs | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyLineItemPayKlarnaPeriodCosts {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost', [], ['Integer', 'Integer'], [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoLineItemPayKlarnaPeriodCosts
#----------------------------------------------------------------------------------------
# syntax       $ahLineItemPayKlarnaPeriodCostIDs = $Package->GetAllInfoLineItemPayKlarnaPeriodCosts($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table lineitempayklarnaperiodcost.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahLineItemPayKlarnaPeriodCostIDs | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoLineItemPayKlarnaPeriodCosts {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost', [],
        ['Integer', 'Integer'],
        [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]
    );
}

#========================================================================================
# function     GetInfoLineItemPayKlarnaPeriodCost
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoLineItemPayKlarnaPeriodCost($dbi, $LineItemPaymentKlarnaID, $KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $LineItemPaymentKlarnaID | line item payment klarna id (part of primary key) | int
# input        $KlarnaPeriodicCostID | klarna periodic cost id (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>| reference to hash
#========================================================================================
sub GetInfoLineItemPayKlarnaPeriodCost {
    my $self = shift;
    my ($dbi, $LineItemPaymentKlarnaID, $KlarnaPeriodicCostID) = @_;
    return $dbi->execute('SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost WHERE lineitempaymentklarnaid=? AND klarnaperiodiccostid=?',
        [$LineItemPaymentKlarnaID, $KlarnaPeriodicCostID],
        ['Integer', 'Integer'],
        [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]
    )->[0];
}

#========================================================================================
# function     GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID($dbi, $LineItemPaymentKlarnaID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table lineitempayklarnaperiodcost by primary key.
#               Uses the index pk_lineitempayklarnaperiodcost.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $LineItemPaymentKlarnaID | line item payment klarna id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID {
    my $self = shift;
    my ($dbi, $LineItemPaymentKlarnaID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'lineitempaymentklarnaid = ?';
    unshift @Parameter, $LineItemPaymentKlarnaID;

    my $sql = 'SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer'], [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID($dbi, $LineItemPaymentKlarnaID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table lineitempayklarnaperiodcost by primary key.
#               Uses the index pk_lineitempayklarnaperiodcost.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $LineItemPaymentKlarnaID | line item payment klarna id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoLineItemPayKlarnaPeriodCostsByLineItemPaymentKlarnaID {
    my $self = shift;
    my ($dbi, $LineItemPaymentKlarnaID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'lineitempaymentklarnaid = ?';
    unshift @Parameter, $LineItemPaymentKlarnaID;

    my $sql = 'SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'Integer'],
        [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]
    );
}

#========================================================================================
# function     GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID($dbi, $KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table lineitempayklarnaperiodcost by index i_lipk_periodidcostid.
#               Uses the index i_lipk_periodidcostid.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $KlarnaPeriodicCostID | klarna periodic cost id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID {
    my $self = shift;
    my ($dbi, $KlarnaPeriodicCostID) = @_;
    my @Parameter = ($KlarnaPeriodicCostID);
    my $sql = 'SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost WHERE klarnaperiodiccostid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer'], [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID($dbi, $KlarnaPeriodicCostID);
#----------------------------------------------------------------------------------------
# description  get full row of table lineitempayklarnaperiodcost by index i_lipk_periodidcostid
#               Uses the index i_lipk_periodidcostid.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $KlarnaPeriodicCostID | klarna periodic cost id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>LineItemPaymentKlarnaID - line item payment klarna id (part of primary key)- int
#                   <li>KlarnaPeriodicCostID - klarna periodic cost id (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoLineItemPayKlarnaPeriodCostsByKlarnaPeriodicCostID {
    my $self = shift;
    my ($dbi, $KlarnaPeriodicCostID) = @_;
    return $dbi->execute('SELECT lineitempaymentklarnaid, klarnaperiodiccostid FROM lineitempayklarnaperiodcost WHERE klarnaperiodiccostid = ?',
        [$KlarnaPeriodicCostID],
        ['Integer', 'Integer'],
        [qw(LineItemPaymentKlarnaID KlarnaPeriodicCostID)]
    );
}

#========================================================================================
# function     GetPendingKlarnaOrdersForOpenShops();
#----------------------------------------------------------------------------------------
# syntax       $aOrderIDs = $Package->GetPendingKlarnaOrdersForOpenShops($dbi);
#----------------------------------------------------------------------------------------
# description  Returns pending klarna orderids, shopids for open shops
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $aOrderIDs | list of order identifier | ref.array.integer
#========================================================================================
sub GetPendingKlarnaOrdersForOpenShops {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute(<<END_SQL, [], ['Integer'] );
        select co.orderid from lineitempaymentklarna l
        inner join object lo on lo.objectid=l.lineitempaymentklarnaid
        inner join lineitem li on l.lineitempaymentklarnaid=li.lineitemid
        inner join lineitemcontainer lc on li.lineitemcontainerid = lc.lineitemcontainerid
        inner join customerorder co on co.lineitemcontainerid=lc.lineitemcontainerid and co.pendingon is not null
        inner join shop s on co.shopid=s.shopid and s.closedbyprovider=0
END_SQL
    return [map { $_->[0] } @$dbResult];
}

#========================================================================================
# function     GetPendingKlarnaOrdersByShopID
#----------------------------------------------------------------------------------------
# syntax       $aOrderIDs = $Package->GetPendingKlarnaOrdersByShopID($dbi,$ShopID);
#----------------------------------------------------------------------------------------
# description  Returns orderids of pending orders paid with klarna by shopid
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ShopID | shop id | int
# return       $aOrderIDs | list of order identifier | ref.array.integer
#========================================================================================
sub GetPendingKlarnaOrdersByShopID {
    my $self = shift;
    my ($dbi, $ShopID) = @_;

    my $dbResult = $dbi->execute(<<END_SQL, [$ShopID], ['Integer'] );
        select co.orderid from lineitempaymentklarna l
        inner join object lo on lo.objectid=l.lineitempaymentklarnaid and lo.siteid=?
        inner join lineitem li on l.lineitempaymentklarnaid=li.lineitemid
        inner join lineitemcontainer lc on li.lineitemcontainerid = lc.lineitemcontainerid
        inner join customerorder co on co.lineitemcontainerid=lc.lineitemcontainerid and co.pendingon is not null
        inner join shop s on co.shopid=s.shopid
END_SQL
    return [map { $_->[0] } @$dbResult];
}

1;
