#==============================================================================
# package      DE_EPAGES::MediaGallery::API::Object::MediaGallery
# state        public
# description  Interface for the MediaGallery folder.
#==============================================================================
package DE_EPAGES::MediaGallery::API::Object::MediaGallery;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use LWP::UserAgent;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( CompareMD5File
                                    CreateDirectory DeleteDirectory DeleteFile
                                    ExistsDirectory ExistsFile fsplit
                                    GetDirectorySize GetFileSize GetSimilarFileName IsInPath
                                    GetFileContent MoveFile RenameDirectory WriteFile
                                    GetFileNames );
use DE_EPAGES::Core::API::Image qw ( ResizeImage CorrectImage TestImageFormat TestImage );
use DE_EPAGES::Core::API::PerlTools qw ( GetMD5Digest GenerateShortGUID );
use DE_EPAGES::Core::API::TestParameter qw (TestInteger TestString);
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename Unicode2DirectoryName Unicode2Path );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );


sub ErrorDefinition {
    return {
        PermissionDenied        => 'Permission denied to access file system (#Access) outside allowed path (#Path)',
        FileNotFound            => 'The file #File was not found on the file system',
        FileAlreadyExists       => 'File #File already exists',
        DirectoryNotFound       => 'The directory #Directory was not found on the file system',
        DirectoryAlreadyExists  => 'Directory #Directory already exists',
        InvalidChars            => 'New directory or file name #Name contains invalid chars',
        UnsupportedUrlFormat    => 'Only absolute URL\'s with scheme can be handled: #Url',
        WebGetFailed            => 'Could not web get #Url, reason: #Reason',
    };
}

sub new {
    my $self = shift->SUPER::new(@_);
    $self->{'_DirectorySize_'} = -s $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/MediaGallery';
    return $self;
}

#========================================================================================
# function     directorySize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DirectorySize = $MediaGallery->directorySize;
#----------------------------------------------------------------------------------------
# description  Returns the default value of a directory to take in count for feature
#               GallerySize.
#----------------------------------------------------------------------------------------
# return       $DirectorySize | directory size in bytes | int
#========================================================================================
sub directorySize { shift->{'_DirectorySize_'}; }

#========================================================================================
# function     testGallerySize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->testGallerySize;
#----------------------------------------------------------------------------------------
# description  test if size of the gallery is to large.
#========================================================================================
sub testGallerySize {
    my $self = shift;
    $self->getSite()->featureTestValue('GallerySize');
    return;
}

#========================================================================================
# function     increaseGallerySize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->increaseGallerySize($Value);
# example      $MediaGallery->increaseGallerySize(1024);
#----------------------------------------------------------------------------------------
# description  Increases the size of the gallery by a particular amount of bytes.
#----------------------------------------------------------------------------------------
# input        $Value | value in bytes to add to the gallery size | int
#========================================================================================
sub increaseGallerySize {
    my $self = shift;
    my ($Bytes) = @_;
    TestInteger('Bytes',$Bytes);
    if( $Bytes != 0 ) {
        my $KBytes = sprintf '%.0f', $Bytes/1024;
        $self->getSite()->featureIncrementValue('GallerySize', $KBytes);
    }
    return;
}

#========================================================================================
# function     relativePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RelativePath = $MediaGallery->relativePath($AbsolutePath);
# example      $RelativePath = $MediaGallery->relativePath("D:/epages/.../MediaGallery/file.txt");
#               # $RelativePath eq "MediaGallery/file.txt"
#----------------------------------------------------------------------------------------
# description  Removes the public path part of the media gallery from an absolute path.
#----------------------------------------------------------------------------------------
# input        $AbsolutePath | absolute path to a file or directory beneath the media
#               gallery root | string
# return       $RelativePath | path relative to the media gallery root | string
#========================================================================================
sub relativePath {
    my $self = shift;
    my ($AbsolutePath) = @_;

    my $PublicPath = $self->get('PublicPath');
    # normalize paths
    $AbsolutePath =~ s/[\\\/]+/\//g;
    $PublicPath =~ s/[\\\/]+/\//g;
    $AbsolutePath =~ s/^\Q$PublicPath\E\/*//;
    return $AbsolutePath;
}

#========================================================================================
# function     absolutePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AbsolutePath = $MediaGallery->absolutePath($RelativePath);
# example      $AbsolutePath = $MediaGallery->absolutePath("file.txt");
#               # $AbsolutePath eq "D:/epages/.../MediaGallery/file.txt"
#----------------------------------------------------------------------------------------
# description  Adds the public path part of the media gallery to an relative path.
#----------------------------------------------------------------------------------------
# return       $RelativePath | path relative to the media gallery root | string
# input        $AbsolutePath | absolute path to a file or directory beneath the media
#               gallery root | string
#========================================================================================
sub absolutePath {
    my $self = shift;
    my ($RelativePath) = @_;
    my $PublicPath = $self->get('PublicPath');
    my $AbsolutePath = $PublicPath;
    $AbsolutePath .= '/'.$RelativePath if defined $RelativePath and ($RelativePath ne '');

    Error('PermissionDenied', { 'Access' => $AbsolutePath, 'Path'=>$PublicPath }) unless IsInPath($AbsolutePath, $PublicPath);
    return $AbsolutePath;
}

#========================================================================================
# function     addUploadedFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->addUploadedFile($Directory, $hFiles, $FileName, $OverwriteFlag);
#----------------------------------------------------------------------------------------
# description  Adds files to the media gallery as given by $Servlet->form.
#----------------------------------------------------------------------------------------
# input        $Directory | a media gallery's sub directory to add the files to | string
# input        $rContent | references to the uploaded file content as given by
#               $Servlet->form->files  | hash.string
# input        $FileName | corresponding file name of the uploaded file | hash.string
# input        $hParams  | see "insertFile" (optional) | hash.ref
# return       $UploadedFile | returns the name of the uploaded file | array.string
#========================================================================================
sub addUploadedFile {
    my $self = shift;
    my ($Directory, $rContent, $FileName, $hParams) = @_;

    my $Shop = $self->getSite();
    my (undef, $File, $Extension) = fsplit($FileName);

    $Directory .= '/' if $Directory ne '' and $Directory !~ /\/$/;

    TriggerHook('UI_BeforeUploadWriteFile', {
        'Shop'              => $Shop,
        'FileName'          => $File.$Extension,
        'FileContent'       => $rContent,
    });

    my $UploadedFile = $self->insertFile($Directory.$File.$Extension, $rContent, $hParams);

    # Trigger Hook (e.q. statistic)
    TriggerHook('UI_AfterUploadWriteFile',{
        'Shop'              => $Shop,
        'FileNameCustomer'  => $File.$Extension,
        'FileNameEPages'    => $File.$Extension,
        'FilePath'          => $Directory
    });

    return $UploadedFile;
}

#========================================================================================
# function     copyFileFromImageDb
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->copyFileFromImageDb($SourceFile, $TargetDirectory);
# syntax       $MediaGallery->copyFileFromImageDb(
#                   "http://host.com/imagedb/image.jpg",
#                   "MyPictures/SubDir");
#----------------------------------------------------------------------------------------
# description  Copies a file from the global image db to the user's media gallery. Uses
#               webget if needed.
#----------------------------------------------------------------------------------------
# input        $SourceFile | the source file in the image db, either as a local file
#               name or as uri (starting with http:// or https://) | string
# input        $TargetDirectory | target directory in the user's media gallery to copy
#               the file to. If the value is undefined the main directory is used
#               | string
# return       $NewFile | return the filepath in mediagallery | string
#========================================================================================
sub copyFileFromImageDb {
    my $self = shift;
    my ($SourceFile, $TargetDirectory) = @_;

    $TargetDirectory = '' if not defined $TargetDirectory;
    TestString('SourceFile', $SourceFile);
    TestString('TargetDirectory', $TargetDirectory) if $TargetDirectory ne '';

    my $System = LoadRootObject();
    my $ImageDbPublicPath = $System->get('ImageDatabasePublicPath');
    my $ImageDbWebPath = $System->get('ImageDatabaseWebPath');
    my (undef, $Name, $Ext) = fsplit($SourceFile);
    my $NewFileName;
    if ($SourceFile =~ s/^\Q$ImageDbWebPath\E/$ImageDbPublicPath/) {
        # local
        $NewFileName = $self->insertFile($TargetDirectory.'/'.$Name.$Ext, GetFileContent($SourceFile, {binary=>1}), { overwrite => 0 });
    } elsif ($SourceFile =~ /^htt(ps|p):\/\//i) {
        # remote
        # webget
        my $UserAgent = LWP::UserAgent->new;
        my $Response = $UserAgent->get($SourceFile);
        if ($Response->is_error || $Response->code != 200) {
            Error('WebGetFailed', { Url => $SourceFile, Reason => $Response->status_line });
        }
        # incl feature test
        $NewFileName = $self->insertFile($TargetDirectory.'/'.$Name.$Ext, \$Response->content, { overwrite => 0 });
    } else {
        Error('UnsupportedUrlFormat', { Url => $SourceFile });
    }
    return $TargetDirectory.'/'.$NewFileName;
}

#========================================================================================
# function     existsFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->existsFile($RelativePath);
# example      $MediaGallery->existsFile('MyFiles/file.txt');
#----------------------------------------------------------------------------------------
# description  Returns whether a file exists in the user's media gallery.
#----------------------------------------------------------------------------------------
# input        $RelativePath | a file in the media gallery relative to the media gallery
#               root | string
# return       $ExistsFile | exists file | boolean
#========================================================================================
sub existsFile {
    my $self = shift;
    my ($Path) = @_;

    return ExistsFile($self->absolutePath($Path));
}

#========================================================================================
# function     existsDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->existsDirectory($RelativePath);
# example      $MediaGallery->existsDirectory('MyFiles/SomeDir');
#----------------------------------------------------------------------------------------
# description  Returns whether a directory exists in the user's media gallery.
#----------------------------------------------------------------------------------------
# input        $RelativePath | a directory in the media gallery relative to the media
#               gallery root | string
# return       $ExistsFile | exists file | boolean
#========================================================================================
sub existsDirectory {
    my $self = shift;
    my ($Path) = @_;

    return ExistsDirectory($self->absolutePath($Path));
}

#========================================================================================
# function     insertFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->insertFile($FileName, $rContent, $hFlags);
# example      $MediaGallery->insertFile('MyFiles/SomeDir/File.ext', $hForm->{'File'}, {overwrite=>1});
#----------------------------------------------------------------------------------------
# description  Inserts a file in the media gallery.
#----------------------------------------------------------------------------------------
# input        $FileName | filename includes path | string
# input        $rContent | content of file | ref.string
# input        $hFlags | some options:
#                           <ul>
#                             <li>overwrite - if true then the file will be overwritten, if it already
#                                   exists, otherwise a similar file name will be found, that
#                                   does not exist - boolean (default 0)</li>
#                             <li>increaseGallerySize - if gallery size should be increased - boolean (default 1)</li>
#                             <li>createThumbnail - if file is an image create  thumbnail - boolean (default 1)</li>
#                             <li>hiddenDirectory - if directory is hidden allow ',' in Path - boolean (default 0)</li>
#                             <li>forceUpload - true if the file should be uploaded even if the already exists
#                                 (MD5Checksum) - booelean (default 0)</li>
#                           </ul> (optional)
# return       $FileName | the filename (with path relative to the media gallery root)
#               of the new created file | string
#========================================================================================
sub insertFile {
    my $self = shift;
    my ($FileName, $rContent, $hFlags) = @_;

    $self->testGallerySize;
    $hFlags->{'overwrite'}           = 0 unless defined $hFlags->{'overwrite'};
    $hFlags->{'increaseGallerySize'} = 1 unless defined $hFlags->{'increaseGallerySize'};
    $hFlags->{'createThumbnail'}     = 1 unless defined $hFlags->{'createThumbnail'};
    $hFlags->{'hiddenDirectory'}     = 0 unless defined $hFlags->{'hiddenDirectory'};
    $hFlags->{'forceUpload'}         = 0 unless defined $hFlags->{'forceUpload'};

    # create directory
    my ($Path, $Name, $Extension) = fsplit($FileName);

    my $AllowedChars = $hFlags->{'hiddenDirectory'} ? ',' : undef;
    $Path = Unicode2Path($Path, $AllowedChars);
    $self->createDirectory($Path, undef, {'hiddendirectory' => $hFlags->{'hiddenDirectory'}}) unless $self->existsDirectory($Path);

    # make file name safe
    $Name = Unicode2Filename($Name,'.,');

    my $MediaGalleryRoot = $self->get('PublicPath');
    my $TargetFile = $self->absolutePath($Path).'/'.$Name.$Extension;
    Error('PermissionDenied', { Access => $TargetFile, Path => $MediaGalleryRoot }) unless IsInPath($TargetFile, $MediaGalleryRoot);

    my $OldFileSize = 0;
    my $MediaGalleryFileName = substr($TargetFile, length($MediaGalleryRoot) + 1);
    if (ExistsFile($TargetFile)) {
        $OldFileSize = GetFileSize($TargetFile);
        if (!$hFlags->{'overwrite'}) {
            # check hashes to see, if the files are identic
            # if not, then create a similar file name
            my $FileSize = length $$rContent;
            if ($OldFileSize == $FileSize &&
                CompareMD5File($TargetFile, GetMD5Digest($rContent)) && !$hFlags->{'forceUpload'}) {
                return $MediaGalleryFileName;
            }
            $TargetFile = GetSimilarFileName($TargetFile); # add some numbers
            # regenerate again
            $MediaGalleryFileName = substr($TargetFile, length($MediaGalleryRoot) + 1);
        }
    }

    my $TempFile = $self->absolutePath($Path).'/'.GenerateShortGUID().$Extension;
    WriteFile($TempFile, $rContent, { binary=>1 });
    # look if it's an image
    eval {
        TestImageFormat('UploadedImage', $TempFile);
    };
    if(!ExistsError()) {
        # looks like an image, try to open
        eval {
            TestImage($TempFile);
        };
        if(ExistsError()) {
            DeleteFile($TempFile);
            my $Err = GetError();
            Error( $Err->code, {
                'File' => $Err->{'Vars'}->{'File'},
                'ImageMagickError' => $Err->{'Vars'}->{'ImageMagickError'}
            });
            return;
        }
        MoveFile($TempFile, $TargetFile);
        CorrectImage($TargetFile);
        $self->createThumbnail($MediaGalleryFileName) if $hFlags->{'createThumbnail'} ;
    } else {
        MoveFile($TempFile, $TargetFile);
    }
    if( $hFlags->{'increaseGallerySize'} ) {
        my $NewFileSize = GetFileSize($TargetFile) - $OldFileSize;
        $self->increaseGallerySize($NewFileSize);
    }

    return $MediaGalleryFileName;
}


#========================================================================================
# function     createDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->createDirectory($Path);
# syntax       $MediaGallery->createDirectory($Name, $BaseDir);
# example      $MediaGallery->createDirectory('MyFiles/SomeDir/NewDir');
# example      $MediaGallery->createDirectory('NewDir', 'MyFiles/SomeDir');
# example      $MediaGallery->createDirectory('NewDir', 'MediaGallery/MyFiles/SomeDir');
#----------------------------------------------------------------------------------------
# description  Creates a directory in the media gallery.
#----------------------------------------------------------------------------------------
# input        $TargetDirectory | name or path of the directory to create. | string
# input        $BaseDirectory | optional, a directory in the media gallery relative to
#               the media gallery root (base path for $TargetDirectory) | string
# input        $hFlags | some options:
#                   usebasedirectory => set to 1 to force the use of the second method
#                   parameter for the base directory (optional, default is 0)
#                   renamedirectory => rename directory if the $TargetDirectory exists.
#                   hiddendirectory => start directory name with ',' to create hidden
#                                      directory (default is 0)
#========================================================================================
sub createDirectory {
    my $self = shift;
    my ($TargetDirectory, $BaseDirectory, $hFlags) = @_;
    $hFlags //= {};

    $self->testGallerySize;

    my $CleanedDirectory;
    my $CleanedDirectoryPath;

    # clear only new dir name, not entire basedir
    if(defined $BaseDirectory or $hFlags->{'usebasedirectory'}) {
        # append slash if not present
        $BaseDirectory = '' if not defined $BaseDirectory;
        $BaseDirectory .= '/' if $BaseDirectory !~ /.*[\/|\\]$/;
        $CleanedDirectory = Unicode2DirectoryName($TargetDirectory);
        $CleanedDirectory = ','.$CleanedDirectory if $hFlags->{'hiddendirectory'};
        if (!$CleanedDirectory) {
            Error('InvalidChars', { Name => $CleanedDirectory });
        }
        $CleanedDirectoryPath = $BaseDirectory . $CleanedDirectory;
    } else {
        $CleanedDirectory = Unicode2Path($TargetDirectory);
        if (!$CleanedDirectory) {
            Error('InvalidChars', { Name => $CleanedDirectory });
        }
        if ($hFlags->{'hiddendirectory'}) {
            my ($Path, $Name, undef) = fsplit($CleanedDirectory);
            $CleanedDirectory = $Path.','.$Name;
        }
        $CleanedDirectoryPath = $CleanedDirectory;
    }

    my $AbsolutePath = $self->absolutePath($CleanedDirectoryPath);
    if(ExistsDirectory($AbsolutePath)) {
        if($hFlags->{'renamedirectory'}) {
            $AbsolutePath = GetSimilarFileName($AbsolutePath);
            my (undef, $NewDirectoryName, undef) = fsplit($AbsolutePath);
            $CleanedDirectory = $NewDirectoryName;
        } else {
            Error( 'DirectoryAlreadyExists', {
                'Directory' => $AbsolutePath,
                'CleanedName' => $CleanedDirectory,
            });
        }
    }

    # actual creation
    $self->increaseGallerySize($self->directorySize * CreateDirectory($AbsolutePath, { recursive => 1 }));

    # return only name of the new dir, stripped slashes
    return $CleanedDirectory;
}

#========================================================================================
# function     deleteFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->deleteFile($FileName, $hParams);
# example      $MediaGallery->deleteFile('MyFiles/file.txt');
# example      $MediaGallery->deleteFile('MyFiles/file.txt', {'DeleteEmptyDirectory' => 1});
#----------------------------------------------------------------------------------------
# description  Deletes a file in the media gallery and may delete the parent folder, if
#               it becomes empty. It may delete thumbnails and decrease the gallery size.
#----------------------------------------------------------------------------------------
# input        $FileName | a file in the media gallery relative to the media gallery root
#               | string
# input        $hParams | <ul>
#                            <li>DeleteEmptyDirectory | if directory becomes empty, then it may be deleted;
#               used for preview images/directories (optional) | boolean </li>
#                            <li>DecreaseGallerySize | if gallery size should be decreased
#                                (optional - default 1) | boolean</li>
#                            <li>DeleteThumbnail | if thumbnail schould be deleted
#                                (optional - default 1) | boolean</li>
#                          </ul> | hash.ref
#========================================================================================
sub deleteFile {
    # delete file
    my $self = shift;
    my ($FileName, $hParams) = @_;
    $hParams->{'DeleteEmptyDirectory'} //= 0;
    $hParams->{'DecreaseGallerySize'} //= 1;
    $hParams->{'DeleteThumbnail'} //= 1;

    my $File = $self->absolutePath($FileName);
    my ($Directory) = fsplit($File);
    return unless ExistsDirectory($Directory);
    if (ExistsFile($File)) {
        my $FileSize = GetFileSize($File);
        DeleteFile($File);
        $self->increaseGallerySize(-$FileSize) if $hParams->{'DecreaseGallerySize'};
        # EPG-10961: check if same extension
        # EPG-13475: only ",mgt" of THIS file
        $self->deleteThumbnail($File) if $hParams->{'DeleteThumbnail'};
    }

    my $Files = GetFileNames($Directory);
    if ($hParams->{'DeleteEmptyDirectory'} and not scalar @$Files) {
        # delete empty directory
        $self->deleteDirectory(substr($Directory, length ($self->get('PublicPath')) + 1));
    }
    return;
}

#========================================================================================
# function     deleteDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->deleteDirectory($Directory);
# example      $MediaGallery->deleteDirectory('MyFiles/SomeDir');
#----------------------------------------------------------------------------------------
# description  Deletes a directory in the media gallery.
#----------------------------------------------------------------------------------------
# input        $Directory | a file in the media gallery relative to the media gallery
#               root | string
#========================================================================================
sub deleteDirectory {
    # create directory
    my $self = shift;
    my ($Directory) = @_;

    $Directory = $self->absolutePath($Directory);
    return unless ExistsDirectory($Directory);

    my $DirectorySize = GetDirectorySize($Directory);
    DeleteDirectory($Directory, { recursive => 1 });
    $self->increaseGallerySize(-$DirectorySize);
    return;
}

#========================================================================================
# function     moveFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->moveFile($File, $NewDirectory);
# example      $MediaGallery->moveFile('MyFiles/OldDir/file.txt', 'MyFiles/NewDir');
#----------------------------------------------------------------------------------------
# description  Moves a file in the media gallery to another directory.
#----------------------------------------------------------------------------------------
# input        $File | a file in the media gallery relative to the media gallery
#               root | string
# input        $NewDirectory | a directory in the media gallery relative to the media gallery
#               root | string
#========================================================================================
sub moveFile {
    my $self = shift;
    my ($File, $NewDirectory) = @_;

    my $SourceFile = $self->absolutePath($File);
    my $TargetFile = $self->absolutePath($NewDirectory);

    # append top directory name to target directory
    my @Parts = fsplit($SourceFile);
    $TargetFile .= '/' . $Parts[1].$Parts[2];

    if (ExistsFile($TargetFile)) {
        Error('FileAlreadyExists', { 'File' => $TargetFile });
    }
    MoveFile($SourceFile, $TargetFile);
    return;
}

#========================================================================================
# function     renameFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->renameFile($File, $NewFileName);
# example      $MediaGallery->renameFile('OldDir/MyFiles/oldfile.txt', 'newfile.txt');
#----------------------------------------------------------------------------------------
# description  Renames a file in the media gallery.
#----------------------------------------------------------------------------------------
# input        $File | a files in the media gallery relative to the media gallery
#               root | string
# input        $NewFileName | the new file name (without any path) | string
#========================================================================================
sub renameFile {
    # rename file
    my $self = shift;
    my ($File, $NewFileName) = @_;

    # make valid
    $NewFileName = Unicode2Filename($NewFileName, '.');

    # add path to target file name
    my $SourceFile = $self->absolutePath($File);
    my @Parts = fsplit($SourceFile);
    my $TargetFile = $Parts[0].'/'.$NewFileName;

    if (ExistsFile($TargetFile)) {
        Error('FileAlreadyExists', { 'File' => $TargetFile });
    }
    MoveFile($SourceFile, $TargetFile);
    #my ($SourcePath, $SourceName, $SourceExtension) = fsplit($SourceFile);
    #my $SourceMatch = $SourcePath.'/'.$SourceName;
    #my ($TargetPath, $TargetName, $TargetExtension) = fsplit($TargetFile);
    #my $TargetMatch = $TargetPath.'/'.$TargetName;

    #foreach my $SourceFile (@{GetFileNames($SourcePath)}) {
    #    my $TargetFile = $SourceFile;
    #    MoveFile($SourceFile, $TargetFile) if $TargetFile =~ s/^\Q$SourceMatch\E/$TargetMatch/;
    #}
    return $NewFileName;
}

#========================================================================================
# function     renameDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->renameDirectory($Directory, $NewDirectoryName);
# example      $MediaGallery->renameDirectory('MyFiles/OldDir', 'NewDir');
#----------------------------------------------------------------------------------------
# description  Renames a directory in the media gallery.
#----------------------------------------------------------------------------------------
# input        $Directory | a directory in the media gallery relative to the media gallery
#               root | string
# input        $NewDirectoryName | the new directory name (without any path) | string
#========================================================================================
sub renameDirectory {
    # rename directory
    my $self = shift;
    my ($Directory, $NewDirectoryName) = @_;

    # make valid
    my $CleanNewDirectoryName = Unicode2DirectoryName($NewDirectoryName);

    if ($CleanNewDirectoryName =~ /[\\\/]/) {
        Error('InvalidChars', { Name => $CleanNewDirectoryName });
    }

    my $SourceDirectory = $self->absolutePath($Directory);
    my @Parts = fsplit($SourceDirectory);

    my $TargetDirectory = $Parts[0].'/'.$CleanNewDirectoryName;

    if (ExistsDirectory($TargetDirectory) && ($SourceDirectory ne $TargetDirectory)) {
        Error( 'DirectoryAlreadyExists', {
            'Directory' => $TargetDirectory,
            'CleanedName' => $TargetDirectory,
        });
    }

    unless (RenameDirectory($SourceDirectory, $TargetDirectory)) {
        Error('DirectoryNotFound', { Directory => $SourceDirectory });
    }
    return $CleanNewDirectoryName;
}

#========================================================================================
# function     resizeImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MediaGallery->resizeImage($SourceFile, $TargetDirectory, $Width, $Height, $Quality);
# example      $MediaGallery->resizeImage(
#                   'MyFiles/image.jpg',
#                   'MyFiles/Thumbnails',
#                   100,
#                   80,
#                   85);
#----------------------------------------------------------------------------------------
# description  Resizes an image file in the media gallery and writes the results to a
#               specified directory in the media gallery.
#----------------------------------------------------------------------------------------
# input        $SourceFile | a file in the media gallery relative to the media gallery
#               root | string
# input        $TargetFile | absolute file name of the target file | string
# input        $Width | maximum width of the resized image in pixels | int
# input        $Height | maximum height of the resized image in pixels | int
# input        $Quality | quality of the resized image (if applies), 1..100
#               (optional = DEFAULT_JPEG_QUALITY) | int
#========================================================================================
sub resizeImage {
    my $self = shift;
    my ($SourceFile, $TargetFile, $Width, $Height, $Quality) = @_;

    $self->testGallerySize;

    my ($Path) = fsplit($TargetFile);
    $self->createDirectory($Path, '') unless $self->existsDirectory($Path);

    Error('FileNotFound', { File => $SourceFile }) unless $self->existsFile($SourceFile);

    my $Target = $self->absolutePath($TargetFile);
    my $OldFileSize = ExistsFile($Target) ? GetFileSize($Target) : 0;

    ResizeImage($self->absolutePath($SourceFile), $Target, $Width, $Height, $Quality);

    $self->increaseGallerySize(GetFileSize($Target) - $OldFileSize);
    return;
}

#========================================================================================
# function     gallery2attributeFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AttributeFile = $GalleryImage->gallery2attributeFileName($GalleryFile);
#----------------------------------------------------------------------------------------
# description  Adds MediaGallery/ in front of filename.
#----------------------------------------------------------------------------------------
# return       $GalleryFile | 'Dir/File.ext' | string
# input        $AttributeFile | 'MediaGallery/Dir/File.ext'| string
#========================================================================================
sub gallery2attributeFileName {
    my $self = shift;
    local $_ = shift;
    return 'MediaGallery/'.$_;
}

#========================================================================================
# function     previewPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GalleryFile = $GalleryImage->attribute2galleryFileName($AttributeFile);
#----------------------------------------------------------------------------------------
# description  Removes MediaGallery/ in front of filename.
#----------------------------------------------------------------------------------------
# input        $AttributeFile | 'MediaGallery/Dir/File.ext'| string
# return       $GalleryFile | 'Dir/File.ext' | string
#========================================================================================
sub attribute2galleryFileName {
    my $self = shift;
    local $_ = shift;
    s/^MediaGallery[\/\\]//;
    return $_;
}

#========================================================================================
# function     previewPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PreviewPath = $GalleryImage->previewPath($FileName, $PreviewType);
#----------------------------------------------------------------------------------------
# description  Adds ',' $PreviewType to original filename.
#----------------------------------------------------------------------------------------
# input        $FileName | original filename | string
# input        $PreviewType | preview type | string
# return       $PreviewPath | attribute image name | string
#========================================================================================
sub previewPath {
    my $self = shift;
    my ($FileName, $PreviewType) = @_;
    my ($Path, $Name, $Extension) = fsplit($FileName);
    return join('',$Path,'/',$Name,',',$PreviewType, $Extension);
}

#========================================================================================
# function     createThumbnail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GalleryImage->createThumbnail($ImageFile);
#----------------------------------------------------------------------------------------
# description  Creates an thumbnail for display on mediagallery.
#----------------------------------------------------------------------------------------
# input        $ImageFile | original filename | string
#========================================================================================
sub createThumbnail {
    my $self = shift;
    my ($ImageFile) = @_;

    return if $ImageFile =~ /,/ or GetFileSize($self->absolutePath($ImageFile)) < 20480;
    $self->resizeImage($ImageFile, $self->previewPath($ImageFile, 'mgt'), 48, 48, 85, 1);

    return;
}

#========================================================================================
# function     deleteThumbnail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GalleryImage->deleteThumbnail($ImageFile);
#----------------------------------------------------------------------------------------
# description  Deletes an thumbnail of the given mediagallery file.
#----------------------------------------------------------------------------------------
# input        $ImageFile | original filename (relative path) | string
#========================================================================================
sub deleteThumbnail {
    my $self = shift;
    my ($ImageFile) = @_;

    my $ThumbnailFile = $self->previewPath($ImageFile, 'mgt');
    return if !ExistsFile($ThumbnailFile);

    DeleteFile($ThumbnailFile);
    return;
}

1;
