use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw(GetAllInfoCartridges);
use DE_EPAGES::Core::API::File qw ( GetFileNames );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw ( ImportAttributes );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use Getopt::Long;

#========================================================================================
# function     ImportStrings
# state        private
#----------------------------------------------------------------------------------------
# syntax       ImportStrings($SourceDir, $SourceLang)
#----------------------------------------------------------------------------------------
# description  Imports localized strings from XML files into the database.
#========================================================================================
sub ImportStrings {

    my ($hParams) = @_;
    my $SourceDir =  $hParams->{'SourceDir'};
    my $SourceLang = $hParams->{'SourceLang'};
    my $Cartridges = $hParams->{'Cartridges'};

    #get all needed files
    my $aFiles;

    if($SourceDir and not $Cartridges) {
        # use files from all Cartridges
        $aFiles = GetFileNames( $SourceDir, {'recursive' => 1} );
    }
    else {
        #use only files from installed cartridges
        # build the list of installed cartridges
        my $ahCartridges = GetAllInfoCartridges();
        foreach my $hCartridge (@$ahCartridges) {
            my $CartridgeDir = $hCartridge->{'Package'};
            $CartridgeDir =~ s/::/\//;
            $CartridgeDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$CartridgeDir;
            push @$aFiles,@{GetFileNames( $CartridgeDir, {'recursive' => 1} )};
        }
    }

    #import
    my $aWarnings = BlockWarnings( sub {
        foreach my $FilePath ( @{$aFiles} ) {
            next if $FilePath !~ /\/Database\/XML\//;
            if ($FilePath =~ /\.$SourceLang\.xml$/) {
                # avoid error for dialects (en-US, pt-BR, es-MX)
                next if $FilePath =~ m/\/Languages\.$SourceLang\.xml$/;
                print "import file: $FilePath\n";
                ImportAttributes( $FilePath );
            }
        }
    });

    #print WarningList
    foreach my $Warning (@$aWarnings) {
        my $Message = $Warning->shortMessage;
        $Message=~ s/^\s*(.*?)\s*$/$1/;
        print  $Warning->vars->{'FileName'}.' '.$Message;
        print "\n";
    }
}

#========================================================================================
# function     Main
# state        private
#========================================================================================
sub Main {
    # set defaults
    my $StoreName;
    my $Password;
    my $Cartridges;
    my $SourceDir;
    my $SourceLang;
    GetOptions(
        'storename=s'                 => \$StoreName,
        'passwd=s'                    => \$Password,
        'sdir=s'                      => \$SourceDir,
        'cartridges'                  => \$Cartridges,
        'slang=s'                     => \$SourceLang,
    );
    usage() unless $SourceLang;
    usage() unless $SourceDir || $Cartridges;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub {
            ImportStrings({
                'SourceDir'  => $SourceDir,
                'SourceLang' => $SourceLang,
                'Cartridges' => $Cartridges,
            });
        },
    );
    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

#========================================================================================
# function     usage
# state        private
#----------------------------------------------------------------------------------------
# syntax       usage ()
#----------------------------------------------------------------------------------------
# description  Prints the required parameters of the script
#========================================================================================
sub usage() {
    print <<END_USAGE;
Usage:
    perl $0 -storename -passwd -slang -sdir -cartridges|-sdir directory

options:
    -storename      name of store          ()
    -passwd         database user password ()      optional
    -cartridges     install strings from installed cartridges (0) optional
    -slang          source languagecode    ()
    -sdir           source directory       () optional

Example:
    perl $0 -storename Store -passwd epages -slang de -sdir D:\\temp
    perl $0 -storename Store -passwd epages -slang de -cartridges
END_USAGE
    exit 2;
}

RunScript(Sub=>\&Main);
