#========================================================================================
# description      Check for all Unity shops if they exceed the revenue threshold and
#                   set the related attribute accordingly.
#========================================================================================

use strict;
use Getopt::Long;
use Time::HiRes qw (time);;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime SubtractDuration );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Order::API::SalesVolume qw (GetSalesVolumeByTaxModel);
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {

            my $Now = GetCurrentDateTime();
            # reset today to 00:00:00
            my $Today = $Now->clone->truncate( to => 'day' );
            # calculate date of a year before
            my $LastYear = $Today->clone;
            SubtractDuration($LastYear, 'years', 1);

            my $aShopIDs = GetUnityShopIDs();

            foreach my $ShopID (@$aShopIDs) {
                eval {
                    my $Shop = LoadObject($ShopID);
                    my $hInfoShop = $Shop->get(['CurrencyID', 'IsUnityRevenueThresholdExceeded']);
                    my $IsUnityRevenueThresholdExceeded;
                    # get feature value
                    my $UnityRevenueThreshold = $Shop->featureMaxValue('UnityRevenueThreshold');

                    # only do the check if the feature value exists
                    if ($UnityRevenueThreshold) {
                        # use the shop default currency
                        my $CurrencyID = $hInfoShop->{'CurrencyID'};
                        # (gross) revenue of this year
                        my ($SalesVolumeYear)
                            = GetSalesVolumeByTaxModel($Shop, $LastYear, $Today, $CurrencyID, TAX_MODEL_GROSS);
                        #check if limit has been exceeded
                        $IsUnityRevenueThresholdExceeded = $SalesVolumeYear > $UnityRevenueThreshold ? 1 : 0;
                    } else {
                        # if the feature value is 0 => no threshold defined => could not be exceeded
                        $IsUnityRevenueThresholdExceeded = 0;
                    }

                    # set the new value if it has been changed
                    $Shop->set({'IsUnityRevenueThresholdExceeded' => $IsUnityRevenueThresholdExceeded })
                        if $hInfoShop->{'IsUnityRevenueThresholdExceeded'} != $IsUnityRevenueThresholdExceeded;
                };
                Warning(GetError()) if ExistsError();
            }
            eval { SynchronizeCache(); };
            Warning(GetError()) if ExistsError();
        },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1 if not $NoOutput;;
}

sub usage {
    print <<END_USAGE;
Description:
    Check for all Unity shops if they exceed the revenue threshold and
    set the related attribute accordingly.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
