#========================================================================================
# package      DE_EPAGES::Product::API::CSV::ProductAnalyzer;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a separate class to analyze a CSV which
#               contains products and return an hash with the values.
#========================================================================================
package DE_EPAGES::Product::API::CSV::ProductAnalyzer;

use strict;
use Encode qw( decode );
use DE_EPAGES::Content::API::ShortUrl qw ( TestShortUrl );
use DE_EPAGES::Core::API::CSV qw( ParseCSV );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError);
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::CSVExportImport::API::Tools qw ( ParseValue);
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Product::API::Product qw( SubProductFolder ExistsSubProductByAttributeValues GetSubProductByAttributeValues );
use DE_EPAGES::Product::API::Constants qw( PRODUCT_ATTRIBUTE_DEPENDENCY $CSV_SPECIAL_PRODUCTITEMS_UNITY );

#ERRORDEFINITION
sub ErrorDefinition {
    my $hSuperErrorDefinition = DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler::ErrorDefinition();

    # Override base classes warning.
    $hSuperErrorDefinition->{'REFERENCED_IMAGE_MISSING'}
        = 'line #Line, column #AttributeName: referenced image not found';
    return {
        %$hSuperErrorDefinition,
        'WRONG_PREDEF' => 'line #Line, column #Attribute: void predefined value (#Value)',
        'WRONG_PREDEF_VALUE' => 'line #Line, column #Attribute: wrong predefined value: #New (must be #Old)',
        'PRODUCT_EMPTY_ALIAS' => 'line #Line, column Alias: alias has empty value',
        'NO_SUCH_PRODUCTCLASS' => 'line #Line, column #ID: class is not in shop (#ClassAlias)',
        'NO_SUPER_PRODUCT' => 'line #Line, column SuperProduct: product does not exist (#Super)',
        'WRONG_PRODUCTCLASS' => 'line #Line, column #ID: wrong class: #New (must be #Old)',
        'NO_PREDEF_ATTRIBUTE' => 'line #Line, column #Attribute does not exist in this ProductClass',
        'NO_PREDEF_VALUE' => 'line #Line, column #Attribute : has empty value',
        'SHORTURL_ALREADY_EXISTS' => 'line #Line, column #Column: shorturl already exists.',
        'SHORTURL_INVALID_CHARACTER' => 'line #Line, column #Column: invalid character in shorturl.',
        'SHORTURL_NOT_ALLOWED' => 'line #Line, column #Column: setting shorturl for this contentobject is not allowed.',
        'VARIATION_EXISTS' => 'line #Line, column VariationString: variation already exists (#VariationString).',
        'WRONG_ECOPARTICIPATIONCATEGORY' => 'line #Line, column #Attribute: wrong category value (#Value)',
        'MISSING_ECOPARTICIPATIONCATEGORY' => 'line #Line, column #Attribute: missing value for attribute EcoParticipationCategory',
        'MISSING_FEATURE' => 'line #Line: missing feature #Feature',
        'NAME_NOT_VALID' => 'line #Line, column #Attribute: name \'#Value\' must not include special characters only.',
    };
}
#ERRORDEFINITION

sub new {
    my $class = shift;
    my ($ImportHandler) = @_;

    my $hOptions = {
        'ImportHandler' => $ImportHandler,
    };

    my $self = bless $hOptions, $class;
    return $self;
}

# Init is calles each time a new line is analyzed. It resets some variables needed per dataset.
sub _init {
    my $self = shift;

    $self->{'IsDownloadProduct'} = 0;
    $self->{'IsProductSub'} = 0;
    $self->{'IsBundleProduct'} = 0;
    $self->{'Column'}            = undef;
    $self->{'Product'}           = undef;
    $self->{'ProductClass'}      = undef;
    $self->{'SuperProduct'}      = undef;
    $self->{'ImportAttributes'}  = undef;
    $self->{'AttributeValues'}   = undef;
    $self->{'VariationColumns'}  = undef;
    $self->{'DoNotImport'}       = 0;
}

#========================================================================================
# function     analyze
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hAttributeValues = $self->analyze( $aColumns );
#----------------------------------------------------------------------------------------
# description  Analyzes an array of CSV columns which contain product information and
#               returns a hash which can be imported into the application.
# input        $aColumns | CSV columns | ref array
# return       $hAttributeValues | properties of a object of CSV file with following keys
#               <ul>
#                 <li>DirectAttributes  - direct attributes of the object with following keys
#                   <ul>
#                       <li>Alias - alias of the object - string</li>
#                       <li>Class - class object of the object - object</li>
#                       <li>Parent - parent object of the object - object</li>
#                       <li> more direct attributes which should set via $Object->set method,
#                            or can used by $Class->insert($hAttributeValues->{'DirectAttributes'});</li>
#                   </ul> - ref.hash
#                 </li>
#                 <li>DoNotImport -  0 or 1, set to 1 if there are errors to set the parsed values - boolean</li>
#                 <li>Object - object if it is found or undef if it doesn't exist - object</li>
#                 <li>LocalizedAttributes -localized attributes hash with language id as key - ref.hash</li>
#                 <li>PriceAttributes - price attributes hash with CurrencyID/TaxModel as key e.g. ('EUR/1') -ref.hash</li>
#                 <li>VariationValueIDs - variation value ids which are used at the subproduct - ref.array</li>
#                 <li>PreDefMultiValueIDs - value ids for a multiple-choice option attribute - ref.hash.array.int</li>
#               </ul>
#               | ref.hash
#========================================================================================
sub analyze {
    my $self = shift;
    my ($aColumns) = @_;

    $self->_init();
    my $ImportHandler    = $self->{'ImportHandler'};
    my $ImportAttributes = $ImportHandler->topID('ImportAttributes');
    $self->{'ImportAttributes'} = $ImportAttributes;
    $self->{'Column'} = $aColumns;

    my $ProductAlias = $aColumns->[ $ImportAttributes->{'AliasColumn'} ];
    $ProductAlias =~ s/\s+$//;
    $ProductAlias =~ s/^\s+//;
    unless ( defined $ProductAlias && $ProductAlias =~ /\S/ ) {
        $self->error( 'PRODUCT_EMPTY_ALIAS', { ID => $ImportAttributes->{'AliasColumn'} + 1 } );
        return;
    }

    my $Shop = $ImportHandler->object;

    # Get parent -> is sub product ?
    my $Parent = $self->_getProductSubInfo( $aColumns, $Shop, $ImportAttributes, $ProductAlias );

    if ( defined $ImportAttributes->{'IsDownloadProductColumn'} ) {
        $self->{'IsDownloadProduct'} = $aColumns->[ $ImportAttributes->{'IsDownloadProductColumn'} ];
    }

    if ( defined $ImportAttributes->{'IsBundleProductColumn'} ) {
        $self->{'IsBundleProduct'} = $aColumns->[ $ImportAttributes->{'IsBundleProductColumn'} ];
    }

    # Exists product ?
    $self->{'Product'} = $Parent->existsChild($ProductAlias) ? $Parent->child($ProductAlias) : undef;

    if ( defined $self->{'Product'} ) {
        $self->{'IsDownloadProduct'} = $self->{'Product'}->get('IsDownloadProduct');
        $self->{'IsBundleProduct'}   = $self->{'Product'}->get('IsBundleProduct');
    }

    # Check features for downloadproducts and bundleproducts.
    if ( $self->{'IsDownloadProduct'} && !$Shop->featureMaxValue('ProductDownloads') ) {
        $self->error( 'MISSING_FEATURE', { Feature => 'ProductDownloads' } );
    }

    if ( $self->{'IsBundleProduct'} && !$Shop->featureMaxValue('ProductBundles') ) {
        $self->error( 'MISSING_FEATURE', { Feature => 'ProductBundles' } );
    }

    # get product class
    $self->_getProductClass( $aColumns, $self->{'IsProductSub'}, $Shop, $ImportAttributes );

    my $hFormat     = $ImportHandler->locale->get( [qw(LocaleID DecimalSep ThousandSep)] );
    my @UserAttributePreDefs = grep { $_->get('Type') =~ /^PreDef(Localized)?String$/x }
    @{ $self->{'ProductClass'}->get('UserDefinedAttributes') };


    # we do a special handling for unity shops
    my $IsUnityShop = $Shop->get('IsUnityRegistered');

    # get default attribute values for setting values for ignored attributes if we have a unity shop
    my $AttributeDefaults = DE_EPAGES::Product::API::Object::Product->default($Shop->get('ProductFolder'));

    # Iterate all columns of that line and handle each value bases by its type.
    for ( my $i = 0; $i < scalar(@$aColumns); $i++ ) {
        my $hAttribute = $ImportAttributes->{$i};

        # ignore unknown attribute columns (see also DE_EPAGES::Product::API::CSV::ProductImportHandler::getUsedAttribute)
        next if exists $hAttribute->{'IGNORE_COLUMN'};

        #ignore all attributes which are not base, direct or my class
        next unless ( $hAttribute->{'ClassAlias'} eq 'ProductClass'
            or $hAttribute->{'ClassAlias'} eq $self->{'ProductClass'}->alias );
        next if ( $hAttribute->{'Alias'} eq 'Class' or $hAttribute->{'Alias'} eq 'SuperProduct' );

        my $Value = $aColumns->[$i];
        $Value = 0 if $hAttribute->{'Alias'} eq 'Position' && $Value eq '';

        my $MaxLength = $hAttribute->{'Attribute'}->get('Length');

        if( defined $MaxLength and length $Value > $MaxLength ) {
            $self->error( 'STRING_TOO_LONG', {
                'Attribute' => $hAttribute->{'Alias'},
                'ID'=> $i + 1,
                'Value' => $Value,
                'MaxLength' => $MaxLength,
            });
            next;
        }

        #use CSV_SPECIAL_PRODUCTITEMS_UNITY for ignored attributes an set default values to special attributes
        # filter attributes for importing only unity relevant data
        # if merchant tries to import a normal epages6 import file

        if($IsUnityShop){
            if (ElementOf($hAttribute->{'Alias'}, @{$CSV_SPECIAL_PRODUCTITEMS_UNITY->{0}})) {
               my $DefaultValue = $AttributeDefaults->{$hAttribute->{'Alias'}};
               if($hAttribute->{'IsObject'}){ # use alias as value
                    $Value = $DefaultValue ? $DefaultValue->alias() : '';
               }else {
                    $Value =  $DefaultValue // '';
               }
            }
        }

        # note: ParseValue sets $Value to undef if it was ''
        eval { ParseValue( $hAttribute->{'Alias'}, $hAttribute->{'Type'}, \$Value, $hFormat ) };
        if ( ExistsError() ) {
            $self->error( 'VOID_FIELD', { Attribute => $hAttribute->{'Alias'}, ID => $i + 1, Error => GetError()->shortMessage } );
            next;
        }

        if ( !defined $Value && $hAttribute->{'Attribute'}->get('IsMandatory') ) {
            $self->error( 'NO_MAND_VALUE', { Attribute => $hAttribute->{'Alias'} } );
            next;
        }

        if ( !defined $Value ) {
            #Ignore attributes without a value if they depend on another attribute that don't have a value, too.
            my $DependsOnAlias = PRODUCT_ATTRIBUTE_DEPENDENCY->{$hAttribute->{'Alias'}};
            if($DependsOnAlias){
                my $DependsOnKey = ( grep { ref $ImportAttributes->{$_} eq 'HASH'
                                            and $ImportAttributes->{$_}{'Alias'} eq $DependsOnAlias
                                          } keys %$ImportAttributes )[0];
                next unless $aColumns->[$DependsOnKey];
            }
        }

        my $AttributeType = $hAttribute->{'Type'};
        if ( $AttributeType =~ /^PreDefMulti(Localized)?String$/x ) {
            $self->_analyzePreDefMultiAttribute( $Value, $hAttribute );
        }
        elsif ( $hAttribute->{'IsObject'} ) {
            $Value = $self->_handleObject( $hAttribute, $Value, \@UserAttributePreDefs );
        }
        else {    # not IsObject
            $Value = $self->_handleNonObject( $hAttribute, $Value, $i );
        }

        # do not allow names consisting of special characters only (UNITY-3683)
        if ($hAttribute->{'Alias'} eq 'Name' && defined $Value && $Value !~ /\w/) {
            $self->error( 'NAME_NOT_VALID', {
                'Attribute' => $hAttribute->{'Alias'},
                'ID'=> $i + 1,
                'Value' => $Value
            });
            next;
        }
    }

    if ( ( not $self->{'DoNotImport'} ) and $self->{'IsProductSub'} ) {
        $self->_handleVariations();
    }
    $self->{'AttributeValues'}->{'DirectAttributes'}{'Class'}  = $self->{'ProductClass'};
    $self->{'AttributeValues'}->{'DirectAttributes'}{'Parent'} = $Parent;
    $self->{'AttributeValues'}->{'DirectAttributes'}{'Shop'}   = $Shop;

    $self->{'AttributeValues'}->{'DoNotImport'} = $self->{'DoNotImport'};
    $self->{'AttributeValues'}->{'Object'}      = $self->{'Product'};
    return $self->{'AttributeValues'};
}

sub _analyzePreDefMultiAttribute {
    my $self = shift;
    my ( $Value, $hAttribute ) = @_;

    unless ( defined $Value ) {
        # delete any previous selected values
        $self->{'AttributeValues'}->{'PreDefMultiValueIDs'}->{ $hAttribute->{'Alias'} } = [];
        return;
    }

    my $aValueAliases = ParseCSV( $Value, { 'sep_char' => '~' } );
    my $ValuesFolder = $hAttribute->{'Attribute'}->folder('PreDefAttributes');
    my @ValueIDs;
    foreach my $ValueAlias (@$aValueAliases) {
        my $ValueID = $ValuesFolder->findChild($ValueAlias);
        if ( !defined $ValueID ) {
            $self->error(
                'WRONG_PREDEF', {
                    Value     => $ValueAlias,
                    Attribute => $hAttribute->{'Alias'}
                }
            );
            return;
        }
        push @ValueIDs, $ValueID;
    }

    $self->{'AttributeValues'}->{'PreDefMultiValueIDs'}->{ $hAttribute->{'Alias'} } = \@ValueIDs;
    return;
}

sub _getProductSubInfo {
    my $self = shift;
    my ( $aColumns, $Shop, $ImportAttributes, $ProductAlias ) = @_;

    my $ProductFolder = $Shop->folder('Products');
    return $ProductFolder unless defined $ImportAttributes->{'SuperProductAliasColumn'};

    my $SuperProductAlias = $aColumns->[ $ImportAttributes->{'SuperProductAliasColumn'} ];
    if ( defined $SuperProductAlias ) {
        $SuperProductAlias =~ s/\s+$//;
        $SuperProductAlias =~ s/^\s+//;
        $SuperProductAlias = undef if $SuperProductAlias eq '';
    }
    return $ProductFolder unless defined $SuperProductAlias;

    if ( !$ProductFolder->existsChild($SuperProductAlias) ) {
        $self->error('NO_SUPER_PRODUCT', {
            Sub   => $ProductAlias,
            Super => $SuperProductAlias
        });
        return $ProductFolder;
    }

    $self->{'IsProductSub'} = 1;
    $self->{'SuperProduct'} = $ProductFolder->child($SuperProductAlias);
    return SubProductFolder($self->{'SuperProduct'});
}

sub _getProductClass {
    my $self = shift;
    my ( $aColumns, $ProductIsSub, $Shop ) = @_;

    my ( $ClassAlias );
    if ( defined $self->{'ImportAttributes'}->{'ClassColumn'} ) {
        $ClassAlias = $aColumns->[ $self->{'ImportAttributes'}->{'ClassColumn'} ];
        $ClassAlias =~ s/\s+$//;
        $ClassAlias =~ s/^\s+//;
        $ClassAlias = undef if $ClassAlias eq '';
        my $ProductTypeFolder = $Shop->folder('ProductTypes');
        if ( not defined $ClassAlias or $ClassAlias eq 'ProductClass' ) {
            $self->{'ProductClass'} = $Shop->get('ProductClass');
        }
        elsif ( $ProductTypeFolder->existsChild($ClassAlias) ) {
            $self->{'ProductClass'} = $ProductTypeFolder->child($ClassAlias);
        }
        else {
            $self->error( 'NO_SUCH_PRODUCTCLASS',
                { ID => $self->{'ImportAttributes'}->{'ClassColumn'} + 1, ClassAlias => $ClassAlias } );
        }
    }
    if ( defined $self->{'ProductClass'} ) {

        # can\'t change the product type of sub products or master products
        # sub products must use the same product type as the super product
        if ( $ProductIsSub || defined $self->{'Product'}) {
            my $ExistingProductClass = $ProductIsSub ? $self->{'SuperProduct'}->class : $self->{'Product'}->class;
            if ( $ExistingProductClass->id != $self->{'ProductClass'}->id ) {
                if ($ProductIsSub || $ExistingProductClass->id != $Shop->get('ProductClass')->id) {
                    $self->error(
                        'WRONG_PRODUCTCLASS', {
                            ID  => $self->{'ImportAttributes'}->{'ClassColumn'} + 1,
                            Old => $ExistingProductClass->alias,
                            New => defined $ClassAlias ? $ClassAlias : ''
                        }
                    );
                    $self->{'ProductClass'} = $ExistingProductClass;
                }
            }
        }
    }
    else {
        if ( defined $self->{'Product'} ) {
            $self->{'ProductClass'} = $self->{'Product'}->class;
        }
        elsif ( defined $self->{'SuperProduct'} ) {
            $self->{'ProductClass'} = $self->{'SuperProduct'}->class;
        }
        else {
            $self->{'ProductClass'} = $Shop->get('ProductClass');
        }
    }
    return $self->{'ProductClass'};
}

sub error {
    my $self = shift;
    my ( $ErrorCode, $hErrorVars ) = @_;

    $hErrorVars->{'Line'} =  $self->{'ImportHandler'}->lineNumber;
    Warning( $ErrorCode, $hErrorVars );

    $self->{'DoNotImport'} = 1;
}

sub testPrice {
    my $self = shift;
    my ( $hAttribute, $Value ) = @_;

    if ( $hAttribute->{'Alias'} eq 'EcoParticipations' ) {
        $Value = undef if $self->{'IsDownloadProduct'};
        unless ( $self->testEcoParticipation( $hAttribute, $Value ) ) {
            return 0;
        }
    }

    my $CurrencyID = $hAttribute->{'CurrencyID'};
    my $TaxModel   = $hAttribute->{'TaxModel'};
    my $Alias      = $hAttribute->{'Alias'};
    $self->{'AttributeValues'}{'PriceAttributes'}{"$CurrencyID/$TaxModel/$Alias"} = {
        'Alias'      => $Alias,
        'CurrencyID' => $CurrencyID,
        'TaxModel'   => $TaxModel,
        'Price'      => $Value
    };
    return 1;
}

sub testEcoParticipation {
    my $self = shift;
    my ( $hAttribute, $Value ) = @_;

    if ( defined $Value and $self->_isCategoryMissing() ) {
        $self->error(
            'MISSING_ECOPARTICIPATIONCATEGORY', {
                Value     => $Value,
                Attribute => 'EcoParticipationCategory',
            }
        );
        return 0;
    }
    return 1;
}

sub _isCategoryMissing {
    my $self = shift;

    my $MissingCategory = 0;
    my $EcoColumnIndex  = $self->{'ImportAttributes'}->{'EcoParticipationCategoryColumn'};

    if ( !defined $EcoColumnIndex ) {
        if ( !defined $self->{'Product'} ) {
            $MissingCategory = 1;
        }
        elsif ( !defined $self->{'Product'}->get('EcoParticipationCategory') ) {
            $MissingCategory = 1;
        }
    }
    elsif ( $self->{'Column'}->[$EcoColumnIndex] !~ /\S/ ) {
        $MissingCategory = 1;
    }
    return $MissingCategory;
}

sub _handleObject { ##no critic (ProhibitExcessComplexity)
    my $self = shift;
    my ($hAttribute, $Value, $aUserAttributePreDefs ) = @_;

    # Some handy variables.
    my $hUnits      = $self->{'ImportHandler'}->topID('ShopUnits');
    my $hTaxClasses = $self->{'ImportHandler'}->topID('ShopTaxClasses');
    my $Shop        = $self->{'ImportHandler'}->object;
    my $AttributeType = $hAttribute->{'Type'};

    if ( not defined $Value ) {
        return if $hAttribute->{'Alias'} eq 'TaxClass';
        $self->{'AttributeValues'}->{'DirectAttributes'}{ $hAttribute->{'Alias'} } = undef
            if $hAttribute->{'Alias'} ne 'SelectedVariations';
    }
    elsif ( $AttributeType eq 'UnitOfMeasurement' ) {
        if ($self->{'IsDownloadProduct'} || $self->{'IsBundleProduct'}) {
            if ( $hAttribute->{'Alias'} eq 'OrderUnit' ) {
                $Value = $hUnits->{'piece'};
            } elsif($hAttribute->{'Alias'} eq 'WeightUnit'
                and $hUnits->{$Value}->get('Type') eq 'mass'){
               $Value = $hUnits->{$Value};
            } else {
                $Value = undef;
            }
        }
        elsif ( !defined $hUnits->{$Value} ) {
            $self->error(
                'WRONG_PREDEF', {
                    Value     => $Value,
                    Attribute => $hAttribute->{'Alias'}
                }
            );
            return;
        }
        elsif ( $hAttribute->{'Alias'} eq 'WeightUnit'
            and $hUnits->{$Value}->get('Type') ne 'mass' ) {
            $self->error(
                'WRONG_PREDEF', {
                    Value     => $Value,
                    Attribute => $hAttribute->{'Alias'}
                }
            );
            return;
        }
        else {
            $Value = $hUnits->{$Value};
        }
    }
    elsif ( $hAttribute->{'Alias'} eq 'SelectedVariations' ) {
        #define variation columns
        my $aVariationTypes = ParseCSV( $Value, { 'sep_char' => '~' } );
        $self->{'VariationColumns'}->{$_} = {} foreach @$aVariationTypes;
    }
    elsif ( $hAttribute->{'Alias'} eq 'ShippingMethods' ) {
        if ($self->{'IsDownloadProduct'}) {
            $Value = undef;
        }
        else {
            #define variation columns
            my $aMethods = ParseCSV( $Value, { 'sep_char' => '~' } );
            my $Folder = $Shop->child('ShippingMethods');
            if ( scalar grep { !$Folder->existsChild($_) } @$aMethods ) {
                $self->error(
                    'WRONG_PREDEF', {
                        Value     => $Value,
                        Attribute => $hAttribute->{'Alias'}
                    }
                );
                return;
            }
            else {
                $Value = [ map { $Folder->child($_) } @$aMethods ];
            }
        }
    } elsif ($hAttribute->{'Alias'} eq 'TaxClassDigital') {
         my $TMD = $Shop->get('TaxMatrixDigital');
         if ($TMD->existsChild($Value)) {
             $Value = $TMD->child($Value);
         } else {
             $self->error(
                 'WRONG_PREDEF', {
                    Value     => $Value,
                    Attribute => $hAttribute->{'Alias'},
                 }
             );
             return;
         }
    } elsif ( $hAttribute->{'Alias'} eq 'TaxClass' ) {
        # HACK: The tax classes "ermssigt" and "keine" in /TaxMatrixGermany
        # have been renamed to "reduced" and "none" respectively. For compatipility
        # with old import files, we map the old values to the new names.
        if ( !defined $hTaxClasses->{$Value}
            && $Shop->get('TaxMatrix')->get('Path') eq '/TaxMatrixGermany' ) {
            my $NewValue;
            $NewValue = 'none' if $Value eq 'keine';
            $NewValue = 'reduced' if $Value eq decode( 'iso-8859-1', 'ermssigt' );
            if ( defined $NewValue && defined $hTaxClasses->{$NewValue} ) {
                GetLog()->debug( sprintf 'Mapping old tax class \'%s\' to \'%s\' in shop %s',
                    $Value, $NewValue, $Shop->alias );
                $Value = $NewValue;
            }
        }
        if ( defined $hTaxClasses->{$Value} ) {
            $Value = $hTaxClasses->{$Value};
        }
        else {
            $self->error(
                'WRONG_PREDEF', {
                    Value     => $Value,
                    Attribute => $hAttribute->{'Alias'}
                }
            );
            return;
        }
    }
    elsif ( $hAttribute->{'Alias'} eq 'EcoParticipationCategory' ) {
        if ($self->{'IsDownloadProduct'}) {
            $Value = undef;
        }
        else {
            my $EcoParticipationCategories = LoadObjectByPath('/EcoParticipationCategories/');
            if ( $EcoParticipationCategories->existsChild( $Value ) ) {
                $Value = $EcoParticipationCategories->child( $Value );
            }
            else {
                $self->error(
                    'WRONG_ECOPARTICIPATIONCATEGORY', {
                        Value     => $Value,
                        Attribute => $hAttribute->{'Alias'},
                    }
                );
                return;
            }
        }
    }
    else {
        #predef
        my ($UserAttributePreDef)
        = grep { $_->alias eq $hAttribute->{'Alias'} } @{$aUserAttributePreDefs};
        if ( not defined $UserAttributePreDef ) {
            $self->error(
                'NO_PREDEF_ATTRIBUTE', {
                    Value     => $Value,
                    Attribute => $hAttribute->{'Alias'}
                }
            );
            return;
        }
        else {
            if ( !$UserAttributePreDef->existsPreDefAttributeValue($Value) ) {
                $self->error(
                    'WRONG_PREDEF', {
                        Value     => $Value,
                        Attribute => $hAttribute->{'Alias'}
                    }
                );
                return;
            }
            else {
                $Value = $UserAttributePreDef->preDefAttributeValue($Value);
            }
        }
    }
    $self->{'AttributeValues'}->{'DirectAttributes'}{ $hAttribute->{'Alias'} } = $Value
    if $hAttribute->{'Alias'} ne 'SelectedVariations';
    return $Value;
}

# returns the value of a scalar attribute
sub _handleNonObject {
    my $self = shift;
    my ($hAttribute, $Value, $i ) = @_;

    my $Shop = $self->{'ImportHandler'}->object;
    my $AttributeType = $hAttribute->{'Type'};

    # test for maximum string length
    if ( defined $Value && $AttributeType =~ /^(?:Localized)?String$/ ) {
        my $MaxLength = $hAttribute->{'Attribute'}->get('Length');
        if( defined $MaxLength && length $Value > $MaxLength ) {
            $self->error( 'STRING_TOO_LONG', {
                'Attribute' => $hAttribute->{'Alias'},
                'ID'=> $i + 1,
                'Value' => $Value,
                'MaxLength' => $MaxLength,
            });
            return undef;
        }
    }

    if ( $AttributeType =~ /^Localized/ ) {
        $self->{'AttributeValues'}->{'LocalizedAttributes'}{ $hAttribute->{'LanguageID'} }
        { $hAttribute->{'Alias'} } = $Value;
    }
    elsif ( $AttributeType eq 'Price' ) {
        return undef unless $self->testPrice( $hAttribute, $Value );
    }
    elsif ( not $Value and $hAttribute->{'Alias'} eq 'PriceQuantity' ) {
        $self->error( 'FLOAT_TOO_SMALL', {
            Value     => $Value,
            Attribute => $hAttribute->{'Alias'}
        });
        return undef;
    }
    elsif ( defined $Value and not $Value and $hAttribute->{'Alias'} eq 'RefAmount' )
    {
        if ($self->{'IsDownloadProduct'}) {
            $Value = undef;
        }
        else { # not defined or != 0 is allowed
            $self->error( 'FLOAT_TOO_SMALL', {
                Value     => $Value,
                Attribute => $hAttribute->{'Alias'}
            });
            return undef;
        }
    }
    elsif ( not $Value and $hAttribute->{'Alias'} eq 'IntervalOrder' ) {
        $self->error( 'FLOAT_TOO_SMALL', {
            Value     => $Value,
            Attribute => $hAttribute->{'Alias'}
        });
        return undef;
    }
    elsif ( not $Value and $hAttribute->{'Alias'} eq 'MinOrder' ) {
        $self->error( 'FLOAT_TOO_SMALL', {
            Value     => $Value,
            Attribute => $hAttribute->{'Alias'}
        });
        return undef;
    }
    else {
        if ( ElementOf( $hAttribute->{'Alias'},
                qw( StockLevel StockLevelAlert Length Height Width RefAmount RefContentAmount )
             )
        ) {
            $Value = undef if $self->{'IsDownloadProduct'};
            if (defined $Value && $Value <= 0 && $hAttribute->{'Alias'} eq 'StockLevelAlert') {
                $self->error( 'FLOAT_TOO_SMALL', {
                    Value     => $Value,
                    Attribute => $hAttribute->{'Alias'}
                });
                return undef;
            }
        }
        elsif ( ElementOf( $hAttribute->{'Alias'}, qw( MaxDownloadTime MaxDownloadCount ) ) ) {
            $Value = undef unless $self->{'IsDownloadProduct'};
        }
        elsif ( $hAttribute->{'Alias'} eq 'IsDownloadProduct' ) {
            $Value = $self->{'IsDownloadProduct'};
        }

        $self->{'AttributeValues'}->{'DirectAttributes'}{ $hAttribute->{'Alias'} } = $Value;
    }

    if ( $hAttribute->{'Alias'} eq 'URI' ) {
        # Short URL for sub products are not allowed - suppress warnings caused by EPG-27883
        # and delete existing short URLs
        $Value = undef if $self->{'IsProductSub'};

        my $URI = $Value;
        if ( defined($URI) && $URI ne '' ) {
            eval { TestShortUrl( $Shop, $self->{'Product'}, $URI, $hAttribute->{'LanguageID'} ); };
            if ( ExistsError() ) {
                my $Error = GetError();
                $self->error(
                    'SHORTURL_ALREADY_EXISTS', {
                        Column => 'URI/'
                        . GetCodeByLanguageID( $hAttribute->{'LanguageID'} )
                    }
                ) if $Error->code eq 'SHORTURL_ALREADY_EXISTS';
                $self->error(
                    'SHORTURL_INVALID_CHARACTER', {
                        Column => 'URI/'
                        . GetCodeByLanguageID( $hAttribute->{'LanguageID'} )
                    }
                ) if $Error->code eq 'INVALID_CHARACTER';
                $self->error(
                    'SHORTURL_NOT_ALLOWED', {
                        Column => 'URI/'
                        . GetCodeByLanguageID( $hAttribute->{'LanguageID'} )
                    }
                ) if $Error->code eq 'ShortURL_NOT_ALLOWED';
                return undef;
            }
        }
    }
    if ( $hAttribute->{'Alias'} eq 'Title' && $self->{'IsProductSub'} ) {
        $Value = undef;
    }

    return $Value;
}

sub _handleVariations {
    my $self = shift;

    my @VariationAttributes = map { $_->{'Attribute'} } @{ $self->{'SuperProduct'}->get('VariationTypeValues') };
    delete $self->{'VariationColumns'}->{ $_->alias } foreach @VariationAttributes;

    foreach my $AttrAlias ( keys %{$self->{'VariationColumns'}} ) {
        if ( $self->{'ProductClass'}->existsAttribute($AttrAlias) ) {
            push @VariationAttributes, $self->{'ProductClass'}->attribute($AttrAlias);
        }
        else {
            $self->error( 'NO_PREDEF_ATTRIBUTE', { Attribute => $AttrAlias, } );
        }
    }

    # this condition is true if the sub product does not yet exist and the column
    # SelectedVariations does not exist or is empty
    if( 0 == scalar @VariationAttributes ) {
        $self->error( 'NO_PREDEF_VALUE', { Attribute => 'SelectedVariations' } );
        return;
    }

    my @VariationValueIDs;
    foreach my $Attribute (@VariationAttributes) {
        my $AttrAlias = $Attribute->alias;
        $self->{'VariationColumns'}->{$AttrAlias}{'Atttribute'} = $AttrAlias;

        #exists $AttributeValues{'DirectAttributes'}{$AttrAlias}; -> vorher prfen
        my $Value = $self->{'AttributeValues'}->{'DirectAttributes'}{$AttrAlias};
        if ( defined $self->{'Product'} ) {
            #Product exists
            my $ExistingValue = $self->{'Product'}->getWithParent($AttrAlias);
            if ( defined $Value ) {
                my $ValueAlias = $Value->alias;
                my $ExistingValueAlias = defined $ExistingValue ? $ExistingValue->alias : undef;
                if ( defined $ExistingValueAlias && $ExistingValueAlias ne $ValueAlias ) {
                    $self->error(
                        'WRONG_PREDEF_VALUE', {
                            Attribute => $AttrAlias,
                            Old       => $ExistingValueAlias,
                            New       => $ValueAlias
                        }
                    );
                }
            }
            else {
                $Value = $ExistingValue;
                $self->{'AttributeValues'}->{'DirectAttributes'}{$AttrAlias} = $Value;
            }
        }
        else {
            if ( not defined $Value ) {
                $self->error( 'NO_PREDEF_VALUE', { Attribute => $AttrAlias, } );
                return;
            }
        }
        $self->{'VariationColumns'}->{$AttrAlias}{'Value'} = $Value;
        push @VariationValueIDs, $Value->id if defined $Value;
    }

    #test if variation already exists
    if ( ExistsSubProductByAttributeValues($self->{'SuperProduct'}->id, \@VariationValueIDs) ) {
        $self->error( 'VARIATION_EXISTS', { VariationString => join(', ', @VariationValueIDs) } ) unless defined $self->{'Product'};
        my $SubProduct = GetSubProductByAttributeValues($self->{'SuperProduct'}->id, \@VariationValueIDs);
        if ( defined $self->{'Product'} && $SubProduct->id != $self->{'Product'}->id ) {
            $self->error( 'VARIATION_EXISTS', { VariationString => join(', ', @VariationValueIDs) } );
        }
    }

    $self->{'AttributeValues'}->{'DirectAttributes'}{'SuperProduct'}    = $self->{'SuperProduct'};
    $self->{'AttributeValues'}->{'VariationValueIDs'}                   = \@VariationValueIDs;
}

1;
