#========================================================================================
# package      DE_EPAGES::Product::Hooks::Product
# state        public
# description  assign categories from the original product to the cloned product
#========================================================================================
package DE_EPAGES::Product::Hooks::Product;

use strict;
use DE_EPAGES::Core::API::PerlTools qw( CopyStructure );
use DE_EPAGES::Core::API::File qw( DeleteFile );
use DE_EPAGES::Presentation::API::Constants qw (BASIC_JSON_ATTRIBUTES);
use DE_EPAGES::Product::API::Constants qw ( @FROM_MASTERPRODUCT_INHERITABLE_ATTRIBUTES_ON_UPDATE);
use DE_EPAGES::Product::API::Product qw ( GetVisibleSubProductIDs );
use DE_EPAGES::Shop::API::Constants qw ( STOCK_NOT_VISIBLE );

#========================================================================================
# function     OnJSONChildren
#----------------------------------------------------------------------------------------
# syntax       OnJSONChildren( $hParams );
#----------------------------------------------------------------------------------------
# description  Assign sub products to json children for content tree.
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object (only products used) - object
#                   <li>Children - list of child info - ref.array.hash.string
#                   <li>Parameters - servlet parameters - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnJSONChildren {
    my ($hParams) = @_;
    my $Product = $hParams->{'Object'};
    my $IsProduct = $Product->instanceOf('Product');
    my $aChildren = $hParams->{'Children'};

    return unless $IsProduct;
    @$aChildren = ();
    # create foreach clone the product assignments from the original
    my @AttributeAliases = map { $_->{'Attribute'}->alias } @{$Product->get('VariationTypeValues')};
    foreach my $SubProduct (@{$Product->get('SubProducts')}) {
        my $hInfo = $SubProduct->get(BASIC_JSON_ATTRIBUTES);
        $hInfo->{'IsVisible'} = $SubProduct->get('IsVisible');
        $hInfo->{'title'} = join (',', map { $SubProduct->getWithParent($_)->get('ValueOrAlias') } @AttributeAliases);
        $hInfo->{'_isProduct'} = 1;
        $hInfo->{'hasChildren'} = 0;
        push @$aChildren, $hInfo;
    }
    return;
}

#========================================================================================
# function     OnSEOCheckTests
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSEOCheckTests( $hValues );
#----------------------------------------------------------------------------------------
# description  Extends/changes values from the SEO check for content objects
#               (Hook SEOCheckTests)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - object
#                   <li>Shop - shop object - object
#                   <li>Values - values and ratings for SEO check - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnSEOCheckTests {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    return unless $Product->instanceOf('Product');

    my $Shop = $hParams->{'Shop'};
    my $hValues = $hParams->{'Values'};

    my $MainImage = $Product->get('ImageLarge');
    my %AllImages;
    $AllImages{$MainImage} = 1 if $MainImage;

    my $ahImagesSlideShow = $Product->get('ImagesSlideShow');
    if($ahImagesSlideShow) {
        foreach my $hSlideShowImage (@$ahImagesSlideShow) {
            $AllImages{$hSlideShowImage->{'ImageName'}} = 1;
        }
    }

    my @AllImageNames = keys %AllImages;
    my $ProductImagesRating = _getProductImagesRating(\@AllImageNames, $Shop);
    $hValues->{'ProductImages'} = {
        'Value' => scalar(@AllImageNames),
        'Rating' => $ProductImagesRating,
        'RatingClass' => qw ( green green yellow red )[$ProductImagesRating-1],
        'Percentage' => 0.1,
    };
}

sub _getProductImagesRating {
    my ($aImages, $Shop) = @_;

    my $HasSlideshowFeature = $Shop->featureMaxValue('ProductSlideShow');

    return 4 if !$aImages || scalar(@$aImages) < 1;
    return 3 if scalar(@$aImages) == 1 && $HasSlideshowFeature;
    return 2 if scalar(@$aImages) == 1 && !$HasSlideshowFeature;
    return 1;
}

#========================================================================================
# function     OnSEOCockpitSearchUntested
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSEOCockpitSearchUntested( $hValues );
#----------------------------------------------------------------------------------------
# description  Extends pager search options for the list of untested contentobject
#               objects on the SEO cockpit.
#               (Hook SEOCockpitSearchUntested)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnSEOCockpitSearchUntested {
    my ($hParams) = @_;

    my $hOptions = $hParams->{'Options'};

    # Tell search definition to use this parameter, but no value is needed.
    # Adds a search parameter for filtering variation products.
    $hOptions->{'Parameters'}->{'OnlySuperProducts'} = 1;
}

#========================================================================================
# function     OnBeforeUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateProduct( $hValues );
#----------------------------------------------------------------------------------------
# description  Checks the status of the the product notifications.
#               Inherites boolean attrbiutes from the master to sub products.
#               (Hook OBJ_BeforeUpdateProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $LanguageID = $hParams->{'LanguageID'};
    my $hNewValues = $hParams->{'Values'};

    my $Shop = $Product->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        # change the status of the product notification for Unity shops
        if (exists $hNewValues->{'StockLevel'}) {
            # product out of stock
            my $ProductOutOfStockNotification = $Shop->folder('ShopNotifications')->child('ProductOutOfStock');
            my $NewStockLevel = $hNewValues->{'StockLevel'};
            # stock level changes from above to below zero
            if (defined $NewStockLevel && $NewStockLevel <= 0) {
                $ProductOutOfStockNotification->addObject($Product);
            # stock level changes from below to above zero
            } else {
                $ProductOutOfStockNotification->deleteObject($Product);
            }
        }
        if (exists $hNewValues->{'StockLevel'} || exists $hNewValues->{'StockLevelAlert'}) {
            # product low on stock
            my $ProductLowOnStockNotification = $Shop->folder('ShopNotifications')->child('ProductLowOnStock');
            my $hInfoProduct = $Product->get(['StockLevel', 'StockLevelAlert']);
            my $OldStockLevel = $hInfoProduct->{'StockLevel'};
            my $OldMinStockLevel = $hInfoProduct->{'StockLevelAlert'};
            if (exists $hNewValues->{'StockLevel'} && ((!defined $OldStockLevel) || ($hNewValues->{'StockLevel'} != $OldStockLevel))) {
                # new stock level
                my $NewStockLevel = $hNewValues->{'StockLevel'};
                if (defined $NewStockLevel) {
                    if (exists $hNewValues->{'StockLevelAlert'} && $hNewValues->{'StockLevelAlert'} != $OldMinStockLevel) {
                        # new min stock level
                        my $NewMinStockLevel = $hNewValues->{'StockLevelAlert'};
                        if (defined $NewMinStockLevel) {
                            if ($NewStockLevel < $NewMinStockLevel) {
                                # stock level below minimum
                                $ProductLowOnStockNotification->addObject($Product);
                            } else {
                                # stock level equal or above minimum
                                $ProductLowOnStockNotification->deleteObject($Product);
                            }
                        } else {
                            # undefined new minimum
                            $ProductLowOnStockNotification->deleteObject($Product);
                        }
                    } elsif (defined $OldMinStockLevel) {
                        # operate with old min stock level
                        if ($NewStockLevel < $OldMinStockLevel) {
                            # stock level below minimum
                            $ProductLowOnStockNotification->addObject($Product);
                        } else {
                            # stock level equal or above minimum
                            $ProductLowOnStockNotification->deleteObject($Product);
                        }
                    }
                } else {
                    # undefined new stock level
                    $ProductLowOnStockNotification->deleteObject($Product);
                }
            } elsif (exists $hNewValues->{'StockLevelAlert'} && ((!defined $OldMinStockLevel) || ($hNewValues->{'StockLevelAlert'} != $OldMinStockLevel))) {
                # no new stock level --> new minimum
                my $NewMinStockLevel = $hNewValues->{'StockLevelAlert'};
                if (defined $NewMinStockLevel) {
                    # operate with old stock level
                    if (defined $OldStockLevel) {
                        if ($OldStockLevel < $NewMinStockLevel) {
                            # stock level below minimum
                            $ProductLowOnStockNotification->addObject($Product);
                        } else {
                            # stock level equal or above minimum
                            $ProductLowOnStockNotification->deleteObject($Product);
                        }
                    }
                } else {
                   # undefined new minimum
                   $ProductLowOnStockNotification->deleteObject($Product);
                }
            }
        }
    }

    # test here if we have to delete an old energy label file or manufacturer information
    # because of inheriting attributes, we delete only the fil from _$Attributename
    foreach my $AttributeName (qw(EnergyLabelSourceFile ProductDataSheet)) {
        if (exists $hNewValues->{$AttributeName}){
            my $AttributeFile = $Product->get('_'.$AttributeName,$LanguageID);
            if ($hNewValues->{$AttributeName} and $AttributeFile) {
                my $FileName = $Product->get('PublicPath') .'/'. $AttributeFile;
                DeleteFile( $FileName ) unless $AttributeFile eq $hNewValues->{$AttributeName};
            }
        }
    }


    # we do not update subproducts if we have a unity shop
    return if $Product->getSite()->get('IsUnityRegistered');


    # is this a master product and do we have sub products?
    return if $Product->superProduct;

    my $aSubProducts = $Product->get('SubProducts');
    return unless scalar @$aSubProducts;

    # add userdefined bool attributes to the list of attributes to update on subproduct
    my @UserDefinedBoolAttributes;
    foreach my $Key (keys %$hNewValues) {
        my $Attribute = $Product->class->attribute($Key);
        next unless $Attribute->type eq 'Boolean';
        next unless $Attribute->get('IsUserDefined');
        push @UserDefinedBoolAttributes, $Key ;
    }

    # get attributes from master product
    my $hMasterValues = $Product->get([@FROM_MASTERPRODUCT_INHERITABLE_ATTRIBUTES_ON_UPDATE, @UserDefinedBoolAttributes]);

    # filter out changed values
    my $hNewSubProductValues;
    foreach my $AttrName (keys %$hMasterValues) {
        if (defined $hNewValues->{$AttrName} and $hMasterValues->{$AttrName} != $hNewValues->{$AttrName}  ){
            $hNewSubProductValues->{$AttrName} = $hNewValues->{$AttrName};
        }
    }
    return unless defined $hNewSubProductValues;

    # set changed attributes on each sub product
    foreach my $SubProduct (@$aSubProducts) {
        my $hNewValuesToSet  = _PrepareIsVisible($SubProduct, $hNewSubProductValues);
        _PrepareIsNew($SubProduct, $hNewValuesToSet);
        $SubProduct->set($hNewValuesToSet);
    }

}

#========================================================================================
# function     OnAfterUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateProduct( $hValues );
#----------------------------------------------------------------------------------------
# description  Inherites boolean attrbiutes from the master to sub products
#               (Hook OBJ_AfterUpdateProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $hNewValues = $hParams->{'Values'};
    # set attribute EnergyLabelHighestEfficiencyClass to null if we have not an file specified on attribute EnergyLabelSourceFile
    if (exists $hNewValues->{'EnergyLabelSourceFile'}){
        if(! $Product->get('EnergyLabelSourceFile')){
            $Product->set({
                'EnergyLabelHighestEfficiencyClass' => undef,
                'EnergyLabelsString' => undef
            });
        }
    }

    if ($Product->getSite()->get('IsUnityRegistered')){
        my $SuperProduct = $Product->superProduct;
        if ( (defined $hNewValues->{'IsVisible'}) and $SuperProduct ) {
            if ( $hNewValues->{'IsVisible'} ) {
                # make the master product visible if variations are set visible - UNITY-3003
                $SuperProduct->set( { 'IsVisible' => 1 } );
            } else {
                # set master product invisible if all variations are invisible for unity
                $SuperProduct->set( { 'IsVisible' => 0 } ) unless scalar @{ GetVisibleSubProductIDs($SuperProduct) };
            }
        }
    }

    return;
}

# test if we can set the sub product to IsVisible=1
# it is dependend from the product availability settings, minorder and the stock level
sub _PrepareIsVisible {
    my ($SubProduct,$hNewSubProductValues) = @_;

    my $hNewValuesToSet = CopyStructure($hNewSubProductValues);
    # do not check, if set to invisible
    return $hNewValuesToSet unless $hNewSubProductValues->{'IsVisible'};
    # do not check, if we have no stock level settings
    return $hNewValuesToSet unless defined $SubProduct->get('StockLevel');

    # test if we have the correct stock setting
    my $Shop = $SubProduct->getSite();
    my $ProductLowOnStockAction = $Shop->get('ProductLowOnStockAction');
    return $hNewValuesToSet unless $ProductLowOnStockAction == STOCK_NOT_VISIBLE;

    # test stocklevel and remove entry if stock level is 0
    my $StockLevel = $SubProduct->get('StockLevel');
    my $MinOrder = $SubProduct->get('MinOrder');

    # do not set IsVisible attribute
    if ( $StockLevel <= 0 or ( (defined $MinOrder) and ($StockLevel < $MinOrder) ) ) {
        delete $hNewValuesToSet->{'IsVisible'} ;
    }

    return $hNewValuesToSet;

}


# if the master is set to IsNew, the sub product can have the attribute NewnessDate set to a value.
# we have to reset the NewnessDate on the sub products if the master product IsNew value has changed
sub _PrepareIsNew {
    my ($SubProduct,$hNewValuesToSet) = @_;

    if (exists $hNewValuesToSet->{'IsNew'}) {
        $hNewValuesToSet->{'NewnessDate'} = undef;
    }

}

#========================================================================================
# function     OnDeleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct( $hValues );
#----------------------------------------------------------------------------------------
# description  deletes the product notifications
#               (Hook OBJ_DeleteProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};

    my $Shop = $Product->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        my $ProductOutOfStockNotification = $Shop->folder('ShopNotifications')->child('ProductOutOfStock');
        my $ProductLowOnStockNotification = $Shop->folder('ShopNotifications')->child('ProductLowOnStock');
        # change the status of the product notification for Unity shops
        # product out of stock
        $ProductOutOfStockNotification->deleteObject($Product);
        # product low on stock
        $ProductLowOnStockNotification->deleteObject($Product);
    }
}

#========================================================================================
# function     OnInsertProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInsertProduct( $hValues );
#----------------------------------------------------------------------------------------
# description  activates the product notifications
#               (Hook OBJ_InsertProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};

    my $Shop = $Product->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        my $StockLevel = $hValues->{'StockLevel'};
        my $MinStockLevel = $hValues->{'StockLevelAlert'};
        # change the status of the product out of stock notification for Unity shops
        # product out of stock
        if (defined $StockLevel && $StockLevel <= 0) {
            my $ProductOutOfStockNotification = $Shop->folder('ShopNotifications')->child('ProductOutOfStock');
            $ProductOutOfStockNotification->addObject($Product);
        }
        # product low on stock
        if (defined $MinStockLevel && defined $StockLevel && $StockLevel < $MinStockLevel) {
            my $ProductLowOnStockNotification = $Shop->folder('ShopNotifications')->child('ProductLowOnStock');
            $ProductLowOnStockNotification->addObject($Product);
        }
    }
}

1;


