#==============================================================================
# package      DE_EPAGES::Product::UI::DownloadProduct
# state        private
#==============================================================================
package DE_EPAGES::Product::UI::DownloadProduct;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError HandleError );
use DE_EPAGES::Core::API::File qw ( fsplit GetFileContent GetFileSize MoveFile );
use DE_EPAGES::Core::API::IDN qw ( EncodeDomainInUrl );
use DE_EPAGES::Core::API::MP3 qw ( CutMp3 );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw ( TestUrl );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Presentation::API::Upload qw ( Upload );
use DE_EPAGES::Product::API::Constants qw (CONVERSION_FACTOR_BYTE);
use DE_EPAGES::Product::API::Table::LineItemProductMaxDownload qw (
    ExistsLineItemProductMaxDownload
    GetInfoLineItemProductMaxDownload
    ExistsLineItemProductMaxDownloadByLinkGUID
    GetInfoLineItemProductMaxDownloadByLinkGUID



);
use HTTP::Status;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'InvalidFileID' => 'Invalid file ID #FileID for product #ProductID',
        'InvalidFileName' => 'Invalid file name: #FileName',
        'ProductDeleted' => 'Product of line item #LineItemID has been deleted',
    }
}
#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================

#========================================================================================
# function     SaveDownloadProductMaps
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveDownloadProductMaps( $Servlet );
#----------------------------------------------------------------------------------------
# description  Saves files and attributes to product.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub SaveDownloadProductMaps {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Form = $Servlet->form;
    my $hFormValues = $Form->form($Product, 'SaveDownloadProduct');

    # Save Product attributes.
    $Product->set({
        'MaxDownloadTime' => $hFormValues->{'MaxDownloadTime'},
        'MaxDownloadCount' => $hFormValues->{'MaxDownloadCount'},
    }) unless $Servlet->params('useUpload');

    # if we have an ID, the map with this id mus be updated (filesize, targeturl)
    my $MapIDToUpdate = $hFormValues->{'UpdateDownloadProductMapID'};

    # Save file or URL.
    if( $hFormValues->{'DownloadFile'} ) {


        # upload file normally or as temp file, if we have to refresh the existing file
        my $PrivatePath = $Product->get('PrivatePath') . '/';
        my ($UploadedFile, $hResponseContent) = Upload($Product, $Form, {
            'FormName' => 'SaveDownloadProduct',
            'InputName' => 'DownloadFile',
            'FileName' => $MapIDToUpdate ? 'tmp_upload' : undef,
            'Index' => 0,
            'Path' => $PrivatePath
        });
        $Servlet->vars('ResponseContent', $hResponseContent);

        my $FileName = $UploadedFile;

        # get the target file name and move the uploaded temp file to the target filename
        if ($MapIDToUpdate) {
            my $Class = LoadClassByAlias('DownloadProductMap');
            my $DownloadProductMap = $Class->loadObject($MapIDToUpdate);
            my $UpdateTargetURL = $DownloadProductMap->get('TargetUrl');

           # move temp_upload to the new filename
            my $ExistingFile = $PrivatePath . $UpdateTargetURL;
            my $NewUploadedFile =  $PrivatePath . $FileName;
            MoveFile($NewUploadedFile,$ExistingFile,{});
            $FileName = $UpdateTargetURL;
        }

        eval {
            $self->_SaveDownloadFile( $Product, $FileName );
        };
        HandleError( OnError => sub {
            my ($Error) = @_;
            my $DownloadProductMapObject = $Error->vars->{'Object'};

            # delete me
            $DownloadProductMapObject->delete if (defined $DownloadProductMapObject);
            $Product->renumberDownloadProductMaps;

            # execute form error
            if( $Error->code eq 'InvalidMP3File' ) {
                $Servlet->form->addFormError({
                    'Reason'=> 'InvalidMP3File',
                    'Form'  => 'SaveDownloadProduct',
                    'Name'  => 'DownloadFile',
                });
                return 'FIXED';
            }
        });
        $Servlet->form->executeFormError();
    }# end DownloadFile


    elsif( $hFormValues->{'DownloadFileUrl'} ) {
        $self->_SaveDownloadUrl( $Product, $hFormValues->{'DownloadFileUrl'},$MapIDToUpdate );
    }

    # Save positions.
    my $Class = LoadClassByAlias('DownloadProductMap');
    foreach my $hEntry (@{$hFormValues->{'DownloadProductMaps'}}) {
        my $ObjectID = $hEntry->{'DownloadProductMapID'};
        delete $hEntry->{'DownloadProductMapID'};
        my $DownloadProductMap = $Class->loadObject($ObjectID);

        # security check
        Error('InvalidFileID', { 'FileID' => $ObjectID, 'ProductID' => $Product->id })
            unless $Product->id == $DownloadProductMap->get('DownloadProduct')->id;

        # update Position
        $DownloadProductMap->set($hEntry);
    }
    $Product->renumberDownloadProductMaps;
}

sub _createMP3Preview {
    my $self = shift;
    my ($Product, $File, $DownloadProductMapObject) = @_;

    my (undef, $FileName, $Extension) = fsplit($File);
    return if lc($Extension) ne '.mp3';

    $FileName .= $Extension;
    my $NewFile = $Product->get('PublicPath') . '/' . $FileName;
    my $DownloadPreviewLength = EpagesConfig()->get( 'DE_EPAGES::Product', 'DownloadPreviewLength', '00.30' ); ##no critic qw(ProhibitMagicNumbers)

    CutMp3($File, $NewFile, '00.00', $DownloadPreviewLength, $DownloadProductMapObject);
}

# save a download file
sub _SaveDownloadFile {
    my $self = shift;
    my ($Product, $FileName) = @_;

    my $PrivatePath = $Product->get('PrivatePath');
    my $FilePath = $PrivatePath . '/' . $FileName;
    my $FileSize = GetFileSize($FilePath) / CONVERSION_FACTOR_BYTE;

    my $DownloadProductMapObject = $self->_SetDownloadProductMap( $Product, 0, $FileName, $FileSize );

    # create preview for mp3-files
    $self->_createMP3Preview($Product, $FilePath, $DownloadProductMapObject);

    return;
}

# save a download URL
sub _SaveDownloadUrl {
    my $self = shift;
    my ($Product, $TargetUrl,$MapIDToUpdate) = @_;

    # get file size with HTTP HEAD
    # in case of an error, use undef as file size
    my $UserAgent = LWP::UserAgent->new(
        timeout => 10,
        protocols_allowed => ['ftp', 'http', 'https'],
    );
    my $Request = HTTP::Request->new('HEAD', $TargetUrl);
    my $Response = $UserAgent->request( $Request );
    my $ContentLenth = $Response->content_length;
    my $FileSize = defined $ContentLenth ? $ContentLenth / CONVERSION_FACTOR_BYTE : undef;

    if ($MapIDToUpdate) {
        $self->_UpdateDownloadProductMap( $Product, 1, $TargetUrl, $FileSize,$MapIDToUpdate );
    }else {
        $self->_SetDownloadProductMap( $Product, 1, $TargetUrl, $FileSize );
    }

    return;
}

# update or insert a DownloadProductMap object
sub _SetDownloadProductMap {
    my $self = shift;
    my ($Product, $IsExternal, $TargetUrl, $FileSize) = @_;

    my $MapClass = LoadClassByAlias('DownloadProductMap');
    my $aMapClassObjects = $MapClass->getObjects('ByDownloadProductIDTargetUrl', [$Product->id, $TargetUrl]);
    my $DownloadProductMapObject;
    if (scalar @$aMapClassObjects) {
        $aMapClassObjects->[0]->set({'FileSize' => $FileSize});
    }
    else {
        $DownloadProductMapObject = $MapClass->insertObject({
            'DownloadProduct'  => $Product,
            'TargetUrl' => $TargetUrl,
            'IsExternal' => $IsExternal,
            'FileSize' => (sprintf '%.0f', $FileSize),
            'Position' => 9999,
        });
    }

    return $DownloadProductMapObject;
}

# simply update the DownloadProductMap by the given DownloadProductMapID
sub _UpdateDownloadProductMap {
    my $self = shift;
    my ($Product,$IsExternal, $TargetUrl, $FileSize,$DownloadProductMapID) = @_;

    my $Class = LoadClassByAlias('DownloadProductMap');
    my $DownloadProductMap = $Class->loadObject($DownloadProductMapID);

    $DownloadProductMap->set({'DownloadProduct'  => $Product,
                              'FileSize' => $FileSize,
                              'TargetUrl' => $TargetUrl,
                              'IsExternal' => $IsExternal,
                            });
    return;
}



#========================================================================================
# function     DeleteDownloadProductMaps
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteDownloadProductMaps( $Servlet );
#----------------------------------------------------------------------------------------
# description  Deletes files from product and renumbers positions.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub DeleteDownloadProductMaps {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    my $Class = LoadClassByAlias('DownloadProductMap');
    my $Form = $Servlet->form;
    my $hValues = $Form->form($Servlet->object, 'ListedObjects');
    my @ListObjects = map  {   $Class->loadObject( $_->{'ListObjectID'} ) }
                      grep { $Class->existsObject( $_->{'ListObjectID'} ) }
                      @{$hValues->{'ListObjectIDs'}};

    return unless scalar @ListObjects;

    if ($Servlet->user->get('DeleteConfirmation')
        and not $Servlet->params('ACCEPT')) {

        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @ListObjects]);
        $Servlet->vars('DELETE', 1);
        return;
    } else {
        $Servlet->vars('DELETE', 0);
        $self->DeleteListedObjects($Servlet, $Class);
        $Product->renumberDownloadProductMaps;
    }

    return;
}

#========================================================================================
# function     testObjectDeleteListedObjects
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->testObjectDeleteListedObjects( $Object );
#----------------------------------------------------------------------------------------
# description  Returns the object to test the Delete permission.
#----------------------------------------------------------------------------------------
# input        $Object | object to delete | object
#----------------------------------------------------------------------------------------
# return       $Object | test object | object
#========================================================================================
sub testObjectDeleteListedObjects {
    my $self = shift;
    my ($Object) = @_;
    return $Object->get('DownloadProduct');
}

#========================================================================================
# function     MBODownloadFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->MBODownloadFile( $Servlet );
#----------------------------------------------------------------------------------------
# description  Exports the file from a product (only for MBO-Users).
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub MBODownloadFile {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $FileID = $Servlet->params('DownloadFileID');

    $self->_DownloadFile( $Servlet, $Product, $FileID );
    return;
}

#========================================================================================
# function     ViewDownloadFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewDownloadFile( $Servlet );
#----------------------------------------------------------------------------------------
# description  Exports the file from a product if MaxDownloadTime is in future and
#               substract 1 from MaxDownloadCount. Checks the validity of the request URL
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub ViewDownloadFile {
    my $self = shift;
    my ($Servlet) = @_;

    my $LineItemProduct = $Servlet->object;
    my $FileID;

    if ($Servlet->params('DownloadID')) {
        my $DownloadID = $Servlet->params('DownloadID');
        if (ExistsLineItemProductMaxDownloadByLinkGUID($DownloadID)) {
            my $hInfo = GetInfoLineItemProductMaxDownloadByLinkGUID($DownloadID);
            die 'line item id doesn\'t match' unless $LineItemProduct->id == $hInfo->{'LineItemProductID'};
            $FileID = $hInfo->{'DownloadProductMapID'};
        }
        else {
            # Download GUID wasn't found
            Error('OBJECT_NOT_FOUND');
        }
    }
    else {
        $FileID = $Servlet->params('DownloadFileID');

        if (ExistsLineItemProductMaxDownload($LineItemProduct->id, $FileID)) {
            my $hInfo = GetInfoLineItemProductMaxDownload($LineItemProduct->id, $FileID);

            if (defined $hInfo->{'LinkGUID'}) {
                # FileID was provided, but a GUID is stored
                # this is treated as an unallowed URL manipulation
                Error('OBJECT_NOT_FOUND');
            }
        }
    }

    my $LineItemContainer = $LineItemProduct->get('LineItemContainer');
    my $Shop = $LineItemProduct->getSite();

    my $Order = $LineItemContainer->alias eq 'BundledProductContainer' ? $LineItemContainer->parent->parent->parent : $LineItemContainer->parent;
    $Servlet->params('ErrorObjectID', $Order->id);

    $Servlet->form->executeFormError({
        'Reason'=> 'UnpaidOrder',
        'Form'  => 'SaveDownloadProduct',
        'Name'  => 'Files',
    }) unless $Shop->get('AllowDownloadOfUnpaidProducts') or defined $Order->get('PaidOn');

    my $Product = $LineItemProduct->get('Product');
    Errror('ProductDeleted', { 'LineItemID' => $LineItemProduct->id }) unless defined $Product;

    my $EndDateTime = $LineItemProduct->get('EndDownloadTime');
    $Servlet->form->executeFormError({
        'Reason'=> 'TimeIsUp',
        'Form'  => 'SaveDownloadProduct',
        'Name'  => 'Files',
    }) if defined $EndDateTime and $EndDateTime < GetCurrentDateTime();

    my $hMaxDownloads = $LineItemProduct->get('MaxDownloadCounts');
    $Servlet->form->executeFormError({
        'Reason'=> 'LimitReached',
        'Form'  => 'SaveDownloadProduct',
        'Name'  => 'Files',
    }) if defined $hMaxDownloads and defined $hMaxDownloads->{$FileID} and $hMaxDownloads->{$FileID} <= 0;

    $Servlet->monitor->blockTimeOut;

    $self->_DownloadFile( $Servlet, $Product, $FileID );

    # reduce the download counter
    if (defined $hMaxDownloads and defined $hMaxDownloads->{$FileID}) {
        $hMaxDownloads->{$FileID} = $hMaxDownloads->{$FileID} - 1;
        $LineItemProduct->set({'MaxDownloadCounts' => $hMaxDownloads});
    }
    return;
}

# sends the file to the browser
sub _DownloadFile {
    my $self = shift;
    my ($Servlet, $Product, $FileID) = @_;

    my $DownloadProductMap;
    eval {
        $DownloadProductMap = LoadClassByAlias('DownloadProductMap')->loadObject($FileID);
    };
    if (ExistsError()) {
        Error('OBJECT_NOT_FOUND');
    }

    # security check
    my $DownloadProductID = $DownloadProductMap->get('DownloadProduct')->id;
    unless ($Product->id == $DownloadProductID) {
        my $SuperProduct = $Product->get('SuperProduct');
        Error('InvalidFileID', { 'FileID' => $FileID, 'ProductID' => $Product->id }) if !defined $SuperProduct || $SuperProduct->id != $DownloadProductID;
    }


    if( $DownloadProductMap->get('IsExternal') ) {
        my $TargetUrl = $DownloadProductMap->get('TargetUrl');
        #validate target URL
        my $isValidURL = 1;
        eval { TestUrl('ESD Url', $TargetUrl); };
        if ( ExistsError() ) {
            $isValidURL = 0;        #wrong URL format
        } else {
            #optional HEAD request to check if URL available?
        }
        Error( 'InvalidFileName', { 'FileName' => $TargetUrl } ) unless $isValidURL;
        my $Response = $Servlet->response;
        $Response->code(RC_FOUND);
        $Response->header( 'Location' => EncodeDomainInUrl( $TargetUrl ) );
    }
    else {
        my $FileName = $DownloadProductMap->get('FileName');
        Error( 'InvalidFileName', { 'FileName' => $FileName } ) if $FileName =~ /[\\\/]/;
        my $FilePath = $Product->get('PrivatePath') .'/'. $FileName;

        my $rContent= GetFileContent( $FilePath, {'binary'=>1} );
        my $Response = $Servlet->response;
        $Response->content( $$rContent );
        $Response->header( 'Content-Type' => 'application/octet-stream',
                           'Content-Disposition' => 'attachment; filename='.$FileName,
                           'Content-Description' => 'Downloadfile' );

    }

    $Servlet->vars('ExitAfterEvent',1);
    return;
}

1;
