#========================================================================================
# package      DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements the high level interface for import and export of
#               objects to CSV files.
#========================================================================================
package DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;

use strict;
use DE_EPAGES::Core::API::CSV qw( CombineCSV );
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestObject );
use DE_EPAGES::Object::API::Table::Language qw( GetInfoLanguagesByCode2 );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::ShopCSVExportImport::API::Constants qw( @SHOP_CSV_WARNING_CODES );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver= DE_EPAGES::Tax::API::CSV::TaxExportImport->new(%options);
#----------------------------------------------------------------------------------------
# example      $Driver= DE_EPAGES::Tax::API::CSV::TaxExportImport->new;
#----------------------------------------------------------------------------------------
# description  Creates a new csv export driver.
#----------------------------------------------------------------------------------------
# input        %options |  exporter options | hash
# return       $Driver | import/export driver | object
#========================================================================================
sub new {
    my $class = shift;
    my %params = @_;
    my $self = bless \%params, $class;
    return $self;
}

#========================================================================================
# function     warningCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aWarningCodes = $Driver->warningCodes();
#----------------------------------------------------------------------------------------
# description  Returns the warning codes for csv export.
#----------------------------------------------------------------------------------------
# return       $aWarningCodes | warning codes | array.string
#========================================================================================
sub warningCodes {
    return [ @SHOP_CSV_WARNING_CODES ];
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    die 'please implement this function';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    die 'please implement this function';
}

#========================================================================================
# function     csvExport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->csvExport($Shop, $PathFileName, $hOptions, $aObjects);
#----------------------------------------------------------------------------------------
# description  Exports objects to a CSV file.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $PathFileName   | CSV file name          | string
# input        $hOptions    | defines what to export and how
#                   <ul>
#                       <li>LanguageCode2s - list of ISO language codes. defines which
#                           languages are exported. If the list is empty, then all
#                           languages are exported. - array of strings</li>
#                       <li>LocaleID - locale id - string</li>
#                   </ul> | ref.hash
# input        $aObjects       | list of exportable objects | ref.array.object
#========================================================================================
sub csvExport {
    my $self = shift;
    my ($Shop, $PathFileName, $hOptions, $aObjects) = @_;
    TestObject('Shop', $Shop);
    my $LocaleID        = $hOptions->{'LocaleID'};
    my $aLanguageCode2s = $hOptions->{'LanguageCode2s'};
    my $Driver = $self->exportDriver->new;

    $Driver->exportCSV(
        $PathFileName,
        $hOptions,
        $self->csvAttributes($Shop, $LocaleID, $aLanguageCode2s),
        $Shop,
        $self->getExportObjects($Shop, $aObjects));
    return;
}

#========================================================================================
# function     getExportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->getExportObjects($Shop, $aObjects);
#----------------------------------------------------------------------------------------
# description  Returns list of objects to export.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $aObjects       | list of exportable objects | ref.array.integer
# return       $aExportObjects | edited list of exportable objects | ref.array
#========================================================================================
sub getExportObjects {
    my $self = shift;
    my ($Shop, $aObjects) = @_;
    return $aObjects;
}

#========================================================================================
# function     csvImport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->csvImport($Shop, $PathFileName, $hOptions);
#----------------------------------------------------------------------------------------
# description  Imports objects from a CSV file into a shop.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $PathFileName   | CSV file name          | string
# input        $hOptions       | defines what to export and how
#                   <ul>
#                       <li>LanguageCode2s - list of ISO language codes. defines which
#                           languages are imported. If the list is empty, then all
#                           languages are imported. - array of strings</li>
#                       <li>LocaleID - locale id - string</li>
#                       <li>... other options</li>
#                   </ul> | ref.hash
#========================================================================================
sub csvImport {
    my $self = shift;
    my ($Shop, $PathFileName, $hOptions) = @_;
    TestObject('Shop', $Shop);
    my $LocaleID        = $hOptions->{'LocaleID'};
    my $aLanguageCode2s = $hOptions->{'LanguageCode2s'};
    my $IsImport        = 1;
    $self->importDriver->resetFlagsAndWarnings();
    $self->importDriver->importCSV($PathFileName, $hOptions, $self->csvAttributes($Shop, $LocaleID, $aLanguageCode2s,$IsImport), $Shop);
    return;
}

#========================================================================================
# function     getAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aAttributes = $Driver->getAttributes($Shop);
#----------------------------------------------------------------------------------------
# description  Return list of exportable attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $aAttributes | list of attributes | ref.array.object
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Shop) = @_;
    die 'please implement this function';
}

#========================================================================================
# function     csvAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aCSVAttributes = $Driver->csvAttributes($Shop, $LocaleID, $aLanguageCode2s,$IsImport);
#----------------------------------------------------------------------------------------
# description  Return list of exportable csv attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LocaleID | locale identifier (e.g. en_GB) | string
# input        $aLanguageCode2s | array of languages identified by code2 | ref.array.string
# input        $IsImport | true if this is an import ? | boolean
# return       $aAttributes | list of attributes | ref.array.hash
#========================================================================================
sub csvAttributes {
    my $self = shift;
    my ($Shop, $LocaleID, $aLanguageCode2s,$IsImport) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    my $ahLanguages = $self->getLanguages($Shop, $aLanguageCode2s);
    my $aAttributes = $self->getAttributes($Shop);

    my @CSVAttributes;
    foreach my $Attribute ( @$aAttributes ) {
        push @CSVAttributes, @{ $self->getAttributeInfo($Attribute, $hLocale->{'LanguageID'}, $ahLanguages)};
    }
    return \@CSVAttributes;
}

#========================================================================================
# function     getLanguages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getLanguages($Shop, $aLanguageCode2s);
#----------------------------------------------------------------------------------------
# description  Returns array of languages for which localized attributes should exported.
#----------------------------------------------------------------------------------------
# input        $Shop | object of shop | object
# input        $aLanguageCode2s | list of code2 of languages | ref.array.string
# return       $ahLanguages | list of information for exportable languages
#                   <ul>
#                       <li>LanguageID    - language id - int
#                       <li>Code2         - iso-639-1 language code - string
#                       <li>English       - english name of language - string
#                       <li>Native        - local name of language - string
#                   </ul> | ref.array.hash
#========================================================================================
sub getLanguages {
    my $self = shift;
    my ($Shop, $aLanguageCode2s) = @_;

    my $ahLanguages;
    if (scalar(@$aLanguageCode2s)) {
        foreach my $LanguageCode2 ( @$aLanguageCode2s ) {
            push @$ahLanguages, @{ GetInfoLanguagesByCode2($LanguageCode2) };
        }
    } else {
        $ahLanguages = $Shop->get('Languages'); # get all shop languages
    }
    return $ahLanguages;
}

#========================================================================================
# function     getAttributeInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahInfo = $self->getAttributeInfo($Attribute, $LanguageID, $ahLanguages, $hInfo);
#----------------------------------------------------------------------------------------
# description  Returns hash with information about the attribute.
#----------------------------------------------------------------------------------------
# input        $Attribute | attribute object | object
# input        $LanguageID | language identifier | integer
# input        $ahLanguages | list of information for exportable languages
#                   <ul>
#                       <li>LanguageID    - language id - int
#                       <li>Code2         - iso-639-1 language code - string
#                       <li>English       - english name of language - string
#                       <li>Native        - local name of language - string
#                   </ul> | ref.array.hash
# input        $hInfo | information for result | ref.hash
# return       $ahInfo | information of attribute for each language
#                   <ul>
#                       <li>Type - data type  - string
#                       <li>IsObject - is object - boolean
#                       <li>Position - position (column in csv file) - integer
#                       <li>Alias - alias string - string
#                       <li>Name - name - string
#                       <li>Attribute - attribute object - object
#                       <li>IsBaseClassAttribute - class is special class - boolean
#                       <li>LanguageID - language identifier for localized attributes - integer
#                       <li>HeaderString - localized name for attribute - string
#                       <li>AliasString  - alias name for attribute (identifier) - string
#                   </ul> | ref.array.hash
#========================================================================================
sub getAttributeInfo {
    my $self = shift;
    my ($Attribute, $LanguageID, $ahLanguages, $hInfo) = @_;

    my %Info;
    %Info = %$hInfo if defined $hInfo;
    $Info{$_} = $Attribute->get($_) foreach grep { !exists $Info{$_} } qw (Type IsObject Position);

    $Info{'Position'} = 0 unless exists $Info{'Position'};
    $Info{'Alias'} = $Attribute->alias unless exists $Info{'Alias'};
    $Info{'Name'} = $Attribute->get('NameOrAlias', $LanguageID) unless exists $Info{'Name'};
    $Info{'Attribute'} = $Attribute;
    $Info{'IsBaseClassAttribute'} = 1 unless exists $Info{'IsBaseClassAttribute'};

    my @Infos;

    if ($Info{'Type'} =~ /Localized/ and $Info{'Type'} ne 'PreDefLocalizedString') {
        foreach my $hLanguage (@$ahLanguages) {
            my $hHeaderStrings = $self->getHeaderStrings( \%Info, $LanguageID, { 'hLanguage' => $hLanguage});
            $Info{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
            $Info{'AliasString'}  = $hHeaderStrings->{'AliasString'};
            $Info{'LanguageID'} = $hLanguage->{'LanguageID'};
            push @Infos, {%Info};
        }
    }
    elsif( $Info{'Type'} eq 'Price') {
        # can't call getHeaderStrings(), bacause there's no CurrencyID
        push @Infos, {%Info};
    }
    else {
        my $hHeaderStrings = $self->getHeaderStrings( \%Info, $LanguageID, {});
        $Info{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
        $Info{'AliasString'}  = $hHeaderStrings->{'AliasString'};
        push @Infos, {%Info};
    }
    return \@Infos;
}

#========================================================================================
# function     getHeaderStrings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getHeaderStrings($hAttribute, $LanguageID, $hParams);
#----------------------------------------------------------------------------------------
# description  Returns hash with names for first line in csv (headline). Names are native
#               and localized.
#----------------------------------------------------------------------------------------
# input        $hAttribute | attribute information
#                   <ul>
#                       <li>Alias - alias of attribute - string
#                       <li>Name - name of attribute - string
#                       <li>IsBaseClassAttribute - attribute is from objects class - boolean
#                       <li>Attribute - attribute object (optional) - object
#                   </ul> | ref.hash
# input        $LanguageID   | language identifier | integer
# input        $hParams    | used for localized attributes or prices (optional)
#                   <ul>
#                       <li>CurrencyID - currency identifier - string
#                       <li>TaxModelText - net or gross - string
#                       <li>hLanguage - code2 | native - ref.hash
#                   </ul> | ref.hash
# return       $hHeaderStrings | localized name and identifier for headline
#                   <ul>
#                       <li>HeaderString - localized name for attribute - string
#                       <li>AliasString  - alias name for attribute (identifier) - string
#                   </ul> | ref.hash
#========================================================================================
sub getHeaderStrings {
    my $self = shift;
    my ($hAttribute, $LanguageID, $hParams) = @_;
    TestString('hAttribute.Alias', $hAttribute->{'Alias'});
    TestString('hAttribute.Name', $hAttribute->{'Name'});

    my @AliasParts  = ($hAttribute->{'Alias'});
    my @NativeParts = ($hAttribute->{'Name'});

    # add class if it is special class
    if ($hAttribute->{'IsBaseClassAttribute'} == 0) {
        my $AttributeClass = exists $hAttribute->{'Class'} ? $hAttribute->{'Class'} : $hAttribute->{'Attribute'}->get('AttributeClass');
        unshift(@AliasParts,  $AttributeClass->alias );
        unshift(@NativeParts, $AttributeClass->get('NameOrAlias', $LanguageID));
    }

    my $AttributeType = $hAttribute->{'Type'} // '';
    if ($AttributeType eq 'Price') {
        # e.g. "ListenPreis/EUR/gross [ListPrice/EUR/gross]"
        TestString('hParams.CurrencyID', $hParams->{'CurrencyID'});
        TestString('hParams.TaxModelText', $hParams->{'CurrencyID'});
        push @AliasParts,  ($hParams->{'CurrencyID'}, $hParams->{'TaxModelText'});
        push @NativeParts, ($hParams->{'CurrencyID'}, $hParams->{'TaxModelText'});
    } elsif ( $AttributeType =~ /^Localized(String|File)$/ ) {
        # e.g. "Zelt/Innenzeltmaterial/English [Tent/MaterialInside/en]"
        push @AliasParts,  $hParams->{'hLanguage'}{'Code2'};
        push @NativeParts, $hParams->{'hLanguage'}{'Native'};
    } #else { # e.g. "Bezeichner [Alias]" }

    my $AliasString = CombineCSV( \@AliasParts, { 'sep_char' => '/' } );
    return {
        'HeaderString' => join( '/', @NativeParts ). " [$AliasString]",
        'AliasString' => $AliasString
    };
}

1;
