package DE_EPAGES::ShopCSVExportImport::UI::CSVExportImport;
use base qw ( DE_EPAGES::CSVExportImport::UI::CSVExportImport );

use strict;
use Archive::Extract;
use DE_EPAGES::Core::API::Error qw( Error ExistsError );
use DE_EPAGES::Core::API::File qw( DeleteDirectory DeleteFile fsplit GetFileSize );
use DE_EPAGES::Core::API::Log qw( LogDebug );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename );
use DE_EPAGES::CSVExportImport::API::Constants qw( MAX_WARNINGS );
use DE_EPAGES::Presentation::API::Upload qw( Upload );
use DE_EPAGES::ShopCSVExportImport::API::CSV::ShopExportImport qw ( ExportCSV ImportCSV );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
#==============================================================================
# Helper methods
#==============================================================================

sub createDriver {
    die 'abstract method \'createDriver\' was not implemented';

# example:
#    return DE_EPAGES::Product::API::CSV::ProductExportImport->new;
}

sub type {
    die 'abstract method \'type\' was not implemented';

# example:
#    return 'Product';
}

sub ExportFileName {
    my $self = shift;
    my ($FileName) = @_;
    return $FileName ? Unicode2Filename($FileName) : 'export';
}

#==============================================================================
# Constants
#==============================================================================

my %DELIMITERS = (
    'Comma' => ',',
    'Semicolon' => ';',
    'Tabulator' => "\t",
);

#==============================================================================
# Event handlers
#==============================================================================

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->params( 'CSVExportImportType', $self->type );
    return;
}

sub CSVExportImport {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hForm = $Servlet->form->form($Shop, 'CSVExportImport');

    if (defined $hForm->{'Import'}) {
        $self->CSVImport($Servlet);
    }
    elsif (defined $hForm->{'Export'}) {
        $self->CSVExport($Servlet);
    }

    return;
}

sub CSVExport {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hForm = $Servlet->form->form($Shop, 'CSVExportImport');
    my @ObjectIDs = $Servlet->form->value('SelectedObjects');
    my @Objects;

    $Servlet->params('CSVFileName', $self->ExportFileName($Servlet->params('ExportFileName')));

    #check if selective csv export is required
    if($Servlet->params('EXPORT_SELECTED_OBJECTS')) {
        foreach my $ObjectID (@ObjectIDs){
            my $Object = $Shop->loadObject($ObjectID, 'Object');
            push(@Objects, $Object->id);
        }
    }

    my $PathFileName = $self->getPrivateCSVExportFileName($Servlet);
    my $aLanguages = $hForm->{'CSVLanguage'} eq 'AllLanguages' ?  [] : [$hForm->{'CSVLanguage'}];
    my %Options = (
        'LanguageCode2s' => $aLanguages,
        'LocaleID' => $hForm->{'CSVLocale'},
    );
    $Options{'ExportImageURLs'} = $Servlet->params('ExportImageURLs') ? 1 : 0;
    $Options{'sep_char'} = $DELIMITERS{$hForm->{'CSVDelimiter'}} if $hForm->{'CSVDelimiter'};
    $Options{'encoding'} = $hForm->{'CSVEncoding'} if $hForm->{'CSVEncoding'};
    my $DriverPackage = $self->createDriver;
    my $aWarnings;
    if($Servlet->params('EXPORT_SELECTED_OBJECTS')){#selective csv export
        $aWarnings = ExportCSV($Shop, $PathFileName, $DriverPackage, \%Options, \@Objects);
    } else {#all objects should be exported
        $aWarnings = ExportCSV($Shop, $PathFileName, $DriverPackage, \%Options, undef );
    }
    LogDebug('Export Warnings', $aWarnings );

    $Servlet->vars( 'Redirect', 1);#show exportfile to user in following view action
    $Servlet->vars( 'CSVExportImportType', $self->type );
    $Servlet->form->mapFormError({  'Form'  => 'CSVExportImport',
                                    'Name'  => 'Export'}, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;

    #check if view action have to be changed after export
    if (defined $Servlet->params('ViewActionAbort') && defined $Servlet->params('ObjectIDAbort')){
        $Servlet->vars( 'ViewAction', $Servlet->params('ViewActionAbort'));
        $Servlet->vars( 'ObjectID', $Servlet->params('ObjectIDAbort'));
    }
    return;
}

sub CSVImport {
    my $self    = shift;
    my ($Servlet, $BatchHandlers) = @_;

    my $Shop = $Servlet->object;
    my $hForm = $Servlet->form->form($Shop, 'CSVExportImport');

    my $PathFileName = $self->getPrivateCSVImportFileName($Servlet);
    my ($Path, $FileName, $Extension) = fsplit($PathFileName);

    my (undef, $hResponseContent) = Upload($Shop, $Servlet->form, {
        'FormName' => 'CSVExportImport',
        'InputName' => 'NewImportFile',
        'Index' => 0,
        'Path' => $Path,
        'FileName' => $FileName,
        'Extension' => $Extension,
    });

    if($Shop->get('IsUnityRegistered') and $Servlet->params('MediaArchiveFile')){
        eval { 
            # for unity we upload media archive and extract to the MediaGalleryUnity path
            my $hMediaArchiveResponseContent = $self->_uploadAndExtractMediaArchive($Servlet); 
            $Servlet->vars('MediaArchiveResponseContent', $hMediaArchiveResponseContent);
        };

        if ( ExistsError() ) {
            $Servlet->form->executeFormError({
                'Reason'  => 'ARCHIVE_UPLOAD_FAILED',
                'Form'    => 'CSVExportImport',
                'Name'    => 'Import',
            });
        }
    }

    $Servlet->vars('ResponseContent', $hResponseContent);

    $Servlet->vars('ImportFileName', $hForm->{'NewImportFile'});
    $Servlet->vars('ImportFileSize', GetFileSize($PathFileName) / 1024); ## no critic qw( ProhibitMagicNumbers )

    my $aLanguages = $hForm->{'CSVLanguage'} eq 'AllLanguages' ?  [] : [$hForm->{'CSVLanguage'}];
    my %Options = (
        'LanguageCode2s' => $aLanguages,
        'LocaleID' => $hForm->{'CSVLocale'},
    );
    $Options{'sep_char'} = $DELIMITERS{$hForm->{'CSVDelimiter'}} if $hForm->{'CSVDelimiter'};
    $Options{'encoding'} = $hForm->{'CSVEncoding'} if $hForm->{'CSVEncoding'};
    # If the checkbox "SendEmailToUser" is deactivated we want to deactivate the email sending
    $Options{'deactivateEmails'} = 1 unless $hForm->{'SendEmailToUser'};

    my ($aWarnings, $TooManyWarnings);
    if(defined $BatchHandlers) {
        RunBatchProcess($BatchHandlers, sub {
            ($aWarnings, $TooManyWarnings) = ImportCSV($Shop, $PathFileName, $self->createDriver, \%Options);
        });
    } else {
        ($aWarnings, $TooManyWarnings) = ImportCSV($Shop, $PathFileName, $self->createDriver, \%Options);
    }

    if ($TooManyWarnings) {
        $Servlet->vars( 'MaxWarn', MAX_WARNINGS );
        $Servlet->form->addFormError({
            'Reason'  => 'TOO_MANY_WARNINGS',
            'Form'    => 'CSVExportImport',
            'Name'    => 'Import',
        });
    }

    if($Shop->get('IsUnityRegistered')){
        # delete content of UnityMediaGallery here
        my $UnityMediaGallery = $Shop->child('UnityMediaImportFiles');
        DeleteDirectory($UnityMediaGallery->absolutePath(),{'recursive' => 1});
    }

    LogDebug( 'Import Warnings', $aWarnings );
    $Servlet->params( 'CSVExportImportType', $self->type );
    $Servlet->form->mapFormError({ 'Form'  => 'CSVExportImport',
                                   'Name'  => 'Import'}, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;
    $Servlet->vars( 'IMPORT_OK', 1 );
    return;
}

sub _uploadAndExtractMediaArchive {
    my $self    = shift;
    my ($Servlet) = @_;

    # upload here also media files to the WebRoot for image references
    my $Shop = $Servlet->object;
    my $MediaArchivePath = $self->getUnityMediaGalleryArchivePath($Servlet);

    my (undef, $hResponseContent) = Upload($Shop, $Servlet->form, {
        'FormName' => 'CSVExportImport',
        'InputName' => 'MediaArchiveFile',
        'Index' => 0,
        'Path' => $MediaArchivePath,
    });
    # extract archive if we have one
    $self->_extractFilesFromArchive($hResponseContent->{'uploadedObjectFile'}, $MediaArchivePath);
    return $hResponseContent;
  
}

#========================================================================================
# §function     _extractFilesFromArchive
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Package->_extractFilesFromArchive( $FileName, $TargetFolder );
#----------------------------------------------------------------------------------------
# §description  Extracts all files from archive and deletes zip archive.
#----------------------------------------------------------------------------------------
# §input        $FileName | XML file name | string
# §input        $TargetFolder | target folder for extracted files | string
#========================================================================================
sub _extractFilesFromArchive {
    my $self = shift;
    my ($FileName, $TargetFolder) = @_;
    TestString( 'FileName', $FileName );
    
    my $AbsFile = $TargetFolder . $FileName;

    Error( 'FileNotFound', { FileName => $FileName } ) unless -f $AbsFile;

    #extract archive to target  folder
    my $ae = Archive::Extract->new(archive => $AbsFile);
    my $ok = $ae->extract( to => $TargetFolder );

    DeleteFile($AbsFile);

    Error( 'READ_ZIP_ERROR', { FileName => $FileName } ) unless $ok or $ae;
}

1;
