#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService12
# state        public
#----------------------------------------------------------------------------------------
# description  Provides web service access to create and delete shops
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::WebService::API::Client;
#               my $ShopConfigService = DE_EPAGES::WebService::API::Client
#                   ->uri('urn://epages.de/WebService/ShopConfigService/2017/11')
#                   ->proxy("http://localhost:80/epages/Site.soap");
#               $ShopConfigService->userinfo( "/Providers/$ProviderAlias/Users/$Login:$Password" );
#               $ShopConfigService->delete({
#                   Alias => 'Store.DemoShop'
#               });
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService12;
use base DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService11;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService12->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;

    $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ShopConfigService/2017/11',
        @Options
    ));
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopID = $ShopConfigService->create({
#                   ShopType => $ShopType,
#                   Database => $Database,
#                   Alias => $Alias,
#                   ShopAlias => $ShopAlias,
#                   IsClosed    => $IsClosed,
#                   DomainName  => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   WebServerScriptNamePart => $WebServerScriptNamePart,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   ImportFiles => $aImportFiles,
#                   PublicFilesDir => $PublicFilesDir,
#                   MerchantEMail => $MerchantEMail,
#                   ShopAddress_FirstName => $FirstName,
#                   ShopAddress_LastName => $LastName,
#                   ShopAddress_CountryID => $CountryID,
#                   ShopAddress_Street => $Street,
#                   ShopAddress_Zipcode => $Zipcode,
#                   ShopAddress_City => $City,
#                   ShopAddress_State => $State,
#                   Name => $Name,
#               })->result;
# example      my $ShopID = $ShopConfigService->create({
#                   ShopType => 'ECommerce100',
#                   Database => 'Store',
#                   Alias => 'DemoShop',
#                   ShopAlias => 'DemoShop',
#                   IsClosed    => 0,
#                   DomainName  => 'www.myshop.biz',
#                   HasSSLCertificate => 1,
#                   WebServerScriptNamePart => 'demoshop',
#                   IsTrialShop => 0,
#                   IsInternalTestShop => 0,
#                   ImportFiles => [
#                       '\\Server\Shared\Distributor\CommonData.xml',
#                       '\\Server\Shared\Distributor\ExampleProducts.xml',
#                       '\\Server\Shared\Distributor\Styles.xml',
#                   ],
#                   PublicFilesDir => '\\Server\Shared\Distributor\ShopImages',
#                   MerchantEMail => 'hab@ich.net',
#                   ShopAddress_FirstName => 'Max',
#                   ShopAddress_LastName => 'Mustermann',
#                   ShopAddress_CountryID => '276',
#                   ShopAddress_Street => 'Heinrich-Heine-Strae 1',
#                   ShopAddress_Zipcode => '07749',
#                   ShopAddress_City => 'Jena',
#                   ShopAddress_State => 'TH',
#                   Name => 'Milestones',
#               })->result;
#----------------------------------------------------------------------------------------
# description  Creates a new shop within the given database on behalf of the current
#               provider. The new shop is given the feature set of the specified shop
#               type. A given shop address and name are set as well.
#               Initial shop data are imported from import file(s). These import
#               files must be pre-configured corresponding to the shop's feature set.
#               Some example import files are located in the directory
#               %EPAGES_CARTRIDGES%/DE_EPAGES/DemoShop/Database/XML.
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                   <li>ShopType | shop type alias | string
#                   <li>Database | database identifier (store alias) | string
#                   <li>Alias | unique shop name for the provider | string
#                   <li>ShopAlias | unique shop name for the database | string
#                   <li>IsClosed | true if the new shop is closed by provider (optional, default: 0) | boolean
#                   <li>$DomainName | domain name to access the shop (optional) | string
#                   <li>HasSSLCertificate | true if the web server has an SSL certificate
#                       for the shop (optional) | string
#                   <li>WebServerScriptNamePart | script name part of the shop (optional) | string
#                   <li>IsTrialShop | true if the shop is in trial period (optional, default: 0) | boolean
#                   <li>IsInternalTestShop | true if the shop is used for test or support purposes (optional, default: 0) | boolean
#                   <li>aImportFiles | list of import files contianing initial shop date. These
#                       files must be accessible for the application servers that are
#                       responsible for the specified database | string
#                   <li>PublicFilesDir | directory on the application server where image files
#                       for the new shop are located. These files are copied to the
#                       shop directory. (optional) | string
#                   <li>MerchantEMail | merchant email address (optional) | string</li>
#                   <li>ShopAddress_FirstName | merchant first name | string</li>
#                   <li>ShopAddress_LastName | merchant last name | string</li>
#                   <li>ShopAddress_CountryID | merchant enumeric country code according to ISO 3166-1 | integer</li>
#                   <li>ShopAddress_Street | merchant street and house number | string</li>
#                   <li>ShopAddress_Zipcode | merchant zipcode | string</li>
#                   <li>ShopAddress_City | merchant city | string</li>
#                   <li>ShopAddress_State | merchant state code according to ISO 3166-2 | string</li>
#                   <li>Name | name of the shop | string</li>
#               </ul> | reference to hash
# return       $ShopID | object id of the shop within the database | integer
#========================================================================================
sub create {
    my $self = shift;
    my ( $hOptions ) = @_;

    $self->SUPER::create($hOptions);
    $self->setInitialShopData($hOptions);

    return;
}


#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopConfigService->update({
#                   Alias => $Alias,
#                   ShopType => $ShopType,
#                   IsClosed => $IsClosed,
#                   DomainName => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   WebServerScriptNamePart => $WebServerScriptNamePart,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   ImportFiles => $ImportFiles,
#                   MerchantEMail => $MerchantEMail,
#                   ShopAddress_FirstName => $FirstName,
#                   ShopAddress_LastName => $LastName,
#                   ShopAddress_CountryID => $CountryID,
#                   ShopAddress_Street => $Street,
#                   ShopAddress_Zipcode => $Zipcode,
#                   ShopAddress_City => $City,
#                   ShopAddress_State => $State,
#                   Name => $Name,
#               });
#----------------------------------------------------------------------------------------
# description  Updates a shop by changing the shop type, setting the status, address, name
#               or importing data from XML files.
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                   <li>Alias | unique shop name for the provider | string
#                   <li>ShopType | (optional) new shop type alias | string
#                   <li>IsClosed | (optional) true if closed by provider | boolean
#                   <li>MarkedForDelete | (optional) true if shop is marked for deletion | boolean
#                   <li>DomainName | (optional) domain name to access the shop | string
#                   <li>HasSSLCertificate | true if the web server has an SSL certificate
#                                    for the shop (optional) | string
#                   <li>WebServerScriptNamePart | script name part of the shop (optional) | string
#                   <li>IsTrialShop | true if the shop is in trial period (optional, default: 0) | boolean
#                   <li>IsInternalTestShop | true if the shop is used for test or support purposes (optional, default: 0) | boolean
#                   <li>ImportFiles | (optional) list of files names to import | ref.array.string
#                   <li>MerchantEMail | merchant email address (optional) | string</li>
#                   <li>ShopAddress_FirstName | merchant first name | string</li>
#                   <li>ShopAddress_LastName | merchant last name | string</li>
#                   <li>ShopAddress_CountryID | merchant enumeric country code according to ISO 3166-1 | integer</li>
#                   <li>ShopAddress_Street | merchant street and house number | string</li>
#                   <li>ShopAddress_Zipcode | merchant zipcode | string</li>
#                   <li>ShopAddress_City | merchant city | string</li>
#                   <li>ShopAddress_State | merchant state code according to ISO 3166-2 | string</li>
#                   <li>Name | name of the shop | string</li>
#               </ul> | reference to hash
#========================================================================================
sub update {
    my $self = shift;
    my ( $hOptions ) = @_;

    $self->setInitialShopData($hOptions);
    $self->SUPER::update($hOptions);

    return;
}

#========================================================================================
# function     shopInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hShopInfo = $ShopConfigService->shopInfo($ShopRef, $hOptions);
#----------------------------------------------------------------------------------------
# description  Returns the data for getTInfoShop() as hash reference.
#               Makes it easier to re-use the code from subclassed web services.
#----------------------------------------------------------------------------------------
# input        $ShopRef | ShopRef | object
# input        $hOptions | options for extended shop attributes,
#                           keys: Attributes, Languages | ref.hash
# input        $isGetRemote | get data from the remote shop, too | boolean
# return       $hShopInfo | shop data | ref.hash
#========================================================================================
sub shopInfo {
    my $self = shift;
    my ($ShopRef, $hOptions, $isGetRemote) = @_;

    my $hShopInfo = $self->SUPER::shopInfo( $ShopRef, $hOptions, $isGetRemote );
    my $hAdditionalInfo = $isGetRemote ? $self->getInitialShopData($ShopRef) : {};

    my %ReturnHash = (%$hShopInfo, %$hAdditionalInfo);
    return \%ReturnHash;
}

1;
