#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService8
# state        public
#----------------------------------------------------------------------------------------
# description  Provides web service access to create and delete shops
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::WebService::API::Client;
#               my $ShopConfigService = DE_EPAGES::WebService::API::Client
#                   ->uri('urn://epages.de/WebService/ShopConfigService/2015/07')
#                   ->proxy("http://localhost:80/epages/Site.soap");
#               $ShopConfigService->userinfo( "/Providers/$ProviderAlias/Users/$Login:$Password" );
#               $ShopConfigService->delete({
#                   Alias => 'Store.DemoShop'
#               });
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService8;
use base DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService7;

use strict;
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::WebService::Servlet::Soap qw( GetSoapServer );
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::Log qw( LogDebug );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService8->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ShopConfigService/2015/07',
        @Options,
    ));
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopConfigService->update({
#                   Alias => $Alias,
#                   ...,
#                   IsClosedTemporarily => 1,
#               });
#----------------------------------------------------------------------------------------
# description  Updates a shop.
#               Sets IsClosedTemporarily for the shop
#               See <function DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService::update>
#----------------------------------------------------------------------------------------
# input        $Alias | unique shop name for the provider | string
# input        $IsClosedTemporarily | shop is closed temporarily by provider | ref.array.boolean
#========================================================================================
sub update {
    my $self = shift;
    my ( $hOptions ) = @_;
    my $Provider = $self->provider;

    #copy $hOptions for logging and add provider
    my %LogOptions = %$hOptions;
    $LogOptions{Provider} = $Provider->alias;

    #remove password from logging
    my $hPWOptions;
    $hPWOptions->{MerchantPassword} = delete $LogOptions{MerchantPassword};
    $hPWOptions->{MerchantLogin}    =        $LogOptions{MerchantLogin};
    #log sensitive data only for special category
    LogDebug('update', $hPWOptions, 'ShopConfigPasswords' );

    #do general logging without sesitive data
    LogDebug('update', \%LogOptions );

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    Error( 'NoResults_ShopRef', {
        ShopRefAlias => $ShopRefAlias,
        ProviderAlias => $Provider->alias,
    } ) unless $Provider->existsShop( $ShopRefAlias );

    my $ShopRef = $Provider->shop( $ShopRefAlias );

    if( defined $hOptions->{'IsClosedTemporarily'} ) {
        $ShopRef->set({'IsClosedTemporarily' => $hOptions->{'IsClosedTemporarily'}});
    }

    return $self->SUPER::update( $hOptions );
}

#========================================================================================
# function     getInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShopInfo = $ShopConfigService->getInfo({
#                   Alias => $Alias
#               })->result;
#----------------------------------------------------------------------------------------
# description  Returns information about a shop from the site database.
#----------------------------------------------------------------------------------------
# input        $Alias | unique shop name for the provider | string
# return       $hShopInfo | shop data <ul>
#                         <li>Provider - provider alias - string</li>
#                         <li>Alias - unique shop name for the provider - string</li>
#                         <li>ShopType - shop type alias - string</li>
#                         <li>Database - store database identifier - string</li>
#                         <li>ObjectPath - object path of the shop in the store database - string</li>
#                         <li>IsClosed - true if closed by provider - boolean</li>
#                         <li>IsClosedTemporarily - true if closed by provider temporarily - boolean</li>
#                         <li>IsDeleted - true if deleted in store database - boolean</li>
#                         <li>DomainName - shop domain name - string</li>
#                         <li>HasSSLCertificate - true if the web server has an SSL certificate
#                                                 for the shop - string</li>
#                       </ul> | ref.hash.string
#========================================================================================
sub getInfo {
    my $self = shift;
    my ( $hOptions ) = @_;

    LogDebug('getInfo', \($self, $hOptions));

    my $Provider = $self->provider;

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    Error( 'NoResults_ShopRef', {
        ShopRefAlias => $ShopRefAlias,
        ProviderAlias => $Provider->alias,
    } ) unless $Provider->existsShop( $ShopRefAlias );

    my $ShopRef = $Provider->shop( $ShopRefAlias );

    # register type names of complex types
    GetSoapServer()->serializer->maptype->{'TInfoShop'} = $self->getServiceUrn;

    return $self->getTInfoShop( $ShopRef, $hOptions, 1 );
}


#========================================================================================
sub getTInfoShop {
    my $self = shift;
    my @Params = @_;
    return SOAP::Data->type( 'TInfoShop')->name('ShopInfo' => $self->shopInfo(@Params) );
}

#========================================================================================
# function     shopInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hShopInfo = $ShopConfigService->shopInfo($ShopRef, $hOptions);
#----------------------------------------------------------------------------------------
# description  Returns the data for getTInfoShop() as hash reference.
#               Makes it easier to re-use the code from subclassed web services.
#----------------------------------------------------------------------------------------
# input        $ShopRef | ShopRef | object
# input        $hOptions | options for extended sho attributes,
#                           keys: Attributes, Languages | ref.hash
# input        $isGetRemote | get data from the remote shop, too | boolean
# return       $hShopInfo | shop data | ref.hash
#========================================================================================
sub shopInfo {
    my $self = shift;
    my ($ShopRef, $hOptions, $isGetRemote) = @_;

    my $hShopInfo = $self->SUPER::shopInfo( $ShopRef, $hOptions, $isGetRemote);
    $hShopInfo->{'IsClosedTemporarily'}    = $self->soapBool( $ShopRef->get('IsClosedTemporarily') );

    return $hShopInfo;
}

#========================================================================================




1;
