#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::WebService::SimpleProvisioningService6
# state        public
#----------------------------------------------------------------------------------------
# description  Provides simpliefied web service access to create and update shops
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::WebService::API::Client;
#               my $SimpleProvisioningService = DE_EPAGES::WebService::API::Client
#                   ->uri('urn://epages.de/WebService/SimpleProvisioningService/2017/11')
#                   ->proxy("http://localhost:80/epages/Site.soap")
#                   ->userinfo( "/Providers/$ProviderAlias/Users/$Login:$Password" );
#                   ->create({
#                     Alias => 'DemoShop2',
#                     ShopType => 'ECom100',
#               });
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::WebService::SimpleProvisioningService6;
use base DE_EPAGES::ShopConfiguration::API::WebService::SimpleProvisioningService5;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::ShopConfiguration::API::WebService::SimpleProvisioningService6->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    return $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/SimpleProvisioningService/2017/11',
        @Options,
    ));
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SimpleProvisioningService->create({
#                   Alias => $Alias,
#                   ShopType => $ShopType,
#                   IsClosed => $IsClosed,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   DomainName  => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   MerchantLogin => $MerchantLogin,
#                   MerchantPassword => $MerchantPassword,
#                   MerchantEMail => $MerchantEMail,
#                   ShopAddress_FirstName => $FirstName,
#                   ShopAddress_LastName => $LastName,
#                   ShopAddress_CountryID => $CountryID,
#                   ShopAddress_Street => $Street,
#                   ShopAddress_Zipcode => $Zipcode,
#                   ShopAddress_City => $City,
#                   ShopAddress_State => $State,
#                   Name => $Name,
#               })->result;
# example      my $ShopID = $SimpleProvisioningService->create({
#                   Alias => 'DemoShop',
#                   ShopType => 'ECom100',
#                   MerchantLogin => 'smith',
#                   MerchantPassword => 'geheim',
#                   MerchantEMail => 'asmith@hotmail.com',
#                   MerchantEMail => 'hab@ich.net',
#                   ShopAddress_FirstName => 'Max',
#                   ShopAddress_LastName => 'Mustermann',
#                   ShopAddress_CountryID => '276',
#                   ShopAddress_Street => 'Heinrich-Heine-Strae 1',
#                   ShopAddress_Zipcode => '07749',
#                   ShopAddress_City => 'Jena',
#                   ShopAddress_State => 'TH',
#                   Name => 'Milestones',
#               })->result;
#----------------------------------------------------------------------------------------
# description  Creates a new shop on behalf of the current provider.
#               Sets login, email and password for the primary administrator as well as
#               the shop address and the shop name.
#----------------------------------------------------------------------------------------
# input        $hOptions | shop data <ul>
#                         <li>Alias | unique shop identifier | string</li>
#                         <li>ShopType | shop type alias | string</li>
#                         <li>IsClosed | true if the new shop is closed by provider (optional, default: 0) | boolean</li>
#                         <li>IsTrialShop | true if the shop is in trial period (optional, default: 0) | boolean</li>
#                         <li>IsInternalTestShop | true if the shop is used for test or support purposes (optional, default: 0) | boolean</li>
#                         <li>DomainName | domain name to access the shop (optional) | string</li>
#                         <li>HasSSLCertificate | true if the web server has an SSL certificate for the shop (optional) | string</li>
#                         <li>MerchantLogin | merchant login (optional) | string</li>
#                         <li>MerchantPassword | merchant password (optional) | string</li>
#                         <li>MerchantEMail | merchant email address (optional) | string</li>
#                         <li>ShopAddress_FirstName | merchant first name | string</li>
#                         <li>ShopAddress_LastName | merchant last name | string</li>
#                         <li>ShopAddress_CountryID | merchant enumeric country code according to ISO 3166-1 | integer</li>
#                         <li>ShopAddress_Street | merchant street and house number | string</li>
#                         <li>ShopAddress_Zipcode | merchant zipcode | string</li>
#                         <li>ShopAddress_City | merchant city | string</li>
#                         <li>ShopAddress_State | merchant state code according to ISO 3166-2 | string</li>
#                         <li>Name | name of the shop | string</li>
#                       </ul> | ref.hash.string
#========================================================================================
sub create {
    my $self = shift;
    my ( $hOptions ) = @_;

    $self->SUPER::create($hOptions);
    $self->setInitialShopData($hOptions);

    return;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SimpleProvisioningService->update({
#                   Alias => $Alias,
#                   ShopType => $ShopType,
#                   IsClosed => $IsClosed,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   DomainName => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   MerchantLogin => $MerchantLogin,
#                   MerchantPassword => $MerchantPassword,
#                   MerchantEMail => $MerchantEMail,
#                   ShopAddress_FirstName => $FirstName,
#                   ShopAddress_LastName => $LastName,
#                   ShopAddress_CountryID => $CountryID,
#                   ShopAddress_Street => $Street,
#                   ShopAddress_Zipcode => $Zipcode,
#                   ShopAddress_City => $City,
#                   ShopAddress_State => $State,
#                   Name => $Name,
#               });
#----------------------------------------------------------------------------------------
# description  Updates a shop.
#               Sets login, email and password for the primary administrator as well as
#               the shop address and the shop name.
#               See <function DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService::update>
#----------------------------------------------------------------------------------------
# input        $hOptions | shop data <ul>
#                         <li>Alias | unique shop identifier | string</li>
#                         <li>IsClosed | true if the new shop is closed by provider (optional, default: 0) | boolean</li>
#                         <li>IsTrialShop | true if the shop is in trial period (optional, default: 0) | boolean</li>
#                         <li>IsInternalTestShop | true if the shop is used for test or support purposes (optional, default: 0) | boolean</li>
#                         <li>DomainName | domain name to access the shop (optional) | string</li>
#                         <li>HasSSLCertificate | true if the web server has an SSL certificate for the shop (optional) | string</li>
#                         <li>MerchantLogin | merchant login (optional) | string</li>
#                         <li>MerchantPassword | merchant password (optional) | string</li>
#                         <li>MerchantEMail | merchant email address (optional) | string</li>
#                         <li>ShopAddress_FirstName | merchant first name | string</li>
#                         <li>ShopAddress_LastName | merchant last name | string</li>
#                         <li>ShopAddress_CountryID | merchant enumeric country code according to ISO 3166-1 | integer</li>
#                         <li>ShopAddress_Street | merchant street and house number | string</li>
#                         <li>ShopAddress_Zipcode | merchant zipcode | string</li>
#                         <li>ShopAddress_City | merchant city | string</li>
#                         <li>ShopAddress_State | merchant state code according to ISO 3166-2 | string</li>
#                         <li>Name | name of the shop | string</li>
#                       </ul> | ref.hash.string
#========================================================================================
sub update {
    my $self = shift;
    my ( $hOptions ) = @_;

    $self->SUPER::update($hOptions);
    $self->setInitialShopData($hOptions);

    return;
}

#========================================================================================
# function     shopInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hShopInfo = $ShopConfigService->shopInfo($ShopRef, $hOptions);
#----------------------------------------------------------------------------------------
# description  Returns the data for getTInfoShop() as hash reference.
#               Makes it easier to re-use the code from subclassed web services.
#----------------------------------------------------------------------------------------
# input        $ShopRef | ShopRef | object
# input        $hOptions | options for extended shop attributes,
#                           keys: Attributes, Languages | ref.hash
# input        $isGetRemote | get data from the remote shop, too | boolean
# return       $hShopInfo | shop data | ref.hash
#========================================================================================
sub shopInfo {
    my $self = shift;
    my ($ShopRef, $hOptions, $isGetRemote) = @_;

    my $hShopInfo = $self->SUPER::shopInfo( $ShopRef, $hOptions, $isGetRemote );
    my $hAdditionalInfo = $isGetRemote ? $self->getInitialShopData($ShopRef) : {};

    return { %$hShopInfo, %$hAdditionalInfo };
}

1;
